@@ -5,7 +5,7 @@ set -e
 
 # Make sure we're run as root
 scriptname=$(basename "$0")
-if [ "$UID" -ne 0 ]; then
+if [ "$(id -u)" -ne 0 ]; then
      echo "${scriptname} must be run as root"
      exit 1
 fi
