@@ -1,17 +1,16 @@
 //! The classes responsible for autoloading functions and completions.
 
+use crate::FLOGF;
 #[cfg(feature = "embed-data")]
-use crate::common::wcs2string;
-use crate::common::{escape, ScopeGuard};
+use crate::common::wcs2bytes;
+use crate::common::{ScopeGuard, escape};
 use crate::env::Environment;
 use crate::io::IoChain;
 use crate::parser::Parser;
-#[cfg(test)]
-use crate::tests::prelude::*;
-use crate::wchar::{wstr, WString, L};
+use crate::wchar::{L, WString, wstr};
 use crate::wchar_ext::WExt;
-use crate::wutil::{file_id_for_path, FileId, INVALID_FILE_ID};
-use crate::FLOGF;
+use crate::wutil::{FileId, INVALID_FILE_ID, file_id_for_path};
+use cfg_if::cfg_if;
 use lru::LruCache;
 #[cfg(feature = "embed-data")]
 use rust_embed::RustEmbed;
@@ -38,25 +37,26 @@ pub struct Autoload {
     current_autoloading: HashSet<WString>,
 
     /// The autoload cache.
-    /// This is a unique_ptr because want to change it if the value of our environment variable
-    /// changes. This is never null (but it may be a cache with no paths).
-    cache: Box<AutoloadFileCache>,
+    cache: AutoloadFileCache,
 }
 
 #[cfg(feature = "embed-data")]
 #[derive(RustEmbed)]
-#[folder = "share/"]
+#[folder = "share"]
+#[exclude = "__fish_build_paths.fish.in"]
 pub struct Asset;
 
-#[cfg(feature = "embed-data")]
-pub fn has_asset(cmd: &str) -> bool {
-    Asset::get(cmd).is_some()
-}
-
-#[cfg(not(feature = "embed-data"))]
-pub fn has_asset(_cmd: &str) -> bool {
-    false
-}
+cfg_if!(
+    if #[cfg(feature = "embed-data")] {
+        pub fn has_asset(cmd: &str) -> bool {
+            Asset::get(cmd).is_some()
+        }
+    } else {
+        pub fn has_asset(_cmd: &str) -> bool {
+            false
+        }
+    }
+);
 
 #[derive(Clone, Copy, Eq, PartialEq)]
 enum AssetDir {
@@ -116,12 +116,12 @@ impl Autoload {
                 match &path {
                     #[cfg(feature = "embed-data")]
                     AutoloadPath::Embedded(_) => {
-                        FLOGF!(autoload, "Embedded: %ls", cmd);
+                        FLOGF!(autoload, "Embedded: %s", cmd);
                     }
                     AutoloadPath::Path(path) => {
                         FLOGF!(
                             autoload,
-                            "Loading %ls from var %ls from path %ls",
+                            "Loading %s from var %s from path %s",
                             cmd,
                             self.env_var_name,
                             path
@@ -150,16 +150,16 @@ impl Autoload {
             }
             #[cfg(feature = "embed-data")]
             AutoloadPath::Embedded(name) => {
-                use crate::common::str2wcstring;
+                use crate::common::bytes2wcstring;
                 use std::sync::Arc;
-                FLOGF!(autoload, "Loading embedded: %ls", name);
+                FLOGF!(autoload, "Loading embedded: %s", name);
                 let emfile = Asset::get(name).expect("Embedded file not found");
-                let src = str2wcstring(&emfile.data);
+                let src = bytes2wcstring(&emfile.data);
                 let mut widename = L!("embedded:").to_owned();
                 widename.push_str(name);
                 let ret = parser.eval_file_wstr(src, Arc::new(widename), &IoChain::new(), None);
                 if let Err(msg) = ret {
-                    eprintf!("%ls", msg);
+                    eprintf!("%s", msg);
                 }
             }
         }
@@ -209,10 +209,9 @@ impl Autoload {
     }
 
     /// Invalidate any underlying cache.
-    /// This is exposed for testing.
     #[cfg(test)]
     fn invalidate_cache(&mut self) {
-        self.cache = Box::new(AutoloadFileCache::with_dirs(self.cache.dirs().to_owned()));
+        self.cache = AutoloadFileCache::with_dirs(self.cache.dirs().to_owned());
     }
 
     /// Like resolve_autoload(), but accepts the paths directly.
@@ -227,7 +226,7 @@ impl Autoload {
         // Note we don't have to modify autoloadable_files_. We'll naturally detect if those have
         // changed when we query the cache.
         if paths != self.cache.dirs() {
-            self.cache = Box::new(AutoloadFileCache::with_dirs(paths.to_owned()));
+            self.cache = AutoloadFileCache::with_dirs(paths.to_owned());
         }
 
         // Do we have an entry to load?
@@ -352,10 +351,13 @@ impl AutoloadFileCache {
 
         // Check hits.
         if let Some(value) = self.known_files.get(cmd) {
-            #[cfg(feature = "embed-data")]
-            let embedded = matches!(value.file, AutoloadableFileInfo::EmbeddedPath(_));
-            #[cfg(not(feature = "embed-data"))]
-            let embedded = false;
+            cfg_if!(
+                if #[cfg(feature = "embed-data")] {
+                    let embedded = matches!(value.file, AutoloadableFileInfo::EmbeddedPath(_));
+                } else {
+                    let embedded = false;
+                }
+            );
             if allow_stale
                 || embedded
                 || Self::is_fresh(value.last_checked, Self::current_timestamp())
@@ -474,7 +476,7 @@ impl AutoloadFileCache {
         if cfg!(test) {
             return None;
         }
-        let narrow = wcs2string(cmd);
+        let narrow = wcs2bytes(cmd);
         let cmdstr = std::str::from_utf8(&narrow).ok()?;
         let p = match asset_dir {
             AssetDir::Functions => "functions/".to_owned() + cmdstr + ".fish",
@@ -484,113 +486,122 @@ impl AutoloadFileCache {
     }
 }
 
-#[test]
-#[serial]
-fn test_autoload() {
-    let _cleanup = test_init();
-    use crate::common::{charptr2wcstring, wcs2zstring};
-    use crate::fds::wopen_cloexec;
-    use crate::wutil::sprintf;
-    use nix::fcntl::OFlag;
+#[cfg(test)]
+mod tests {
+    use super::{Autoload, AutoloadResult};
+    use crate::tests::prelude::*;
+    use crate::wchar::prelude::*;
 
-    macro_rules! run {
-        ( $fmt:expr $(, $arg:expr )* $(,)? ) => {
-             let cmd = wcs2zstring(&sprintf!($fmt $(, $arg)*));
-             let status = unsafe { libc::system(cmd.as_ptr()) };
-             assert!(status == 0);
-        };
+    #[test]
+    #[serial]
+    fn test_autoload() {
+        let _cleanup = test_init();
+        use crate::common::wcs2zstring;
+        use crate::fds::wopen_cloexec;
+        use nix::fcntl::OFlag;
+
+        macro_rules! run {
+            ( $fmt:expr $(, $arg:expr )* $(,)? ) => {
+                 let cmd = wcs2zstring(&sprintf!($fmt $(, $arg)*));
+                 let status = unsafe { libc::system(cmd.as_ptr()) };
+                 assert!(status == 0);
+            };
+        }
+
+        fn touch_file(path: &wstr) {
+            use nix::sys::stat::Mode;
+            use std::io::Write;
+
+            let mut file = wopen_cloexec(
+                path,
+                OFlag::O_RDWR | OFlag::O_CREAT,
+                Mode::from_bits_truncate(0o666),
+            )
+            .unwrap();
+            file.write_all(b"Hello").unwrap();
+        }
+
+        let p1 = fish_tempfile::new_dir().unwrap();
+        let p1 = WString::from(p1.path().to_str().unwrap());
+        let p2 = fish_tempfile::new_dir().unwrap();
+        let p2 = WString::from(p2.path().to_str().unwrap());
+
+        let paths = &[p1.clone(), p2.clone()];
+        let mut autoload = Autoload::new(L!("test_var"));
+        assert!(autoload.resolve_command_impl(L!("file1"), paths).is_none());
+        assert!(
+            autoload
+                .resolve_command_impl(L!("nothing"), paths)
+                .is_none()
+        );
+        assert!(autoload.get_autoloaded_commands().is_empty());
+
+        run!("touch %s/file1.fish", p1);
+        run!("touch %s/file2.fish", p2);
+        autoload.invalidate_cache();
+
+        assert!(!autoload.autoload_in_progress(L!("file1")));
+        assert!(matches!(
+            autoload.resolve_command_impl(L!("file1"), paths),
+            AutoloadResult::Path(_)
+        ));
+        assert!(matches!(
+            autoload.resolve_command_impl(L!("file1"), paths),
+            AutoloadResult::Pending
+        ));
+        assert!(autoload.autoload_in_progress(L!("file1")));
+        assert!(autoload.get_autoloaded_commands() == vec![L!("file1")]);
+        autoload.mark_autoload_finished(L!("file1"));
+        assert!(!autoload.autoload_in_progress(L!("file1")));
+        assert!(autoload.get_autoloaded_commands() == vec![L!("file1")]);
+
+        assert!(matches!(
+            autoload.resolve_command_impl(L!("file1"), paths),
+            AutoloadResult::Loaded
+        ));
+        assert!(
+            autoload
+                .resolve_command_impl(L!("nothing"), paths)
+                .is_none()
+        );
+        assert!(autoload.resolve_command_impl(L!("file2"), paths).is_some());
+        assert!(matches!(
+            autoload.resolve_command_impl(L!("file2"), paths),
+            AutoloadResult::Pending
+        ));
+        autoload.mark_autoload_finished(L!("file2"));
+        assert!(matches!(
+            autoload.resolve_command_impl(L!("file2"), paths),
+            AutoloadResult::Loaded
+        ));
+        assert!((autoload.get_autoloaded_commands() == vec![L!("file1"), L!("file2")]));
+
+        autoload.clear();
+        assert!(autoload.resolve_command_impl(L!("file1"), paths).is_some());
+        autoload.mark_autoload_finished(L!("file1"));
+        assert!(matches!(
+            autoload.resolve_command_impl(L!("file1"), paths),
+            AutoloadResult::Loaded
+        ));
+        assert!(
+            autoload
+                .resolve_command_impl(L!("nothing"), paths)
+                .is_none()
+        );
+        assert!(autoload.resolve_command_impl(L!("file2"), paths).is_some());
+        assert!(matches!(
+            autoload.resolve_command_impl(L!("file2"), paths),
+            AutoloadResult::Pending
+        ));
+        autoload.mark_autoload_finished(L!("file2"));
+
+        assert!(matches!(
+            autoload.resolve_command_impl(L!("file1"), paths),
+            AutoloadResult::Loaded
+        ));
+        touch_file(&sprintf!("%s/file1.fish", p1));
+        autoload.invalidate_cache();
+        assert!(autoload.resolve_command_impl(L!("file1"), paths).is_some());
+        autoload.mark_autoload_finished(L!("file1"));
     }
-
-    fn touch_file(path: &wstr) {
-        use nix::sys::stat::Mode;
-        use std::io::Write;
-
-        let mut file = wopen_cloexec(
-            path,
-            OFlag::O_RDWR | OFlag::O_CREAT,
-            Mode::from_bits_truncate(0o666),
-        )
-        .unwrap();
-        file.write_all(b"Hello").unwrap();
-    }
-
-    let mut t1 = "/tmp/fish_test_autoload.XXXXXX\0".as_bytes().to_vec();
-    let p1 = charptr2wcstring(unsafe { libc::mkdtemp(t1.as_mut_ptr().cast()) });
-    let mut t2 = "/tmp/fish_test_autoload.XXXXXX\0".as_bytes().to_vec();
-    let p2 = charptr2wcstring(unsafe { libc::mkdtemp(t2.as_mut_ptr().cast()) });
-
-    let paths = &[p1.clone(), p2.clone()];
-    let mut autoload = Autoload::new(L!("test_var"));
-    assert!(autoload.resolve_command_impl(L!("file1"), paths).is_none());
-    assert!(autoload
-        .resolve_command_impl(L!("nothing"), paths)
-        .is_none());
-    assert!(autoload.get_autoloaded_commands().is_empty());
-
-    run!("touch %ls/file1.fish", p1);
-    run!("touch %ls/file2.fish", p2);
-    autoload.invalidate_cache();
-
-    assert!(!autoload.autoload_in_progress(L!("file1")));
-    assert!(matches!(
-        autoload.resolve_command_impl(L!("file1"), paths),
-        AutoloadResult::Path(_)
-    ));
-    assert!(matches!(
-        autoload.resolve_command_impl(L!("file1"), paths),
-        AutoloadResult::Pending
-    ));
-    assert!(autoload.autoload_in_progress(L!("file1")));
-    assert!(autoload.get_autoloaded_commands() == vec![L!("file1")]);
-    autoload.mark_autoload_finished(L!("file1"));
-    assert!(!autoload.autoload_in_progress(L!("file1")));
-    assert!(autoload.get_autoloaded_commands() == vec![L!("file1")]);
-
-    assert!(matches!(
-        autoload.resolve_command_impl(L!("file1"), paths),
-        AutoloadResult::Loaded
-    ));
-    assert!(autoload
-        .resolve_command_impl(L!("nothing"), paths)
-        .is_none());
-    assert!(autoload.resolve_command_impl(L!("file2"), paths).is_some());
-    assert!(matches!(
-        autoload.resolve_command_impl(L!("file2"), paths),
-        AutoloadResult::Pending
-    ));
-    autoload.mark_autoload_finished(L!("file2"));
-    assert!(matches!(
-        autoload.resolve_command_impl(L!("file2"), paths),
-        AutoloadResult::Loaded
-    ));
-    assert!((autoload.get_autoloaded_commands() == vec![L!("file1"), L!("file2")]));
-
-    autoload.clear();
-    assert!(autoload.resolve_command_impl(L!("file1"), paths).is_some());
-    autoload.mark_autoload_finished(L!("file1"));
-    assert!(matches!(
-        autoload.resolve_command_impl(L!("file1"), paths),
-        AutoloadResult::Loaded
-    ));
-    assert!(autoload
-        .resolve_command_impl(L!("nothing"), paths)
-        .is_none());
-    assert!(autoload.resolve_command_impl(L!("file2"), paths).is_some());
-    assert!(matches!(
-        autoload.resolve_command_impl(L!("file2"), paths),
-        AutoloadResult::Pending
-    ));
-    autoload.mark_autoload_finished(L!("file2"));
-
-    assert!(matches!(
-        autoload.resolve_command_impl(L!("file1"), paths),
-        AutoloadResult::Loaded
-    ));
-    touch_file(&sprintf!("%ls/file1.fish", p1));
-    autoload.invalidate_cache();
-    assert!(autoload.resolve_command_impl(L!("file1"), paths).is_some());
-    autoload.mark_autoload_finished(L!("file1"));
-
-    run!(L!("rm -Rf %ls"), p1);
-    run!(L!("rm -Rf %ls"), p2);
 }
