@@ -1,5 +1,7 @@
 // Implementation of the bg builtin.
 
+use std::collections::HashSet;
+
 use crate::proc::Pid;
 
 use super::prelude::*;
@@ -16,7 +18,7 @@ fn send_to_bg(
         if !jobs[job_pos].wants_job_control() {
             let job = &jobs[job_pos];
             streams.err.append(wgettext_fmt!(
-                "%ls: Can't put job %s, '%ls' to background because it is not under job control\n",
+                "%s: Can't put job %s, '%s' to background because it is not under job control\n",
                 cmd,
                 job.job_id().to_wstring(),
                 job.command()
@@ -26,7 +28,7 @@ fn send_to_bg(
 
         let job = &jobs[job_pos];
         streams.err.append(wgettext_fmt!(
-            "Send job %s '%ls' to background\n",
+            "Send job %s '%s' to background\n",
             job.job_id().to_wstring(),
             job.command()
         ));
@@ -46,7 +48,7 @@ fn send_to_bg(
 pub fn bg(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> BuiltinResult {
     let opts = HelpOnlyCmdOpts::parse(args, parser, streams)?;
 
-    let Some(&cmd) = args.get(0) else {
+    let Some(&cmd) = args.first() else {
         return Err(STATUS_INVALID_ARGS);
     };
 
@@ -66,7 +68,7 @@ pub fn bg(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Built
         let Some(job_pos) = job_pos else {
             streams
                 .err
-                .append(wgettext_fmt!("%ls: There are no suitable jobs\n", cmd));
+                .append(wgettext_fmt!("%s: There are no suitable jobs\n", cmd));
             return Err(STATUS_CMD_ERROR);
         };
 
@@ -79,14 +81,9 @@ pub fn bg(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Built
     let mut retval: BuiltinResult = Ok(SUCCESS);
     let pids: Vec<Pid> = args[opts.optind..]
         .iter()
-        .filter_map(|arg| match fish_wcstoi(arg).map(Pid::new) {
-            Ok(Some(pid)) => Some(pid),
+        .filter_map(|arg| match parse_pid(streams, cmd, arg) {
+            Ok(pid) => Some(pid),
             _ => {
-                streams.err.append(wgettext_fmt!(
-                    "%ls: '%ls' is not a valid job specifier\n",
-                    cmd,
-                    arg
-                ));
                 retval = Err(STATUS_INVALID_ARGS);
                 None
             }
@@ -97,13 +94,16 @@ pub fn bg(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Built
 
     // Background all existing jobs that match the pids.
     // Non-existent jobs aren't an error, but information about them is useful.
+    let mut seen = HashSet::new();
     for pid in pids {
-        if let Some((job_pos, _job)) = parser.job_get_with_index_from_pid(pid) {
-            send_to_bg(parser, streams, cmd, job_pos)?;
+        if let Some((job_pos, job)) = parser.job_get_with_index_from_pid(pid) {
+            if seen.insert(&*job as *const _) {
+                send_to_bg(parser, streams, cmd, job_pos)?;
+            }
         } else {
             streams
                 .err
-                .append(wgettext_fmt!("%ls: Could not find job '%d'\n", cmd, pid));
+                .append(wgettext_fmt!("%s: Could not find job '%d'\n", cmd, pid));
         }
     }
 
