@@ -3,19 +3,14 @@ use std::sync::atomic::Ordering;
 // Implementation of the block builtin.
 use super::prelude::*;
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+#[derive(Clone, Copy, Debug, Default, PartialEq, Eq)]
 enum Scope {
+    #[default]
     Unset,
     Global,
     Local,
 }
 
-impl Default for Scope {
-    fn default() -> Self {
-        Self::Unset
-    }
-}
-
 #[derive(Debug, Clone, Copy, Default)]
 struct Options {
     scope: Scope,
@@ -90,7 +85,7 @@ pub fn block(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Bu
     if opts.erase {
         if opts.scope != Scope::Unset {
             streams.err.append(wgettext_fmt!(
-                "%ls: Can not specify scope when removing block\n",
+                "%s: Can not specify scope when removing block\n",
                 cmd
             ));
             return Err(STATUS_INVALID_ARGS);
@@ -99,7 +94,7 @@ pub fn block(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Bu
         if parser.global_event_blocks.load(Ordering::Relaxed) == 0 {
             streams
                 .err
-                .append(wgettext_fmt!("%ls: No blocks defined\n", cmd));
+                .append(wgettext_fmt!("%s: No blocks defined\n", cmd));
             return Err(STATUS_CMD_ERROR);
         }
         parser.global_event_blocks.fetch_sub(1, Ordering::Relaxed);
