@@ -1,12 +1,12 @@
 use super::prelude::*;
 use super::read::TokenOutputMode;
 use crate::ast::{self, Kind, Leaf};
-use crate::common::{unescape_string, UnescapeFlags, UnescapeStringStyle};
+use crate::common::{UnescapeFlags, UnescapeStringStyle, unescape_string};
 use crate::complete::Completion;
-use crate::expand::{expand_string, ExpandFlags, ExpandResultCode};
+use crate::expand::{ExpandFlags, ExpandResultCode, expand_string};
 use crate::input::input_function_get_code;
 use crate::input_common::{CharEvent, ReadlineCmd};
-use crate::operation_context::{no_cancel, OperationContext};
+use crate::operation_context::{OperationContext, no_cancel};
 use crate::parse_constants::{ParseTreeFlags, ParserTestErrorBits};
 use crate::parse_util::{
     parse_util_detect_errors, parse_util_get_offset_from_line, parse_util_job_extent,
@@ -141,6 +141,7 @@ fn strip_dollar_prefixes(insert_mode: AppendMode, prefix: &wstr, insert: &wstr)
 /// and skipping non-string tokens
 /// \param buffer the original command line buffer
 /// \param cursor_pos the position of the cursor in the command line
+#[allow(clippy::too_many_arguments)]
 fn write_part(
     parser: &Parser,
     range: Range<usize>,
@@ -393,16 +394,16 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
             let Some(cmd) = input_function_get_code(arg) else {
                 streams
                     .err
-                    .append(wgettext_fmt!("%ls: Unknown input function '%ls'", cmd, arg));
+                    .append(wgettext_fmt!("%s: Unknown input function '%s'", cmd, arg));
                 builtin_print_error_trailer(parser, streams.err, cmd);
                 return Err(STATUS_INVALID_ARGS);
             };
             // Don't enqueue a repaint if we're currently in the middle of one,
             // because that's an infinite loop.
-            if matches!(cmd, RL::RepaintMode | RL::ForceRepaint | RL::Repaint) {
-                if parser.libdata().is_repaint {
-                    continue;
-                }
+            if matches!(cmd, RL::RepaintMode | RL::ForceRepaint | RL::Repaint)
+                && parser.libdata().is_repaint
+            {
+                continue;
             }
 
             // Inserts the readline function at the back of the queue.
@@ -517,7 +518,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
             let Ok(new_coord) = usize::try_from(new_coord) else {
                 streams
                     .err
-                    .append(wgettext_fmt!("%ls: line/column index starts at 1", cmd));
+                    .append(wgettext_fmt!("%s: line/column index starts at 1", cmd));
                 builtin_print_error_trailer(parser, streams.err, cmd);
                 return Err(STATUS_INVALID_ARGS);
             };
@@ -529,7 +530,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
                 ) else {
                     streams
                         .err
-                        .append(wgettext_fmt!("%ls: there is no line %ls\n", cmd, arg));
+                        .append(wgettext_fmt!("%s: there is no line %s\n", cmd, arg));
                     builtin_print_error_trailer(parser, streams.err, cmd);
                     return Err(STATUS_INVALID_ARGS);
                 };
@@ -544,7 +545,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
                         .unwrap_or(rstate.text.len());
                 if line_offset + new_coord > next_line_offset {
                     streams.err.append(wgettext_fmt!(
-                        "%ls: column %ls exceeds line length\n",
+                        "%s: column %s exceeds line length\n",
                         cmd,
                         arg
                     ));
@@ -601,7 +602,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
         let Some(selection) = rstate.selection else {
             return Err(STATUS_CMD_ERROR);
         };
-        streams.out.append(sprintf!("%lu\n", selection.start));
+        streams.out.append(sprintf!("%u\n", selection.start));
         return Ok(SUCCESS);
     }
 
@@ -609,7 +610,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
         let Some(selection) = rstate.selection else {
             return Err(STATUS_CMD_ERROR);
         };
-        streams.out.append(sprintf!("%lu\n", selection.end));
+        streams.out.append(sprintf!("%u\n", selection.end));
         return Ok(SUCCESS);
     }
 
@@ -716,7 +717,7 @@ pub fn commandline(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
         } else {
             streams
                 .out
-                .append(sprintf!("%lu\n", current_cursor_pos - range.start));
+                .append(sprintf!("%u\n", current_cursor_pos - range.start));
         }
         return Ok(SUCCESS);
     }
