@@ -1,6 +1,6 @@
 use super::prelude::*;
-use crate::common::{unescape_string, ScopeGuard, UnescapeFlags, UnescapeStringStyle};
-use crate::complete::{complete_add_wrapper, complete_remove_wrapper, CompletionRequestOptions};
+use crate::common::{ScopeGuard, UnescapeFlags, UnescapeStringStyle, unescape_string};
+use crate::complete::{CompletionRequestOptions, complete_add_wrapper, complete_remove_wrapper};
 use crate::highlight::colorize;
 use crate::highlight::highlight_shell;
 use crate::nix::isatty;
@@ -12,10 +12,10 @@ use crate::proc::is_interactive_session;
 use crate::reader::{commandline_get_state, completion_apply_to_command_line};
 use crate::wcstringutil::string_suffixes_string;
 use crate::{
-    common::str2wcstring,
+    common::bytes2wcstring,
     complete::{
-        complete_add, complete_print, complete_remove, complete_remove_all, CompleteFlags,
-        CompleteOptionType, CompletionMode,
+        CompleteFlags, CompleteOptionType, CompletionMode, complete_add, complete_print,
+        complete_remove, complete_remove_all,
     },
 };
 use libc::STDOUT_FILENO;
@@ -26,6 +26,7 @@ use libc::STDOUT_FILENO;
 // complete_add function only accepts one short switch and one long switch.
 
 /// Silly function.
+#[allow(clippy::too_many_arguments)]
 fn builtin_complete_add2(
     cmd: &wstr,
     cmd_is_path: bool,
@@ -96,6 +97,7 @@ fn builtin_complete_add2(
 }
 
 /// Sily function.
+#[allow(clippy::too_many_arguments)]
 fn builtin_complete_add(
     cmds: &[WString],
     paths: &[WString],
@@ -207,7 +209,7 @@ fn builtin_complete_print(cmd: &wstr, streams: &mut IoStreams, parser: &Parser)
         highlight_shell(&repr, &mut colors, &parser.context(), false, None);
         streams
             .out
-            .append(str2wcstring(&colorize(&repr, &colors, parser.vars())));
+            .append(bytes2wcstring(&colorize(&repr, &colors, parser.vars())));
     } else {
         streams.out.append(repr);
     }
@@ -296,7 +298,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
                     }
                 } else {
                     streams.err.append(wgettext_fmt!(
-                        "%ls: Invalid token '%ls'\n",
+                        "%s: Invalid token '%s'\n",
                         cmd,
                         w.woptarg.unwrap()
                     ));
@@ -318,7 +320,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
                 if arg.is_empty() {
                     streams
                         .err
-                        .append(wgettext_fmt!("%ls: -s requires a non-empty string\n", cmd,));
+                        .append(wgettext_fmt!("%s: -s requires a non-empty string\n", cmd,));
                     return Err(STATUS_INVALID_ARGS);
                 }
             }
@@ -328,7 +330,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
                 if arg.is_empty() {
                     streams
                         .err
-                        .append(wgettext_fmt!("%ls: -l requires a non-empty string\n", cmd,));
+                        .append(wgettext_fmt!("%s: -l requires a non-empty string\n", cmd,));
                     return Err(STATUS_INVALID_ARGS);
                 }
             }
@@ -338,7 +340,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
                 if arg.is_empty() {
                     streams
                         .err
-                        .append(wgettext_fmt!("%ls: -o requires a non-empty string\n", cmd,));
+                        .append(wgettext_fmt!("%s: -o requires a non-empty string\n", cmd,));
                     return Err(STATUS_INVALID_ARGS);
                 }
             }
@@ -442,7 +444,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
 
         if let Err(err_text) = parse_util_detect_errors_in_argument_list(&comp, &prefix) {
             streams.err.append(wgettext_fmt!(
-                "%ls: %ls: contains a syntax error\n",
+                "%s: %s: contains a syntax error\n",
                 cmd,
                 comp
             ));
