@@ -2,7 +2,7 @@ use super::prelude::*;
 use crate::event;
 
 pub fn emit(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> BuiltinResult {
-    let Some(&cmd) = argv.get(0) else {
+    let Some(&cmd) = argv.first() else {
         return Err(STATUS_INVALID_ARGS);
     };
 
@@ -16,7 +16,7 @@ pub fn emit(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
     let Some(event_name) = argv.get(opts.optind) else {
         streams
             .err
-            .append(sprintf!(L!("%ls: expected event name\n"), cmd));
+            .append(sprintf!(L!("%s: expected event name\n"), cmd));
         return Err(STATUS_INVALID_ARGS);
     };
 
