@@ -1,43 +1,36 @@
 //! The fish_indent program.
 
-#![allow(unknown_lints)]
-// Delete this once we require Rust 1.74.
-#![allow(unstable_name_collisions)]
-#![allow(clippy::incompatible_msrv)]
-#![allow(clippy::uninlined_format_args)]
-
-use std::ffi::{CString, OsStr};
+use std::ffi::OsStr;
 use std::fs;
 use std::io::{Read, Write};
 use std::os::unix::ffi::OsStrExt;
 
+use crate::locale::set_libc_locales;
 use crate::panic::panic_handler;
-use libc::LC_ALL;
 
 use super::prelude::*;
 use crate::ast::{self, Ast, Kind, Leaf, Node, NodeVisitor, SourceRangeList, Traversal};
 use crate::common::{
-    str2wcstring, unescape_string, wcs2string, UnescapeFlags, UnescapeStringStyle, PROGRAM_NAME,
+    PROGRAM_NAME, UnescapeFlags, UnescapeStringStyle, bytes2wcstring, get_program_name,
+    unescape_string, wcs2bytes,
 };
+use crate::env::EnvStack;
 use crate::env::env_init;
 use crate::env::environment::Environment;
-use crate::env::EnvStack;
 use crate::expand::INTERNAL_SEPARATOR;
-#[allow(unused_imports)]
-use crate::future::{IsSomeAnd, IsSorted};
 use crate::future_feature_flags;
 use crate::global_safety::RelaxedAtomicBool;
-use crate::highlight::{colorize, highlight_shell, HighlightRole, HighlightSpec};
+use crate::highlight::{HighlightRole, HighlightSpec, colorize, highlight_shell};
 use crate::operation_context::OperationContext;
 use crate::parse_constants::{ParseTokenType, ParseTreeFlags, SourceRange};
-use crate::parse_util::{apply_indents, parse_util_compute_indents, SPACES_PER_INDENT};
+use crate::parse_util::{SPACES_PER_INDENT, apply_indents, parse_util_compute_indents};
 use crate::print_help::print_help;
 use crate::threads;
-use crate::tokenizer::{TokenType, Tokenizer, TOK_SHOW_BLANK_LINES, TOK_SHOW_COMMENTS};
+use crate::tokenizer::{TOK_SHOW_BLANK_LINES, TOK_SHOW_COMMENTS, TokenType, Tokenizer};
 use crate::topic_monitor::topic_monitor_init;
 use crate::wchar::prelude::*;
 use crate::wcstringutil::count_preceding_backslashes;
-use crate::wgetopt::{wopt, ArgType, WGetopter, WOption};
+use crate::wgetopt::{ArgType, WGetopter, WOption, wopt};
 use crate::wutil::fish_iswalnum;
 
 /// Note: this got somewhat more complicated after introducing the new AST, because that AST no
@@ -213,7 +206,7 @@ impl<'source, 'ast> PrettyPrinter<'source, 'ast> {
     // Return the gap ranges from our ast.
     fn compute_gaps(&self) -> Vec<SourceRange> {
         let range_compare = |r1: SourceRange, r2: SourceRange| {
-            (r1.start(), r1.length()).cmp(&(r2.start(), r2.length()))
+            (r1.start(), r1.length()) <= (r2.start(), r2.length())
         };
         // Collect the token ranges into a list.
         let mut tok_ranges = vec![];
@@ -229,7 +222,7 @@ impl<'source, 'ast> PrettyPrinter<'source, 'ast> {
         tok_ranges.push(SourceRange::new(self.state.source.len(), 0));
 
         // Our tokens should be sorted.
-        assert!(tok_ranges.is_sorted_by(|x, y| Some(range_compare(*x, *y))));
+        assert!(tok_ranges.is_sorted_by(|x, y| range_compare(*x, *y)));
 
         // For each range, add a gap range between the previous range and this range.
         let mut gaps = vec![];
@@ -359,7 +352,6 @@ impl<'source, 'ast> PrettyPrinter<'source, 'ast> {
             {
                 next_newline += 1;
             }
-            #[allow(clippy::nonminimal_bool)] // for old clippy; false positive?
             let contains_newline = next_newline != newline_offsets.len() && {
                 let newline_offset = newline_offsets[next_newline];
                 assert!(newline_offset >= brace_statement.source_range().start());
@@ -369,7 +361,7 @@ impl<'source, 'ast> PrettyPrinter<'source, 'ast> {
                 result.push(brace_statement.source_range().start());
             }
         }
-        assert!(result.is_sorted_by(|l, r| Some(l.cmp(r))));
+        assert!(result.is_sorted_by(|l, r| l <= r));
         result
     }
 }
@@ -464,8 +456,10 @@ impl<'source, 'ast> PrettyPrinterState<'source, 'ast> {
     // Emit a space or indent as necessary, depending on the previous output.
     fn emit_space_or_indent(&mut self, flags: GapFlags) {
         if self.at_line_start() {
-            self.output
-                .extend(std::iter::repeat(' ').take(SPACES_PER_INDENT * self.current_indent));
+            self.output.extend(std::iter::repeat_n(
+                ' ',
+                SPACES_PER_INDENT * self.current_indent,
+            ));
         } else if !flags.skip_space && !self.has_preceding_space() {
             self.output.push(' ');
         }
@@ -585,7 +579,7 @@ impl<'source, 'ast> PrettyPrinterState<'source, 'ast> {
         let range_is_before = |x: SourceRange, y: SourceRange| x.end().cmp(&y.start());
         // FIXME: We want to have the errors sorted, but in some cases they aren't.
         // I suspect this is when the ast is unwinding because the source is fudged up.
-        if errs.is_sorted_by(|&x, &y| Some(range_is_before(x, y))) {
+        if errs.is_sorted_by(|&x, &y| range_is_before(x, y).is_le()) {
             errs.partition_point(|&range| range_is_before(range, r).is_lt()) != errs.len()
         } else {
             false
@@ -896,13 +890,10 @@ fn throwing_main() -> i32 {
     let io_chain = IoChain::new();
     let mut streams = IoStreams::new(&mut out, &mut err, &io_chain);
     streams.stdin_fd = STDIN_FILENO;
-    // Using the user's default locale could be a problem if it doesn't use UTF-8 encoding. That's
-    // because the fish project assumes Unicode UTF-8 encoding in all of its scripts.
-    //
-    {
-        let s = CString::new("").unwrap();
-        unsafe { libc::setlocale(LC_ALL, s.as_ptr()) };
-    }
+    // Safety: single-threaded.
+    unsafe {
+        set_libc_locales(/*log_ok=*/ false)
+    };
     crate::wutil::gettext::initialize_gettext();
     env_init(None, true, false);
 
@@ -914,7 +905,7 @@ fn throwing_main() -> i32 {
     }
 
     let args: Vec<WString> = std::env::args_os()
-        .map(|osstr| str2wcstring(osstr.as_bytes()))
+        .map(|osstr| bytes2wcstring(osstr.as_bytes()))
         .collect();
     do_indent(&mut streams, args).builtin_status_code()
 }
@@ -970,7 +961,7 @@ fn do_indent(streams: &mut IoStreams, args: Vec<WString>) -> BuiltinResult {
             'v' => {
                 streams.out.appendln(wgettext_fmt!(
                     "%s, version %s",
-                    PROGRAM_NAME.get().unwrap(),
+                    get_program_name(),
                     crate::BUILD_VERSION
                 ));
                 return Ok(SUCCESS);
@@ -997,8 +988,8 @@ fn do_indent(streams: &mut IoStreams, args: Vec<WString>) -> BuiltinResult {
         if args.is_empty() && i == 0 {
             if output_type == OutputType::File {
                 streams.err.appendln(wgettext_fmt!(
-                    "Expected file path to read/write for -w:\n\n $ %ls -w foo.fish",
-                    PROGRAM_NAME.get().unwrap()
+                    "Expected file path to read/write for -w:\n\n $ %s -w foo.fish",
+                    get_program_name()
                 ));
                 return Err(STATUS_CMD_ERROR);
             }
@@ -1014,10 +1005,10 @@ fn do_indent(streams: &mut IoStreams, args: Vec<WString>) -> BuiltinResult {
                 }
             }
             std::mem::forget(fd);
-            src = str2wcstring(&buf);
+            src = bytes2wcstring(&buf);
         } else {
             let arg = args[i];
-            match fs::File::open(OsStr::from_bytes(&wcs2string(arg))) {
+            match fs::File::open(OsStr::from_bytes(&wcs2bytes(arg))) {
                 Ok(file) => {
                     match read_file(file) {
                         Ok(s) => src = s,
@@ -1038,7 +1029,7 @@ fn do_indent(streams: &mut IoStreams, args: Vec<WString>) -> BuiltinResult {
 
         if output_type == OutputType::PygmentsCsv {
             let output = make_pygments_csv(&src);
-            streams.out.append(str2wcstring(&output));
+            streams.out.append(bytes2wcstring(&output));
             i += 1;
             continue;
         }
@@ -1102,9 +1093,9 @@ fn do_indent(streams: &mut IoStreams, args: Vec<WString>) -> BuiltinResult {
             }
             OutputType::File => {
                 if output_wtext != src {
-                    match fs::File::create(OsStr::from_bytes(&wcs2string(output_location))) {
+                    match fs::File::create(OsStr::from_bytes(&wcs2bytes(output_location))) {
                         Ok(mut file) => {
-                            let _ = file.write_all(&wcs2string(&output_wtext));
+                            let _ = file.write_all(&wcs2bytes(&output_wtext));
                         }
                         Err(err) => {
                             streams.err.appendln(wgettext_fmt!(
@@ -1136,7 +1127,7 @@ fn do_indent(streams: &mut IoStreams, args: Vec<WString>) -> BuiltinResult {
             }
         }
 
-        streams.out.append(str2wcstring(&colored_output));
+        streams.out.append(bytes2wcstring(&colored_output));
         i += 1;
     }
     if retval == 0 {
@@ -1152,7 +1143,7 @@ static DUMP_PARSE_TREE: RelaxedAtomicBool = RelaxedAtomicBool::new(false);
 fn read_file(mut f: impl Read) -> Result<WString, ()> {
     let mut buf = vec![];
     f.read_to_end(&mut buf).map_err(|_| ())?;
-    Ok(str2wcstring(&buf))
+    Ok(bytes2wcstring(&buf))
 }
 
 fn highlight_role_to_string(role: HighlightRole) -> &'static wstr {
@@ -1299,7 +1290,7 @@ fn html_colorize(text: &wstr, colors: &[HighlightSpec]) -> Vec<u8> {
             html.push_str("</span>");
         }
         if i == 0 || color != last_color {
-            sprintf!(=> &mut html, "<span class=\"%ls\">", html_class_name_for_color(color));
+            sprintf!(=> &mut html, "<span class=\"%s\">", html_class_name_for_color(color));
         }
         last_color = color;
 
@@ -1314,9 +1305,9 @@ fn html_colorize(text: &wstr, colors: &[HighlightSpec]) -> Vec<u8> {
         }
     }
     html.push_str("</span></code></pre>");
-    wcs2string(&html)
+    wcs2bytes(&html)
 }
 
 fn no_colorize(text: &wstr) -> Vec<u8> {
-    wcs2string(text)
+    wcs2bytes(text)
 }
