@@ -77,7 +77,7 @@ fn parse_cmd_opts(
     argv: &mut [&wstr],
     parser: &Parser,
     streams: &mut IoStreams,
-) -> c_int {
+) -> BuiltinResult {
     let cmd = L!("function");
     let print_hints = false;
     let mut handling_named_arguments = false;
@@ -94,20 +94,20 @@ fn parse_cmd_opts(
                 if handling_named_arguments {
                     if is_read_only(&woptarg) {
                         streams.err.append(wgettext_fmt!(
-                            "%ls: variable '%ls' is read-only\n",
+                            "%s: variable '%s' is read-only\n",
                             cmd,
                             woptarg
                         ));
-                        return STATUS_INVALID_ARGS;
+                        return Err(STATUS_INVALID_ARGS);
                     }
                     opts.named_arguments.push(woptarg);
                 } else {
                     streams.err.append(wgettext_fmt!(
-                        "%ls: %ls: unexpected positional argument",
+                        "%s: %s: unexpected positional argument",
                         cmd,
                         woptarg
                     ));
-                    return STATUS_INVALID_ARGS;
+                    return Err(STATUS_INVALID_ARGS);
                 }
             }
             'd' => {
@@ -116,11 +116,11 @@ fn parse_cmd_opts(
             's' => {
                 let Some(signal) = Signal::parse(w.woptarg.unwrap()) else {
                     streams.err.append(wgettext_fmt!(
-                        "%ls: Unknown signal '%ls'",
+                        "%s: Unknown signal '%s'",
                         cmd,
                         w.woptarg.unwrap()
                     ));
-                    return STATUS_INVALID_ARGS;
+                    return Err(STATUS_INVALID_ARGS);
                 };
                 opts.events.push(EventDescription::Signal { signal });
             }
@@ -130,7 +130,7 @@ fn parse_cmd_opts(
                     streams
                         .err
                         .append(wgettext_fmt!(BUILTIN_ERR_VARNAME, cmd, name));
-                    return STATUS_INVALID_ARGS;
+                    return Err(STATUS_INVALID_ARGS);
                 }
                 opts.events.push(EventDescription::Variable { name });
             }
@@ -149,34 +149,26 @@ fn parse_cmd_opts(
                     };
                     if caller_id == 0 {
                         streams.err.append(wgettext_fmt!(
-                            "%ls: calling job for event handler not found",
+                            "%s: calling job for event handler not found",
                             cmd
                         ));
-                        return STATUS_INVALID_ARGS;
+                        return Err(STATUS_INVALID_ARGS);
                     }
                     e = EventDescription::CallerExit { caller_id };
                 } else if opt == 'p' && woptarg == "%self" {
                     let pid = Pid::new(getpid());
-                    e = EventDescription::ProcessExit { pid };
+                    e = EventDescription::ProcessExit { pid: Some(pid) };
                 } else {
-                    let Ok(pid @ 0..) = fish_wcstoi(woptarg) else {
-                        streams.err.append(wgettext_fmt!(
-                            "%ls: %ls: invalid process id",
-                            cmd,
-                            woptarg
-                        ));
-                        return STATUS_INVALID_ARGS;
-                    };
+                    let pid = parse_pid_may_be_zero(streams, cmd, woptarg)?;
                     if opt == 'p' {
-                        e = EventDescription::ProcessExit { pid: Pid::new(pid) };
+                        e = EventDescription::ProcessExit { pid };
                     } else {
                         // TODO: rationalize why a default of 0 is sensible.
-                        let internal_job_id = match Pid::new(pid) {
-                            Some(pid) => job_id_for_pid(pid, parser).unwrap_or(0),
-                            None => 0,
-                        };
+                        let internal_job_id = pid
+                            .and_then(|pid| job_id_for_pid(pid, parser))
+                            .unwrap_or_default();
                         e = EventDescription::JobExit {
-                            pid: Pid::new(pid),
+                            pid,
                             internal_job_id,
                         };
                     }
@@ -187,11 +179,11 @@ fn parse_cmd_opts(
                 let name = w.woptarg.unwrap().to_owned();
                 if is_read_only(&name) {
                     streams.err.append(wgettext_fmt!(
-                        "%ls: variable '%ls' is read-only\n",
+                        "%s: variable '%s' is read-only\n",
                         cmd,
                         name
                     ));
-                    return STATUS_INVALID_ARGS;
+                    return Err(STATUS_INVALID_ARGS);
                 }
                 handling_named_arguments = true;
                 opts.named_arguments.push(name);
@@ -208,7 +200,7 @@ fn parse_cmd_opts(
                     streams
                         .err
                         .append(wgettext_fmt!(BUILTIN_ERR_VARNAME, cmd, woptarg));
-                    return STATUS_INVALID_ARGS;
+                    return Err(STATUS_INVALID_ARGS);
                 }
                 opts.inherit_vars.push(woptarg.to_owned());
             }
@@ -217,7 +209,7 @@ fn parse_cmd_opts(
             }
             ':' => {
                 builtin_missing_argument(parser, streams, cmd, argv[w.wopt_index - 1], print_hints);
-                return STATUS_INVALID_ARGS;
+                return Err(STATUS_INVALID_ARGS);
             }
             ';' => {
                 builtin_unexpected_argument(
@@ -227,11 +219,11 @@ fn parse_cmd_opts(
                     argv[w.wopt_index - 1],
                     print_hints,
                 );
-                return STATUS_INVALID_ARGS;
+                return Err(STATUS_INVALID_ARGS);
             }
             '?' => {
                 builtin_unknown_option(parser, streams, cmd, argv[w.wopt_index - 1], print_hints);
-                return STATUS_INVALID_ARGS;
+                return Err(STATUS_INVALID_ARGS);
             }
             other => {
                 panic!("Unexpected retval from WGetopter: {}", other);
@@ -240,27 +232,39 @@ fn parse_cmd_opts(
     }
 
     *optind = w.wopt_index;
-    STATUS_CMD_OK
+    Ok(SUCCESS)
 }
 
-fn validate_function_name(function_name: &wstr, cmd: &wstr, streams: &mut IoStreams) -> c_int {
+fn validate_function_name(
+    argv: &mut [&wstr],
+    function_name: &mut WString,
+    cmd: &wstr,
+    streams: &mut IoStreams,
+) -> BuiltinResult {
+    if argv.len() < 2 {
+        streams
+            .err
+            .append(wgettext_fmt!("%s: function name required", cmd));
+        return Err(STATUS_INVALID_ARGS);
+    }
+    *function_name = argv[1].to_owned();
     if !valid_func_name(function_name) {
         streams.err.append(wgettext_fmt!(
-            "%ls: %ls: invalid function name",
+            "%s: %s: invalid function name",
             cmd,
             function_name,
         ));
-        return STATUS_INVALID_ARGS;
+        return Err(STATUS_INVALID_ARGS);
     }
     if parser_keywords_is_reserved(function_name) {
         streams.err.append(wgettext_fmt!(
-            "%ls: %ls: cannot use reserved keyword as function name",
+            "%s: %s: cannot use reserved keyword as function name",
             cmd,
             function_name
         ));
-        return STATUS_INVALID_ARGS;
+        return Err(STATUS_INVALID_ARGS);
     }
-    STATUS_CMD_OK
+    Ok(SUCCESS)
 }
 
 /// Define a function. Calls into `function.rs` to perform the heavy lifting of defining a
@@ -271,7 +275,7 @@ pub fn function(
     streams: &mut IoStreams,
     c_args: &mut [&wstr],
     func_node: NodeRef<BlockStatement>,
-) -> c_int {
+) -> BuiltinResult {
     // The wgetopt function expects 'function' as the first argument. Make a new vec with
     // that property. This is needed because this builtin has a different signature than the other
     // builtins.
@@ -281,23 +285,17 @@ pub fn function(
     let cmd = argv[0];
 
     // A valid function name has to be the first argument.
-    let function_name = argv[1].to_owned();
-    let mut retval = validate_function_name(&function_name, cmd, streams);
-    if retval != STATUS_CMD_OK {
-        return retval;
-    }
+    let mut function_name = WString::new();
+    validate_function_name(argv, &mut function_name, cmd, streams)?;
     let argv = &mut argv[1..];
 
     let mut opts = FunctionCmdOpts::default();
     let mut optind = 0;
-    retval = parse_cmd_opts(&mut opts, &mut optind, argv, parser, streams);
-    if retval != STATUS_CMD_OK {
-        return retval;
-    }
+    parse_cmd_opts(&mut opts, &mut optind, argv, parser, streams)?;
 
     if opts.print_help {
         builtin_print_error_trailer(parser, streams.err, cmd);
-        return STATUS_CMD_OK;
+        return Ok(SUCCESS);
     }
 
     if argv.len() != optind {
@@ -308,17 +306,17 @@ pub fn function(
                     streams
                         .err
                         .append(wgettext_fmt!(BUILTIN_ERR_VARNAME, cmd, arg));
-                    return STATUS_INVALID_ARGS;
+                    return Err(STATUS_INVALID_ARGS);
                 }
                 opts.named_arguments.push(arg.to_owned());
             }
         } else {
             streams.err.append(wgettext_fmt!(
-                "%ls: %ls: unexpected positional argument",
+                "%s: %s: unexpected positional argument",
                 cmd,
                 argv[optind],
             ));
-            return STATUS_INVALID_ARGS;
+            return Err(STATUS_INVALID_ARGS);
         }
     }
 
@@ -343,7 +341,7 @@ pub fn function(
             streams
                 .err
                 .append(wgettext_fmt!(BUILTIN_ERR_VARNAME, cmd, named));
-            return STATUS_INVALID_ARGS;
+            return Err(STATUS_INVALID_ARGS);
         }
     }
 
@@ -398,5 +396,5 @@ pub fn function(
         }
     }
 
-    STATUS_CMD_OK
+    Ok(SUCCESS)
 }
