@@ -1,7 +1,7 @@
 use super::prelude::*;
+use crate::common::bytes2wcstring;
 use crate::common::escape_string;
 use crate::common::reformat_for_screen;
-use crate::common::str2wcstring;
 use crate::common::valid_func_name;
 use crate::common::{EscapeFlags, EscapeStringStyle};
 use crate::event::{self};
@@ -13,6 +13,7 @@ use crate::parse_util::parse_util_compute_indents;
 use crate::parser_keywords::parser_keywords_is_reserved;
 use crate::termsize::termsize_last;
 
+#[derive(Default)]
 struct FunctionsCmdOpts<'args> {
     print_help: bool,
     erase: bool,
@@ -28,25 +29,6 @@ struct FunctionsCmdOpts<'args> {
     description: Option<&'args wstr>,
 }
 
-impl Default for FunctionsCmdOpts<'_> {
-    fn default() -> Self {
-        Self {
-            print_help: false,
-            erase: false,
-            list: false,
-            show_hidden: false,
-            query: false,
-            copy: false,
-            report_metadata: false,
-            no_metadata: false,
-            verbose: false,
-            handlers: false,
-            handlers_type: None,
-            description: None,
-        }
-    }
-}
-
 const NO_METADATA_SHORT: char = 2 as char;
 
 const SHORT_OPTIONS: &wstr = L!("Ht:Dacd:ehnqv");
@@ -126,7 +108,7 @@ fn parse_cmd_opts<'args>(
 }
 
 pub fn functions(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> BuiltinResult {
-    let Some(&cmd) = args.get(0) else {
+    let Some(&cmd) = args.first() else {
         return Err(STATUS_INVALID_ARGS);
     };
 
@@ -172,7 +154,7 @@ pub fn functions(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -
     if let Some(desc) = opts.description {
         if args.len() != 1 {
             streams.err.append(wgettext_fmt!(
-                "%ls: Expected exactly one function name\n",
+                "%s: Expected exactly one function name\n",
                 cmd
             ));
             builtin_print_error_trailer(parser, streams.err, cmd);
@@ -182,7 +164,7 @@ pub fn functions(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -
 
         if !function::exists(current_func, parser) {
             streams.err.append(wgettext_fmt!(
-                "%ls: Function '%ls' does not exist\n",
+                "%s: Function '%s' does not exist\n",
                 cmd,
                 current_func
             ));
@@ -280,7 +262,7 @@ pub fn functions(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -
             && !event::EVENT_FILTER_NAMES.contains(&opts.handlers_type.unwrap())
         {
             streams.err.append(wgettext_fmt!(
-                "%ls: Expected generic | variable | signal | exit | job-id for --handlers-type\n",
+                "%s: Expected generic | variable | signal | exit | job-id for --handlers-type\n",
                 cmd
             ));
             return Err(STATUS_INVALID_ARGS);
@@ -320,7 +302,7 @@ pub fn functions(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -
     if opts.copy {
         if args.len() != 2 {
             streams.err.append(wgettext_fmt!(
-                "%ls: Expected exactly two names (current function name, and new function name)\n",
+                "%s: Expected exactly two names (current function name, and new function name)\n",
                 cmd
             ));
             builtin_print_error_trailer(parser, streams.err, cmd);
@@ -331,7 +313,7 @@ pub fn functions(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -
 
         if !function::exists(current_func, parser) {
             streams.err.append(wgettext_fmt!(
-                "%ls: Function '%ls' does not exist\n",
+                "%s: Function '%s' does not exist\n",
                 cmd,
                 current_func
             ));
@@ -341,7 +323,7 @@ pub fn functions(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -
 
         if !valid_func_name(new_func) || parser_keywords_is_reserved(new_func) {
             streams.err.append(wgettext_fmt!(
-                "%ls: Illegal function name '%ls'\n",
+                "%s: Illegal function name '%s'\n",
                 cmd,
                 new_func
             ));
@@ -351,7 +333,7 @@ pub fn functions(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -
 
         if function::exists(new_func, parser) {
             streams.err.append(wgettext_fmt!(
-                "%ls: Function '%ls' already exists. Cannot create copy of '%ls'\n",
+                "%s: Function '%s' already exists. Cannot create copy of '%s'\n",
                 cmd,
                 new_func,
                 current_func
@@ -391,7 +373,7 @@ pub fn functions(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -
                 }
                 Some(path) => {
                     comment.push_utfstr(&wgettext_fmt!(
-                        "Defined in %ls @ line %d",
+                        "Defined in %s @ line %d",
                         path,
                         props.definition_lineno()
                     ));
@@ -406,7 +388,7 @@ pub fn functions(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -
                     }
                     Some(path) => {
                         comment.push_utfstr(&wgettext_fmt!(
-                            ", copied in %ls @ line %d",
+                            ", copied in %s @ line %d",
                             path,
                             props.copy_definition_lineno()
                         ));
@@ -420,7 +402,7 @@ pub fn functions(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -
 
         if !comment.is_empty() {
             def.push_utfstr(&sprintf!(
-                "# %ls\n%ls",
+                "# %s\n%s",
                 comment,
                 props.annotated_definition(arg)
             ));
@@ -437,7 +419,7 @@ pub fn functions(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -
             highlight_shell(&def, &mut colors, &parser.context(), false, None);
             streams
                 .out
-                .append(str2wcstring(&colorize(&def, &colors, parser.vars())));
+                .append(bytes2wcstring(&colorize(&def, &colors, parser.vars())));
         } else {
             streams.out.append(def);
         }
