@@ -1,7 +1,7 @@
 //! Implementation of the history builtin.
 
 use crate::history::in_private_mode;
-use crate::history::{self, history_session_id, History};
+use crate::history::{self, History, history_session_id};
 use crate::reader::commandline_get_state;
 
 use super::prelude::*;
@@ -113,7 +113,7 @@ fn check_for_unexpected_hist_args(
     if opts.search_type.is_some() || opts.show_time_format.is_some() || opts.null_terminate {
         let subcmd_str = opts.hist_cmd.to_wstr();
         streams.err.append(wgettext_fmt!(
-            "%ls: %ls: subcommand takes no options\n",
+            "%s: %s: subcommand takes no options\n",
             cmd,
             subcmd_str
         ));
@@ -140,7 +140,7 @@ fn parse_cmd_opts(
     parser: &Parser,
     streams: &mut IoStreams,
 ) -> BuiltinResult {
-    let Some(&cmd) = argv.get(0) else {
+    let Some(&cmd) = argv.first() else {
         return Err(STATUS_INVALID_ARGS);
     };
 
@@ -243,7 +243,7 @@ pub fn history(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) ->
 
     parse_cmd_opts(&mut opts, &mut optind, args, parser, streams)?;
 
-    let Some(&cmd) = args.get(0) else {
+    let Some(&cmd) = args.first() else {
         return Err(STATUS_INVALID_ARGS);
     };
 
@@ -335,7 +335,7 @@ pub fn history(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) ->
 
             if in_private_mode(parser.vars()) {
                 streams.err.append(wgettext_fmt!(
-                    "%ls: can't merge history in private mode\n",
+                    "%s: can't merge history in private mode\n",
                     cmd
                 ));
                 return Err(STATUS_INVALID_ARGS);
