@@ -1,17 +1,17 @@
 // Functions for executing the jobs builtin.
 
 use super::prelude::*;
-use crate::common::{escape_string, timef, EscapeFlags, EscapeStringStyle};
+use crate::common::{EscapeFlags, EscapeStringStyle, escape_string, timef};
 use crate::io::IoStreams;
 use crate::job_group::{JobId, MaybeJobId};
 use crate::parser::Parser;
-use crate::proc::{clock_ticks_to_seconds, have_proc_stat, proc_get_jiffies, Job, Pid};
+use crate::proc::{Job, clock_ticks_to_seconds, have_proc_stat, proc_get_jiffies};
 use crate::wchar_ext::WExt;
-use crate::wgetopt::{wopt, ArgType, WGetopter, WOption};
+use crate::wgetopt::{ArgType, WGetopter, WOption, wopt};
 use crate::wutil::wgettext;
 use crate::{
-    wchar::{wstr, WString, L},
-    wutil::{fish_wcstoi, sprintf, wgettext_fmt},
+    wchar::{L, WString, wstr},
+    wutil::{fish_wcstoi, wgettext_fmt},
 };
 use std::num::NonZeroU32;
 
@@ -22,7 +22,7 @@ enum JobsPrintMode {
     Default,      // print lots of general info
     PrintPid,     // print pid of each process in job
     PrintCommand, // print command name of each process in job
-    PrintGroup,   // print group id of job
+    PrintGroup,   // print group ID of job
     PrintNothing, // print nothing (exit status only)
 }
 
@@ -110,7 +110,7 @@ fn builtin_jobs_print(j: &Job, mode: JobsPrintMode, header: bool, streams: &mut
             }
 
             for p in j.processes() {
-                out += &sprintf!("%ls\n", p.argv0().unwrap())[..];
+                out += &sprintf!("%s\n", p.argv0().unwrap())[..];
             }
             streams.out.append(out);
         }
@@ -130,7 +130,7 @@ const LONG_OPTIONS: &[WOption] = &[
 
 /// The jobs builtin. Used for printing running jobs. Defined in builtin_jobs.c.
 pub fn jobs(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> BuiltinResult {
-    let cmd = match argv.get(0) {
+    let cmd = match argv.first() {
         Some(cmd) => *cmd,
         None => return Err(STATUS_INVALID_ARGS),
     };
@@ -196,7 +196,7 @@ pub fn jobs(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
                 match fish_wcstoi(&arg[1..]).ok().filter(|&job_id| job_id >= 0) {
                     None => {
                         streams.err.append(wgettext_fmt!(
-                            "%ls: '%ls' is not a valid job id\n",
+                            "%s: '%s' is not a valid job ID\n",
                             cmd,
                             arg
                         ));
@@ -214,19 +214,8 @@ pub fn jobs(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
                     }
                 }
             } else {
-                match fish_wcstoi(arg).ok().and_then(Pid::new) {
-                    None => {
-                        streams.err.append(wgettext_fmt!(
-                            "%ls: '%ls' is not a valid process id\n",
-                            cmd,
-                            arg
-                        ));
-                        return Err(STATUS_INVALID_ARGS);
-                    }
-                    Some(job_id) => {
-                        j = parser.job_get_from_pid(job_id);
-                    }
-                }
+                let pid = parse_pid(streams, cmd, arg)?;
+                j = parser.job_get_from_pid(pid)
             }
 
             if let Some(j) = j.filter(|j| !j.is_completed() && j.is_constructed()) {
@@ -236,7 +225,7 @@ pub fn jobs(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
                 if mode != JobsPrintMode::PrintNothing {
                     streams
                         .err
-                        .append(wgettext_fmt!("%ls: No suitable job: %ls\n", cmd, arg));
+                        .append(wgettext_fmt!("%s: No suitable job: %s\n", cmd, arg));
                 }
                 return Err(STATUS_CMD_ERROR);
             }
@@ -256,7 +245,7 @@ pub fn jobs(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
         if !streams.out_is_redirected && mode != JobsPrintMode::PrintNothing {
             streams
                 .out
-                .append(wgettext_fmt!("%ls: There are no jobs\n", argv[0]));
+                .append(wgettext_fmt!("%s: There are no jobs\n", argv[0]));
         }
         return Err(STATUS_CMD_ERROR);
     }
