@@ -9,11 +9,11 @@ use crate::path::path_apply_working_directory;
 use crate::util::wcsfilecmp_glob;
 use crate::wcstringutil::split_string_tok;
 use crate::wutil::{
-    file_id_for_path, lwstat, normalize_path, waccess, wbasename, wdirname, wrealpath, wstat,
-    INVALID_FILE_ID,
+    INVALID_FILE_ID, file_id_for_path, lwstat, normalize_path, waccess, wbasename, wdirname,
+    wrealpath, wstat,
 };
 use bitflags::bitflags;
-use libc::{mode_t, F_OK, PATH_MAX, R_OK, S_ISGID, S_ISUID, W_OK, X_OK};
+use libc::{F_OK, PATH_MAX, R_OK, S_ISGID, S_ISUID, W_OK, X_OK, mode_t};
 
 macro_rules! path_error {
     (
@@ -283,7 +283,7 @@ fn parse_opts<'args>(
                 let types_args = split_string_tok(w.woptarg.unwrap(), L!(","), None);
                 for t in types_args {
                     let Ok(r#type) = t.try_into() else {
-                        path_error!(streams, "%ls: Invalid type '%ls'\n", "path", t);
+                        path_error!(streams, "%s: Invalid type '%s'\n", "path", t);
                         return Err(STATUS_INVALID_ARGS);
                     };
                     *types |= r#type;
@@ -295,7 +295,7 @@ fn parse_opts<'args>(
                 let perms_args = split_string_tok(w.woptarg.unwrap(), L!(","), None);
                 for p in perms_args {
                     let Ok(perm) = p.try_into() else {
-                        path_error!(streams, "%ls: Invalid permission '%ls'\n", "path", p);
+                        path_error!(streams, "%s: Invalid permission '%s'\n", "path", p);
                         return Err(STATUS_INVALID_ARGS);
                     };
                     *perms |= perm;
@@ -465,8 +465,10 @@ fn path_normalize(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr])
 }
 
 fn path_mtime(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> BuiltinResult {
-    let mut opts = Options::default();
-    opts.relative_valid = true;
+    let mut opts = Options {
+        relative_valid: true,
+        ..Default::default()
+    };
     let mut optind = 0;
 
     parse_opts(&mut opts, &mut optind, 0, args, parser, streams)?;
@@ -531,21 +533,6 @@ fn find_extension(path: &wstr) -> Option<usize> {
     }
 }
 
-#[test]
-fn test_find_extension() {
-    let cases = [
-        (L!("foo.wmv"), Some(3)),
-        (L!("verylongfilename.wmv"), Some("verylongfilename".len())),
-        (L!("foo"), None),
-        (L!(".foo"), None),
-        (L!("./foo.wmv"), Some(5)),
-    ];
-
-    for (f, ext_idx) in cases {
-        assert_eq!(find_extension(f), ext_idx);
-    }
-}
-
 fn path_extension(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> BuiltinResult {
     let mut opts = Options::default();
     let mut optind = 0;
@@ -692,9 +679,11 @@ fn path_resolve(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) ->
 }
 
 fn path_sort(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> BuiltinResult {
-    let mut opts = Options::default();
-    opts.reverse_valid = true;
-    opts.unique_valid = true;
+    let mut opts = Options {
+        reverse_valid: true,
+        unique_valid: true,
+        ..Default::default()
+    };
     let mut optind = 0;
 
     parse_opts(&mut opts, &mut optind, 0, args, parser, streams)?;
@@ -709,7 +698,7 @@ fn path_sort(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Bu
         }
         None => wbasename,
         Some(k) => {
-            path_error!(streams, "%ls: Invalid sort key '%ls'\n", args[0], k);
+            path_error!(streams, "%s: Invalid sort key '%s'\n", args[0], k);
             return Err(STATUS_INVALID_ARGS);
         }
     };
@@ -861,11 +850,13 @@ fn path_filter_maybe_is(
     args: &mut [&wstr],
     is_is: bool,
 ) -> BuiltinResult {
-    let mut opts = Options::default();
-    opts.types_valid = true;
-    opts.perms_valid = true;
-    opts.invert_valid = true;
-    opts.all_valid = true;
+    let mut opts = Options {
+        types_valid: true,
+        perms_valid: true,
+        invert_valid: true,
+        all_valid: true,
+        ..Default::default()
+    };
     let mut optind = 0;
 
     parse_opts(&mut opts, &mut optind, 0, args, parser, streams)?;
@@ -959,7 +950,7 @@ fn path_is(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Buil
 
 /// The path builtin, for handling paths.
 pub fn path(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> BuiltinResult {
-    let Some(&cmd) = args.get(0) else {
+    let Some(&cmd) = args.first() else {
         return Err(STATUS_INVALID_ARGS);
     };
     let argc = args.len();
@@ -1006,3 +997,24 @@ pub fn path(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> Bui
     let args = &mut args[1..];
     return subcmd(parser, streams, args);
 }
+
+#[cfg(test)]
+mod tests {
+    use super::find_extension;
+    use crate::wchar::prelude::*;
+
+    #[test]
+    fn test_find_extension() {
+        let cases = [
+            (L!("foo.wmv"), Some(3)),
+            (L!("verylongfilename.wmv"), Some("verylongfilename".len())),
+            (L!("foo"), None),
+            (L!(".foo"), None),
+            (L!("./foo.wmv"), Some(5)),
+        ];
+
+        for (f, ext_idx) in cases {
+            assert_eq!(find_extension(f), ext_idx);
+        }
+    }
+}
