@@ -49,15 +49,15 @@
 // This file has been imported from source code of printf command in GNU Coreutils version 6.9.
 
 use super::prelude::*;
-use crate::locale::{get_numeric_locale, Locale};
+use crate::locale::{Locale, get_numeric_locale};
 use crate::wchar::encode_byte_to_char;
 use crate::wutil::{
     errors::Error,
     wcstod::wcstod,
-    wcstoi::{wcstoi_partial, Options as WcstoiOpts},
+    wcstoi::{Options as WcstoiOpts, wcstoi_partial},
     wstr_offset_in,
 };
-use fish_printf::{sprintf_locale, ToArg};
+use fish_printf::{ToArg, sprintf_locale};
 
 /// Return true if `c` is an octal digit.
 fn is_octal_digit(c: char) -> bool {
@@ -208,10 +208,10 @@ impl<'a, 'b> builtin_printf_state_t<'a, 'b> {
         if errcode != None && errcode != Some(Error::InvalidChar) && errcode != Some(Error::Empty) {
             match errcode.unwrap() {
                 Error::Overflow => {
-                    self.fatal_error(sprintf!("%ls: %ls", s, wgettext!("Number out of range")));
+                    self.fatal_error(sprintf!("%s: %s", s, wgettext!("Number out of range")));
                 }
                 Error::Empty => {
-                    self.fatal_error(sprintf!("%ls: %ls", s, wgettext!("Number was empty")));
+                    self.fatal_error(sprintf!("%s: %s", s, wgettext!("Number was empty")));
                 }
                 Error::InvalidChar => {
                     panic!("Unreachable");
@@ -219,11 +219,11 @@ impl<'a, 'b> builtin_printf_state_t<'a, 'b> {
             }
         } else if !end.is_empty() {
             if s.as_ptr() == end.as_ptr() {
-                self.fatal_error(wgettext_fmt!("%ls: expected a numeric value", s));
+                self.fatal_error(wgettext_fmt!("%s: expected a numeric value", s));
             } else {
                 // This isn't entirely fatal - the value should still be printed.
                 self.nonfatal_error(wgettext_fmt!(
-                    "%ls: value not completely converted (can't convert '%ls')",
+                    "%s: value not completely converted (can't convert '%s')",
                     s,
                     end
                 ));
@@ -246,20 +246,18 @@ impl<'a, 'b> builtin_printf_state_t<'a, 'b> {
         }
     }
 
-    /// Evaluate a printf conversion specification.  SPEC is the start of the directive, and CONVERSION
-    /// specifies the type of conversion.  SPEC does not include any length modifier or the
-    /// conversion specifier itself.  FIELD_WIDTH and PRECISION are the field width and
-    /// precision for '*' values, if HAVE_FIELD_WIDTH and HAVE_PRECISION are true, respectively.
-    /// ARGUMENT is the argument to be formatted.
+    /// Evaluate a printf conversion specification.
+    /// `spec` is the start of the directive, and `conversion` specifies the type of conversion.
+    /// `spec` does not include any length modifier or the conversion specifier itself.
+    /// `field_width` and `precision` are the field width and precision for '*' values, if any.
+    /// `argument` is the argument to be formatted.
     #[allow(clippy::collapsible_else_if, clippy::too_many_arguments)]
-    fn print_direc(
+    fn print_directive(
         &mut self,
         spec: &wstr,
         conversion: char,
-        have_field_width: bool,
-        field_width: i32,
-        have_precision: bool,
-        precision: i32,
+        field_width: Option<i64>,
+        precision: Option<i64>,
         argument: &wstr,
     ) {
         /// Printf macro helper which provides our locale.
@@ -283,21 +281,6 @@ impl<'a, 'b> builtin_printf_state_t<'a, 'b> {
         // Start with everything except the conversion specifier.
         let mut fmt = spec.to_owned();
 
-        // Create a copy of the % directive, with a width modifier substituted for any
-        // existing integer length modifier.
-        match conversion {
-            'x' | 'X' | 'd' | 'i' | 'o' | 'u' => {
-                fmt.push_str("ll");
-            }
-            'a' | 'e' | 'f' | 'g' | 'A' | 'E' | 'F' | 'G' => {
-                fmt.push_str("L");
-            }
-            's' | 'c' => {
-                fmt.push_str("l");
-            }
-            _ => {}
-        }
-
         // Append the conversion itself.
         fmt.push(conversion);
 
@@ -306,77 +289,96 @@ impl<'a, 'b> builtin_printf_state_t<'a, 'b> {
         match conversion {
             'd' | 'i' => {
                 let arg: i64 = string_to_scalar_type(argument, self);
-                if !have_field_width {
-                    if !have_precision {
-                        append_output_fmt!(fmt, arg);
-                    } else {
-                        append_output_fmt!(fmt, precision, arg);
-                    }
-                } else {
-                    if !have_precision {
-                        append_output_fmt!(fmt, field_width, arg);
-                    } else {
-                        append_output_fmt!(fmt, field_width, precision, arg);
-                    }
+                match field_width {
+                    Some(field_width) => match precision {
+                        Some(precision) => {
+                            append_output_fmt!(fmt, field_width, precision, arg);
+                        }
+                        None => {
+                            append_output_fmt!(fmt, field_width, arg);
+                        }
+                    },
+                    None => match precision {
+                        Some(precision) => {
+                            append_output_fmt!(fmt, precision, arg);
+                        }
+                        None => {
+                            append_output_fmt!(fmt, arg);
+                        }
+                    },
                 }
             }
             'o' | 'u' | 'x' | 'X' => {
                 let arg: u64 = string_to_scalar_type(argument, self);
-                if !have_field_width {
-                    if !have_precision {
-                        append_output_fmt!(fmt, arg);
-                    } else {
-                        append_output_fmt!(fmt, precision, arg);
-                    }
-                } else {
-                    if !have_precision {
-                        append_output_fmt!(fmt, field_width, arg);
-                    } else {
-                        append_output_fmt!(fmt, field_width, precision, arg);
-                    }
+                match field_width {
+                    Some(field_width) => match precision {
+                        Some(precision) => {
+                            append_output_fmt!(fmt, field_width, precision, arg);
+                        }
+                        None => {
+                            append_output_fmt!(fmt, field_width, arg);
+                        }
+                    },
+                    None => match precision {
+                        Some(precision) => {
+                            append_output_fmt!(fmt, precision, arg);
+                        }
+                        None => {
+                            append_output_fmt!(fmt, arg);
+                        }
+                    },
                 }
             }
 
             'a' | 'A' | 'e' | 'E' | 'f' | 'F' | 'g' | 'G' => {
                 let arg: f64 = string_to_scalar_type(argument, self);
-                if !have_field_width {
-                    if !have_precision {
-                        append_output_fmt!(fmt, arg);
-                    } else {
-                        append_output_fmt!(fmt, precision, arg);
-                    }
-                } else {
-                    if !have_precision {
-                        append_output_fmt!(fmt, field_width, arg);
-                    } else {
-                        append_output_fmt!(fmt, field_width, precision, arg);
-                    }
+                match field_width {
+                    Some(field_width) => match precision {
+                        Some(precision) => {
+                            append_output_fmt!(fmt, field_width, precision, arg);
+                        }
+                        None => {
+                            append_output_fmt!(fmt, field_width, arg);
+                        }
+                    },
+                    None => match precision {
+                        Some(precision) => {
+                            append_output_fmt!(fmt, precision, arg);
+                        }
+                        None => {
+                            append_output_fmt!(fmt, arg);
+                        }
+                    },
                 }
             }
 
-            'c' => {
-                if !have_field_width {
-                    append_output_fmt!(fmt, argument.char_at(0));
-                } else {
+            'c' => match field_width {
+                Some(field_width) => {
                     append_output_fmt!(fmt, field_width, argument.char_at(0));
                 }
-            }
+                None => {
+                    append_output_fmt!(fmt, argument.char_at(0));
+                }
+            },
 
-            's' => {
-                if !have_field_width {
-                    if !have_precision {
-                        append_output_fmt!(fmt, argument);
-                    } else {
-                        append_output_fmt!(fmt, precision, argument);
-                    }
-                } else {
-                    if !have_precision {
-                        append_output_fmt!(fmt, field_width, argument);
-                    } else {
+            's' => match field_width {
+                Some(field_width) => match precision {
+                    Some(precision) => {
                         append_output_fmt!(fmt, field_width, precision, argument);
                     }
-                }
-            }
+                    None => {
+                        append_output_fmt!(fmt, field_width, argument);
+                    }
+                },
+                None => match precision {
+                    Some(precision) => {
+                        append_output_fmt!(fmt, precision, argument);
+                    }
+                    None => {
+                        append_output_fmt!(fmt, argument);
+                    }
+                },
+            },
 
             _ => {
                 panic!("unexpected opt: {}", conversion);
@@ -384,18 +386,16 @@ impl<'a, 'b> builtin_printf_state_t<'a, 'b> {
         }
     }
 
-    /// Print the text in FORMAT, using ARGV for arguments to any `%' directives.
-    /// Return the number of elements of ARGV used.
+    /// Print the text in `format`, using `argv` for arguments to any `%' directives.
+    /// Return the number of elements of `argv` used.
     fn print_formatted(&mut self, format: &wstr, mut argv: &[&wstr]) -> usize {
         let mut argc = argv.len();
         let save_argc = argc; /* Preserve original value.  */
         let mut f: &wstr; /* Pointer into `format'.  */
-        let mut direc_start: &wstr; /* Start of % directive.  */
-        let mut direc_length: usize; /* Length of % directive.  */
-        let mut have_field_width: bool; /* True if FIELD_WIDTH is valid.  */
-        let mut field_width: c_int = 0; /* Arg to first '*'.  */
-        let mut have_precision: bool; /* True if PRECISION is valid.  */
-        let mut precision = 0; /* Arg to second '*'.  */
+        let mut directive_start: &wstr; /* Start of % directive.  */
+        let mut directive_length: usize; /* Length of % directive.  */
+        let mut field_width: Option<i64>; /* Arg to first '*'.  */
+        let mut precision: Option<i64>; /* Arg to second '*'.  */
         let mut ok = [false; 256]; /* ok['x'] is true if %x is allowed.  */
 
         // N.B. this was originally written as a loop like so:
@@ -414,11 +414,11 @@ impl<'a, 'b> builtin_printf_state_t<'a, 'b> {
 
             match f.char_at(0) {
                 '%' => {
-                    direc_start = f;
+                    directive_start = f;
                     f = &f[1..];
-                    direc_length = 1;
-                    have_field_width = false;
-                    have_precision = false;
+                    directive_length = 1;
+                    field_width = None;
+                    precision = None;
                     if f.char_at(0) == '%' {
                         self.append_output('%');
                         continue;
@@ -438,7 +438,7 @@ impl<'a, 'b> builtin_printf_state_t<'a, 'b> {
                     let mut continue_looking_for_flags = true;
                     while continue_looking_for_flags {
                         match f.char_at(0) {
-                            'I' | '\'' => {
+                            '\'' => {
                                 modify_allowed_format_specifiers(&mut ok, "aAceEosxX", false);
                             }
 
@@ -460,67 +460,62 @@ impl<'a, 'b> builtin_printf_state_t<'a, 'b> {
                         }
                         if continue_looking_for_flags {
                             f = &f[1..];
-                            direc_length += 1;
+                            directive_length += 1;
                         }
                     }
 
                     if f.char_at(0) == '*' {
                         f = &f[1..];
-                        direc_length += 1;
+                        directive_length += 1;
                         if argc > 0 {
                             let width: i64 = string_to_scalar_type(argv[0], self);
                             if (c_int::MIN as i64) <= width && width <= (c_int::MAX as i64) {
-                                field_width = width as c_int;
+                                field_width = Some(width);
                             } else {
-                                self.fatal_error(wgettext_fmt!(
-                                    "invalid field width: %ls",
-                                    argv[0]
-                                ));
+                                self.fatal_error(wgettext_fmt!("invalid field width: %s", argv[0]));
                             }
                             argv = &argv[1..];
                             argc -= 1;
                         } else {
-                            field_width = 0;
+                            field_width = Some(0);
                         }
-                        have_field_width = true;
                     } else {
                         while iswdigit(f.char_at(0)) {
                             f = &f[1..];
-                            direc_length += 1;
+                            directive_length += 1;
                         }
                     }
 
                     if f.char_at(0) == '.' {
                         f = &f[1..];
-                        direc_length += 1;
+                        directive_length += 1;
                         modify_allowed_format_specifiers(&mut ok, "c", false);
                         if f.char_at(0) == '*' {
                             f = &f[1..];
-                            direc_length += 1;
+                            directive_length += 1;
                             if argc > 0 {
                                 let prec: i64 = string_to_scalar_type(argv[0], self);
                                 if prec < 0 {
                                     // A negative precision is taken as if the precision were omitted,
                                     // so -1 is safe here even if prec < INT_MIN.
-                                    precision = -1;
+                                    precision = Some(-1);
                                 } else if (c_int::MAX as i64) < prec {
                                     self.fatal_error(wgettext_fmt!(
-                                        "invalid precision: %ls",
+                                        "invalid precision: %s",
                                         argv[0]
                                     ));
                                 } else {
-                                    precision = prec as c_int;
+                                    precision = Some(prec);
                                 }
                                 argv = &argv[1..];
                                 argc -= 1;
                             } else {
-                                precision = 0;
+                                precision = Some(0);
                             }
-                            have_precision = true;
                         } else {
                             while iswdigit(f.char_at(0)) {
                                 f = &f[1..];
-                                direc_length += 1;
+                                directive_length += 1;
                             }
                         }
                     }
@@ -532,9 +527,9 @@ impl<'a, 'b> builtin_printf_state_t<'a, 'b> {
                     let conversion = f.char_at(0);
                     if (conversion as usize) > 0xFF || !ok[conversion as usize] {
                         self.fatal_error(wgettext_fmt!(
-                            "%.*ls: invalid conversion specification",
-                            wstr_offset_in(f, direc_start) + 1,
-                            direc_start
+                            "%.*s: invalid conversion specification",
+                            wstr_offset_in(f, directive_start) + 1,
+                            directive_start
                         ));
                         return 0;
                     }
@@ -545,12 +540,10 @@ impl<'a, 'b> builtin_printf_state_t<'a, 'b> {
                         argv = &argv[1..];
                         argc -= 1;
                     }
-                    self.print_direc(
-                        &direc_start[..direc_length],
+                    self.print_directive(
+                        &directive_start[..directive_length],
                         f.char_at(0),
-                        have_field_width,
                         field_width,
-                        have_precision,
                         precision,
                         argument,
                     );
