@@ -1,33 +1,32 @@
 //! Implementation of the read builtin.
 
 use super::prelude::*;
+use crate::common::UnescapeStringStyle;
+use crate::common::bytes2wcstring;
 use crate::common::escape;
 use crate::common::read_blocked;
-use crate::common::str2wcstring;
 use crate::common::unescape_string;
 use crate::common::valid_var_name;
-use crate::common::UnescapeStringStyle;
 use crate::env::EnvMode;
 use crate::env::Environment;
 use crate::env::READ_BYTE_LIMIT;
 use crate::env::{EnvVar, EnvVarFlags};
-use crate::input_common::decode_input_byte;
 use crate::input_common::DecodeState;
 use crate::input_common::InvalidPolicy;
+use crate::input_common::decode_one_codepoint_utf8;
 use crate::nix::isatty;
+use crate::reader::ReaderConfig;
 use crate::reader::commandline_set_buffer;
 use crate::reader::reader_save_screen_state;
-use crate::reader::ReaderConfig;
-use crate::reader::{reader_pop, reader_push, reader_readline};
-use crate::tokenizer::Tok;
-use crate::tokenizer::Tokenizer;
+use crate::reader::{reader_pop, reader_push, reader_readline, set_shell_modes_temporarily};
 use crate::tokenizer::TOK_ACCEPT_UNFINISHED;
 use crate::tokenizer::TOK_ARGUMENT_LIST;
+use crate::tokenizer::Tok;
+use crate::tokenizer::Tokenizer;
 use crate::tty_handoff::TtyHandoff;
 use crate::wcstringutil::split_about;
 use crate::wcstringutil::split_string_tok;
 use crate::wutil;
-use crate::wutil::encoding::zero_mbstate;
 use crate::wutil::perror;
 use libc::SEEK_CUR;
 use std::num::NonZeroUsize;
@@ -125,7 +124,7 @@ fn parse_cmd_opts(
             }
             'i' => {
                 streams.err.append(wgettext_fmt!(
-                    "%ls: usage of -i for --silent is deprecated. Please use -s or --silent instead.\n",
+                    "%s: usage of -i for --silent is deprecated. Please use -s or --silent instead.\n",
                     cmd
                 ));
                 return Err(STATUS_INVALID_ARGS);
@@ -150,7 +149,7 @@ fn parse_cmd_opts(
                     Ok(n) if n >= 0 => NonZeroUsize::new(n.try_into().unwrap()),
                     Err(wutil::Error::Overflow) => {
                         streams.err.append(wgettext_fmt!(
-                            "%ls: Argument '%ls' is out of range\n",
+                            "%s: Argument '%s' is out of range\n",
                             cmd,
                             w.woptarg.unwrap()
                         ));
@@ -241,6 +240,7 @@ fn parse_cmd_opts(
 
 /// Read from the tty. This is only valid when the stream is stdin and it is attached to a tty and
 /// we weren't asked to split on null characters.
+#[allow(clippy::too_many_arguments)]
 fn read_interactive(
     parser: &Parser,
     buff: &mut WString,
@@ -255,23 +255,28 @@ fn read_interactive(
     let mut exit_res = Ok(SUCCESS);
 
     // Construct a configuration.
-    let mut conf = ReaderConfig::default();
-    conf.complete_ok = shell;
-    conf.highlight_ok = shell;
-    conf.syntax_check_ok = shell;
+    let conf = ReaderConfig {
+        complete_ok: shell,
+        highlight_ok: shell,
+        syntax_check_ok: shell,
 
-    // No autosuggestions or abbreviations in builtin_read.
-    conf.autosuggest_ok = false;
-    conf.expand_abbrev_ok = false;
+        // No autosuggestions or abbreviations in builtin_read.
+        autosuggest_ok: false,
+        expand_abbrev_ok: false,
 
-    conf.exit_on_interrupt = true;
-    conf.in_silent_mode = silent;
+        exit_on_interrupt: true,
+        in_silent_mode: silent,
 
-    conf.left_prompt_cmd = prompt.to_owned();
-    conf.right_prompt_cmd = right_prompt.to_owned();
-    conf.event = L!("fish_read");
+        left_prompt_cmd: prompt.to_owned(),
+        right_prompt_cmd: right_prompt.to_owned(),
+        event: L!("fish_read"),
 
-    conf.inputfd = inputfd;
+        inputfd,
+
+        ..Default::default()
+    };
+
+    let old_modes = set_shell_modes_temporarily(inputfd);
 
     // Keep in-memory history only.
     reader_push(parser, L!(""), conf);
@@ -288,7 +293,7 @@ fn read_interactive(
         let _interactive = parser.push_scope(|s| s.is_interactive = true);
         let mut scoped_handoff = TtyHandoff::new(reader_save_screen_state);
         scoped_handoff.enable_tty_protocols();
-        reader_readline(parser, nchars)
+        reader_readline(parser, old_modes, nchars)
     };
     if let Some(line) = mline {
         *buff = line;
@@ -366,7 +371,7 @@ fn read_in_chunks(fd: RawFd, buff: &mut WString, split_null: bool, do_seek: bool
         }
     }
 
-    *buff = str2wcstring(&narrow_buff);
+    *buff = bytes2wcstring(&narrow_buff);
     if buff.is_empty() && eof {
         exit_res = Err(STATUS_CMD_ERROR);
     }
@@ -389,8 +394,6 @@ fn read_one_char_at_a_time(
     let mut unconsumed = vec![];
 
     loop {
-        let mut state = zero_mbstate();
-
         let chars_read = buff.len();
         let res = loop {
             let mut b = [0_u8; 1];
@@ -400,17 +403,9 @@ fn read_one_char_at_a_time(
                 }
                 _ => {}
             }
-            let b = b[0];
-            unconsumed.push(b);
+            unconsumed.push(b[0]);
             nbytes += 1;
-            let mut consumed = 0;
-            match decode_input_byte(
-                buff,
-                InvalidPolicy::Passthrough,
-                &mut state,
-                &unconsumed,
-                &mut consumed,
-            ) {
+            match decode_one_codepoint_utf8(buff, InvalidPolicy::Passthrough, &unconsumed) {
                 DecodeState::Incomplete => continue,
                 DecodeState::Complete => {
                     unconsumed.clear();
@@ -457,7 +452,7 @@ fn validate_read_args(
 ) -> BuiltinResult {
     if opts.prompt.is_some() && opts.prompt_str.is_some() {
         streams.err.append(wgettext_fmt!(
-            "%ls: Options %ls and %ls cannot be used together\n",
+            "%s: Options %s and %s cannot be used together\n",
             cmd,
             "-p",
             "-P",
@@ -468,7 +463,7 @@ fn validate_read_args(
 
     if opts.delimiter.is_some() && opts.one_line {
         streams.err.append(wgettext_fmt!(
-            "%ls: Options %ls and %ls cannot be used together\n",
+            "%s: Options %s and %s cannot be used together\n",
             cmd,
             "--delimiter",
             "--line"
@@ -477,7 +472,7 @@ fn validate_read_args(
     }
     if opts.one_line && opts.split_null {
         streams.err.append(wgettext_fmt!(
-            "%ls: Options %ls and %ls cannot be used together\n",
+            "%s: Options %s and %s cannot be used together\n",
             cmd,
             "-z",
             "--line"
@@ -572,7 +567,7 @@ fn validate_read_args(
         }
         if EnvVar::flags_for(arg).contains(EnvVarFlags::READ_ONLY) {
             streams.err.append(wgettext_fmt!(
-                "%ls: %ls: cannot overwrite read-only variable",
+                "%s: %s: cannot overwrite read-only variable",
                 cmd,
                 arg
             ));
@@ -613,7 +608,7 @@ pub fn read(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
     if streams.stdin_fd < 0 {
         streams
             .err
-            .append(wgettext_fmt!("%ls: stdin is closed\n", cmd));
+            .append(wgettext_fmt!("%s: stdin is closed\n", cmd));
         return Err(STATUS_CMD_ERROR);
     }
 
