@@ -1,11 +1,11 @@
 use super::prelude::*;
+use crate::common::EscapeFlags;
+use crate::common::EscapeStringStyle;
 use crate::common::escape;
 use crate::common::escape_string;
 use crate::common::get_ellipsis_char;
 use crate::common::get_ellipsis_str;
 use crate::common::valid_var_name;
-use crate::common::EscapeFlags;
-use crate::common::EscapeStringStyle;
 use crate::env::EnvStackSetResult;
 use crate::env::EnvVarFlags;
 use crate::env::INHERITED_VARS;
@@ -13,8 +13,8 @@ use crate::event;
 use crate::event::Event;
 use crate::expand::expand_escape_string;
 use crate::expand::expand_escape_variable;
-use crate::history::history_session_id;
 use crate::history::History;
+use crate::history::history_session_id;
 use crate::{
     env::{EnvMode, EnvVar, Environment},
     wutil::wcstoi::wcstoi_partial,
@@ -22,10 +22,10 @@ use crate::{
 
 localizable_consts!(
     MISMATCHED_ARGS
-    "%ls: given %d indexes but %d values\n"
+    "%s: given %d indexes but %d values\n"
 
     UVAR_ERR
-    "%ls: successfully set universal '%ls'; but a global by that name shadows it\n"
+    "%s: successfully set universal '%s'; but a global by that name shadows it\n"
 );
 
 #[derive(Debug, Clone)]
@@ -315,10 +315,11 @@ fn warn_if_uvar_shadows_global(
     streams: &mut IoStreams,
     parser: &Parser,
 ) {
-    if opts.universal && parser.is_interactive() {
-        if parser.vars().getf(dest, EnvMode::GLOBAL).is_some() {
-            streams.err.append(wgettext_fmt!(UVAR_ERR, cmd, dest));
-        }
+    if opts.universal
+        && parser.is_interactive()
+        && parser.vars().getf(dest, EnvMode::GLOBAL).is_some()
+    {
+        streams.err.append(wgettext_fmt!(UVAR_ERR, cmd, dest));
     }
 }
 
@@ -327,28 +328,28 @@ fn handle_env_return(retval: EnvStackSetResult, cmd: &wstr, key: &wstr, streams:
         EnvStackSetResult::Ok => (),
         EnvStackSetResult::Perm => {
             streams.err.append(wgettext_fmt!(
-                "%ls: Tried to change the read-only variable '%ls'\n",
+                "%s: Tried to change the read-only variable '%s'\n",
                 cmd,
                 key
             ));
         }
         EnvStackSetResult::Scope => {
             streams.err.append(wgettext_fmt!(
-                "%ls: Tried to modify the special variable '%ls' with the wrong scope\n",
+                "%s: Tried to modify the special variable '%s' with the wrong scope\n",
                 cmd,
                 key
             ));
         }
         EnvStackSetResult::Invalid => {
             streams.err.append(wgettext_fmt!(
-                "%ls: Tried to modify the special variable '%ls' to an invalid value\n",
+                "%s: Tried to modify the special variable '%s' to an invalid value\n",
                 cmd,
                 key
             ));
         }
         EnvStackSetResult::NotFound => {
             streams.err.append(wgettext_fmt!(
-                "%ls: The variable '%ls' does not exist\n",
+                "%s: The variable '%s' does not exist\n",
                 cmd,
                 key
             ));
@@ -389,7 +390,7 @@ impl std::fmt::Display for EnvArrayParseError {
             "{}",
             match self {
                 EnvArrayParseError::InvalidIndex(varname) =>
-                    wgettext_fmt!("%ls: Invalid index starting at '%ls'\n", "set", varname)
+                    wgettext_fmt!("%s: Invalid index starting at '%s'\n", "set", varname)
                         .to_string(),
             }
         )
@@ -428,7 +429,7 @@ fn split_var_and_indexes<'a>(
         Ok(split) => Some(split),
         Err(EnvArrayParseError::InvalidIndex(varname)) => {
             streams.err.append(wgettext_fmt!(
-                "%ls: Invalid index starting at '%ls'\n",
+                "%s: Invalid index starting at '%s'\n",
                 "set",
                 &varname,
             ));
@@ -657,7 +658,7 @@ fn show_scope(var_name: &wstr, scope: EnvMode, streams: &mut IoStreams, vars: &d
     };
     let vals = var.as_list();
     streams.out.append(wgettext_fmt!(
-        "$%ls: set in %ls scope, %ls,%ls with %d elements",
+        "$%s: set in %s scope, %s,%s with %d elements",
         var_name,
         scope_name,
         exportv,
@@ -694,7 +695,7 @@ fn show_scope(var_name: &wstr, scope: EnvMode, streams: &mut IoStreams, vars: &d
         );
         streams
             .out
-            .append(sprintf!("$%ls[%d]: |%ls|\n", var_name, i + 1, &escaped_val));
+            .append(sprintf!("$%s[%d]: |%s|\n", var_name, i + 1, &escaped_val));
     }
 }
 
@@ -720,7 +721,7 @@ fn show(cmd: &wstr, parser: &Parser, streams: &mut IoStreams, args: &[&wstr]) ->
                     EscapeStringStyle::Script(EscapeFlags::NO_PRINTABLES | EscapeFlags::NO_QUOTED),
                 );
                 streams.out.append(wgettext_fmt!(
-                    "$%ls: originally inherited as |%ls|\n",
+                    "$%s: originally inherited as |%s|\n",
                     name,
                     escaped_val
                 ));
@@ -738,7 +739,7 @@ fn show(cmd: &wstr, parser: &Parser, streams: &mut IoStreams, args: &[&wstr]) ->
 
             if arg.contains('[') {
                 streams.err.append(wgettext_fmt!(
-                    "%ls: `set --show` does not allow slices with the var names\n",
+                    "%s: `set --show` does not allow slices with the var names\n",
                     cmd
                 ));
                 builtin_print_error_trailer(parser, streams.err, cmd);
@@ -754,7 +755,7 @@ fn show(cmd: &wstr, parser: &Parser, streams: &mut IoStreams, args: &[&wstr]) ->
                     EscapeStringStyle::Script(EscapeFlags::NO_PRINTABLES | EscapeFlags::NO_QUOTED),
                 );
                 streams.out.append(wgettext_fmt!(
-                    "$%ls: originally inherited as |%ls|\n",
+                    "$%s: originally inherited as |%s|\n",
                     arg,
                     escaped_val
                 ));
@@ -931,7 +932,7 @@ fn set_internal(
             .append(wgettext_fmt!(BUILTIN_ERR_VARNAME, cmd, split.varname));
         if let Some(pos) = split.varname.chars().position(|c| c == '=') {
             streams.err.append(wgettext_fmt!(
-                "%ls: Did you mean `set %ls %ls`?",
+                "%s: Did you mean `set %s %s`?",
                 cmd,
                 &escape(&split.varname[..pos]),
                 &escape(&split.varname[pos + 1..])
@@ -948,7 +949,7 @@ fn set_internal(
             if *ind <= 0 {
                 streams
                     .err
-                    .append(wgettext_fmt!("%ls: array index out of bounds\n", cmd));
+                    .append(wgettext_fmt!("%s: array index out of bounds\n", cmd));
                 builtin_print_error_trailer(parser, streams.err, cmd);
                 return Err(STATUS_INVALID_ARGS);
             }
@@ -956,7 +957,7 @@ fn set_internal(
         // Append and prepend are disallowed.
         if opts.append || opts.prepend {
             streams.err.append(wgettext_fmt!(
-                "%ls: Cannot use --append or --prepend when assigning to a slice",
+                "%s: Cannot use --append or --prepend when assigning to a slice",
                 cmd
             ));
             builtin_print_error_trailer(parser, streams.err, cmd);
