@@ -2,11 +2,11 @@
 
 use super::prelude::*;
 use crate::color::Color;
-use crate::common::str2wcstring;
+use crate::common::bytes2wcstring;
 use crate::screen::{is_dumb, only_grayscale};
-use crate::terminal::{use_terminfo, Outputter};
+use crate::terminal::{Outputter, use_terminfo};
 use crate::text_face::{
-    self, parse_text_face_and_options, PrintColorsArgs, SpecifiedTextFace, TextFace, TextStyling,
+    self, PrintColorsArgs, SpecifiedTextFace, TextFace, TextStyling, parse_text_face_and_options,
 };
 
 fn print_colors(
@@ -47,7 +47,7 @@ fn print_colors(
     } // conveniently, 'normal' is always the last color so we don't need to reset here
 
     let contents = outp.contents();
-    streams.out.append(str2wcstring(contents));
+    streams.out.append(bytes2wcstring(contents));
 }
 
 /// set_color builtin.
@@ -99,12 +99,12 @@ pub fn set_color(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -
             Err(UnknownColor(arg)) => {
                 streams
                     .err
-                    .append(wgettext_fmt!("%ls: Unknown color '%ls'\n", argv[0], arg));
+                    .append(wgettext_fmt!("%s: Unknown color '%s'\n", argv[0], arg));
                 return Err(STATUS_INVALID_ARGS);
             }
             Err(UnknownUnderlineStyle(arg)) => {
                 streams.err.append(wgettext_fmt!(
-                    "%ls: invalid underline style: %ls\n",
+                    "%s: invalid underline style: %s\n",
                     argv[0],
                     arg
                 ));
@@ -147,7 +147,7 @@ pub fn set_color(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -
 
     // Output the collected string.
     let contents = outp.contents();
-    streams.out.append(str2wcstring(contents));
+    streams.out.append(bytes2wcstring(contents));
 
     Ok(SUCCESS)
 }
