@@ -1,7 +1,7 @@
 use std::os::fd::AsRawFd;
 
 use crate::{
-    common::{escape, FilenameRef},
+    common::{FilenameRef, escape},
     fds::wopen_cloexec,
     nix::isatty,
     parser::Block,
@@ -17,7 +17,7 @@ pub fn source(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
     let argc = args.len();
 
     let opts = HelpOnlyCmdOpts::parse(args, parser, streams)?;
-    let cmd = match args.get(0) {
+    let cmd = match args.first() {
         Some(&cmd) => cmd,
         None => return Err(STATUS_INVALID_ARGS),
     };
@@ -39,14 +39,14 @@ pub fn source(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
         if streams.stdin_fd < 0 {
             streams
                 .err
-                .append(wgettext_fmt!("%ls: stdin is closed\n", cmd));
+                .append(wgettext_fmt!("%s: stdin is closed\n", cmd));
             return Err(STATUS_CMD_ERROR);
         }
         // Either a bare `source` which means to implicitly read from stdin or an explicit `-`.
         if argc == optind && isatty(streams.stdin_fd) {
             // Don't implicitly read from the terminal.
             streams.err.append(wgettext_fmt!(
-                "%ls: missing filename argument or input redirection\n",
+                "%s: missing filename argument or input redirection\n",
                 cmd
             ));
             return Err(STATUS_CMD_ERROR);
@@ -61,7 +61,7 @@ pub fn source(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
             Err(_) => {
                 let esc = escape(args[optind]);
                 streams.err.append(wgettext_fmt!(
-                    "%ls: Error encountered while sourcing file '%ls':\n",
+                    "%s: Error encountered while sourcing file '%s':\n",
                     cmd,
                     &esc
                 ));
@@ -98,7 +98,7 @@ pub fn source(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
         Err(err) => {
             let esc = escape(&func_filename);
             streams.err.append(wgettext_fmt!(
-                "%ls: Error while reading file '%ls'\n",
+                "%s: Error while reading file '%s'\n",
                 cmd,
                 if esc == "-" { L!("<stdin>") } else { &esc }
             ));
