@@ -1,17 +1,17 @@
 use std::os::unix::prelude::*;
 
 use super::prelude::*;
-use crate::common::{get_executable_path, str2wcstring, PROGRAM_NAME};
+use crate::common::{bytes2wcstring, get_program_name};
+use crate::env::config_paths::get_fish_path;
 use crate::future_feature_flags::{self as features, feature_test};
 use crate::proc::{
-    get_job_control_mode, get_login, is_interactive_session, set_job_control_mode, JobControl,
+    JobControl, get_job_control_mode, get_login, is_interactive_session, set_job_control_mode,
 };
 use crate::reader::reader_in_interactive_read;
 use crate::tty_handoff::{get_scroll_content_up_capability, xtversion};
-use crate::wutil::{waccess, wbasename, wdirname, wrealpath, Error};
+use crate::wutil::{Error, waccess, wbasename, wdirname, wrealpath};
+use cfg_if::cfg_if;
 use libc::F_OK;
-use nix::errno::Errno;
-use nix::NixPath;
 
 macro_rules! str_enum {
     ($name:ident, $(($val:ident, $str:expr)),* $(,)?) => {
@@ -64,6 +64,7 @@ enum StatusCmd {
     STATUS_BUILDINFO,
     STATUS_GET_FILE,
     STATUS_LIST_FILES,
+    STATUS_HELP_SECTIONS,
     STATUS_TERMINAL,
     STATUS_TEST_TERMINAL_FEATURE,
 }
@@ -87,6 +88,7 @@ str_enum!(
     (STATUS_FUNCTION, "function"),
     (STATUS_GET_FILE, "get-file"),
     (STATUS_LIST_FILES, "list-files"),
+    (STATUS_HELP_SECTIONS, "help-sections"),
     (STATUS_IS_BLOCK, "is-block"),
     (STATUS_IS_BREAKPOINT, "is-breakpoint"),
     (STATUS_IS_COMMAND_SUB, "is-command-substitution"),
@@ -205,7 +207,7 @@ fn print_features(streams: &mut IoStreams) {
             L!("off")
         };
         streams.out.append(sprintf!(
-            "%-*ls%-3s %ls %ls\n",
+            "%-*s%-3s %s %s\n",
             max_len + 1,
             md.name,
             set,
@@ -237,7 +239,7 @@ fn parse_cmd_opts(
                         Ok(level) if level >= 0 => level,
                         Err(Error::Overflow) | Ok(_) => {
                             streams.err.append(wgettext_fmt!(
-                                "%ls: Invalid level value '%ls'\n",
+                                "%s: Invalid level value '%s'\n",
                                 cmd,
                                 arg
                             ));
@@ -273,7 +275,7 @@ fn parse_cmd_opts(
                 }
                 let Ok(job_mode) = w.woptarg.unwrap().try_into() else {
                     streams.err.append(wgettext_fmt!(
-                        "%ls: Invalid job control mode '%ls'\n",
+                        "%s: Invalid job control mode '%s'\n",
                         cmd,
                         w.woptarg.unwrap()
                     ));
@@ -332,15 +334,30 @@ fn parse_cmd_opts(
 use rust_embed::RustEmbed;
 
 #[cfg(feature = "embed-data")]
+cfg_if!(
+    if #[cfg(use_prebuilt_docs)] {
+        #[derive(RustEmbed)]
+        #[folder = "user_doc/man/man1"]
+        #[prefix = "man/man1/"]
+        struct Docs;
+    } else {
+        #[derive(RustEmbed)]
+        #[folder = "$FISH_RESOLVED_BUILD_DIR/fish-man/man1"]
+        #[prefix = "man/man1/"]
+        struct Docs;
+    }
+);
+
+#[cfg(all(using_cmake, feature = "embed-data"))]
 #[derive(RustEmbed)]
-#[folder = "target/fish-man/man1"]
-#[prefix = "man/man1/"]
-struct Docs;
+#[folder = "$FISH_CMAKE_BINARY_DIR/share"]
+#[include = "__fish_build_paths.fish"]
+struct CMakeBinaryDir;
 
 pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> BuiltinResult {
     localizable_consts!(
         #[allow(dead_code)]
-        NO_EMBEDDED_FILES_MSG "%ls: fish was not built with embedded files"
+        NO_EMBEDDED_FILES_MSG "%s: fish was not built with embedded files"
     );
 
     let cmd = args[0];
@@ -391,7 +408,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
         };
         streams
             .out
-            .append(wgettext_fmt!("Job control: %ls\n", job_control_mode));
+            .append(wgettext_fmt!("Job control: %s\n", job_control_mode));
         streams.out.append(parser.stack_trace());
 
         return Ok(SUCCESS);
@@ -427,7 +444,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                     }
                     let Ok(new_mode) = args[0].try_into() else {
                         streams.err.append(wgettext_fmt!(
-                            "%ls: Invalid job control mode '%ls'\n",
+                            "%s: Invalid job control mode '%s'\n",
                             cmd,
                             args[0]
                         ));
@@ -472,29 +489,33 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                 ));
                 return Err(STATUS_INVALID_ARGS);
             }
-            #[cfg(feature = "embed-data")]
-            {
-                let arg = crate::common::wcs2string(args[0]);
-                let arg = std::str::from_utf8(&arg).unwrap();
-                if let Some(emfile) = crate::autoload::Asset::get(arg) {
-                    let src = str2wcstring(&emfile.data);
-                    streams.out.append(src);
-                    return Ok(SUCCESS);
-                } else if let Some(emfile) = Docs::get(arg) {
-                    let src = str2wcstring(&emfile.data);
-                    streams.out.append(src);
-                    return Ok(SUCCESS);
-                } else {
+            cfg_if!(
+                if #[cfg(not(feature = "embed-data"))] {
+                    streams
+                        .err
+                        .appendln(sprintf!(NO_EMBEDDED_FILES_MSG.localize(), cmd));
                     return Err(STATUS_CMD_ERROR);
+                } else {
+                    let arg = crate::common::wcs2bytes(args[0]);
+                    let arg = std::str::from_utf8(&arg).unwrap();
+                    cfg_if!(
+                        if #[cfg(using_cmake)] {
+                            let emfile = CMakeBinaryDir::get(arg);
+                        } else {
+                            let emfile = None;
+                        }
+                    );
+                    let Some(emfile) = emfile
+                        .or_else(|| crate::autoload::Asset::get(arg))
+                        .or_else(|| Docs::get(arg))
+                    else {
+                        return Err(STATUS_CMD_ERROR);
+                    };
+                    let src = bytes2wcstring(&emfile.data);
+                    streams.out.append(src);
+                    return Ok(SUCCESS);
                 }
-            }
-            #[cfg(not(feature = "embed-data"))]
-            {
-                streams
-                    .err
-                    .appendln(sprintf!(NO_EMBEDDED_FILES_MSG.localize(), cmd));
-                return Err(STATUS_CMD_ERROR);
-            }
+            );
         }
         c @ STATUS_LIST_FILES => {
             if args.len() > 1 {
@@ -507,39 +528,40 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                 ));
                 return Err(STATUS_INVALID_ARGS);
             }
-            #[cfg(feature = "embed-data")]
-            {
-                let mut have_file = false;
-                let arg = crate::common::wcs2string(args.get(0).unwrap_or(&L!("")));
-                let arg = std::str::from_utf8(&arg).unwrap();
-                for file in crate::autoload::Asset::iter() {
-                    if arg.is_empty() || file.starts_with(arg) {
-                        have_file = true;
-                        let src = str2wcstring(file.as_bytes());
-                        streams.out.appendln(src);
-                    }
-                }
-                for file in Docs::iter() {
-                    if arg.is_empty() || file.starts_with(arg) {
-                        have_file = true;
-                        let src = str2wcstring(file.as_bytes());
-                        streams.out.appendln(src);
-                    }
-                }
 
-                if have_file {
-                    return Ok(SUCCESS);
-                } else {
+            cfg_if!(
+                if #[cfg(not(feature = "embed-data"))] {
+                    streams
+                        .err
+                        .appendln(sprintf!(NO_EMBEDDED_FILES_MSG.localize(), cmd));
                     return Err(STATUS_CMD_ERROR);
+                } else {
+                    use crate::util::wcsfilecmp_glob;
+
+                    let mut paths = vec![];
+                    let arg = crate::common::wcs2bytes(args.first().unwrap_or(&L!("")));
+                    let arg = std::str::from_utf8(&arg).unwrap();
+                    let embedded_files = crate::autoload::Asset::iter().chain(Docs::iter());
+                    #[cfg(using_cmake)]
+                    let embedded_files = embedded_files.chain(CMakeBinaryDir::iter());
+                    for path in embedded_files {
+                        if arg.is_empty() || path.starts_with(arg) {
+                            paths.push(bytes2wcstring(path.as_bytes()));
+                        }
+                    }
+
+                    paths.sort_by(|a, b| wcsfilecmp_glob(a, b));
+                    for path in &paths {
+                        streams.out.appendln(path);
+                    }
+
+                    if !paths.is_empty() {
+                        return Ok(SUCCESS);
+                    } else {
+                        return Err(STATUS_CMD_ERROR);
+                    }
                 }
-            }
-            #[cfg(not(feature = "embed-data"))]
-            {
-                streams
-                    .err
-                    .appendln(sprintf!(NO_EMBEDDED_FILES_MSG.localize(), cmd));
-                return Err(STATUS_CMD_ERROR);
-            }
+            );
         }
         c @ STATUS_TEST_TERMINAL_FEATURE => {
             if args.len() != 1 {
@@ -554,7 +576,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
             }
             if args[0] != "scroll-content-up" {
                 streams.err.appendln(wgettext_fmt!(
-                    "%s %s: unrecognized feature '%ls'",
+                    "%s %s: unrecognized feature '%s'",
                     cmd,
                     c.to_wstr(),
                     args[0]
@@ -581,16 +603,13 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
             }
             match s {
                 STATUS_BUILD_INFO | STATUS_BUILDINFO => {
-                    let version = str2wcstring(crate::BUILD_VERSION.as_bytes());
-                    let target = str2wcstring(env!("BUILD_TARGET_TRIPLE").as_bytes());
-                    let host = str2wcstring(env!("BUILD_HOST_TRIPLE").as_bytes());
-                    let profile = str2wcstring(env!("BUILD_PROFILE").as_bytes());
+                    let version = bytes2wcstring(crate::BUILD_VERSION.as_bytes());
+                    let target = bytes2wcstring(env!("BUILD_TARGET_TRIPLE").as_bytes());
+                    let host = bytes2wcstring(env!("BUILD_HOST_TRIPLE").as_bytes());
+                    let profile = bytes2wcstring(env!("BUILD_PROFILE").as_bytes());
                     streams.out.append(L!("Build system: "));
-                    let buildsystem = match option_env!("CMAKE") {
-                        Some("1") => "CMake",
-                        _ => "Cargo",
-                    };
-                    streams.out.appendln(str2wcstring(buildsystem.as_bytes()));
+                    let buildsystem = if cfg!(using_cmake) { "CMake" } else { "Cargo" };
+                    streams.out.appendln(bytes2wcstring(buildsystem.as_bytes()));
                     streams.out.append(L!("Version: "));
                     streams.out.appendln(version);
                     if target == host {
@@ -615,7 +634,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                     ];
                     streams
                         .out
-                        .appendln(str2wcstring(features.join(" ").as_bytes()));
+                        .appendln(bytes2wcstring(features.join(" ").as_bytes()));
                     streams.out.appendln("");
                     return Ok(SUCCESS);
                 }
@@ -715,7 +734,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                     if !command.is_empty() {
                         streams.out.appendln(command);
                     } else {
-                        streams.out.appendln(*PROGRAM_NAME.get().unwrap());
+                        streams.out.appendln(get_program_name());
                     }
                 }
                 STATUS_CURRENT_COMMANDLINE => {
@@ -724,31 +743,27 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                     streams.out.append_char('\n');
                 }
                 STATUS_FISH_PATH => {
-                    let path = get_executable_path("fish");
-                    if path.is_empty() {
-                        streams.err.append(sprintf!(
-                            "%ls: Could not get executable path: '%s'\n",
-                            cmd,
-                            Errno::last().to_string()
-                        ));
-                    }
-                    if path.is_absolute() {
-                        let path = str2wcstring(path.as_os_str().as_bytes());
-                        // This is an absolute path, we can canonicalize it
-                        let real = match wrealpath(&path) {
-                            Some(p) if waccess(&p, F_OK) == 0 => p,
-                            // realpath did not work, just append the path
-                            // - maybe this was obtained via $PATH?
-                            _ => path,
-                        };
-
-                        streams.out.append(real);
-                        streams.out.append_char('\n');
-                    } else {
-                        // This is a relative path, we can't canonicalize it
-                        let path = str2wcstring(path.as_os_str().as_bytes());
-                        streams.out.appendln(path);
-                    }
+                    use crate::env::config_paths::FishPath::*;
+                    let result = match get_fish_path() {
+                        Absolute(path) => {
+                            let path = bytes2wcstring(path.as_os_str().as_bytes());
+                            Cow::Owned(match wrealpath(&path) {
+                                Some(p) if waccess(&p, F_OK) == 0 => p,
+                                // realpath did not work, just append the path
+                                // - maybe this was obtained via $PATH?
+                                _ => path,
+                            })
+                        }
+                        LookUpInPath => Cow::Borrowed(get_program_name()),
+                    };
+                    streams.out.appendln(result);
+                }
+                STATUS_HELP_SECTIONS => {
+                    #[cfg(feature = "embed-data")]
+                    streams
+                        .out
+                        .append_narrow(fish_build_man_pages::HELP_SECTIONS);
+                    return Ok(SUCCESS);
                 }
                 STATUS_TERMINAL => {
                     let xtversion = xtversion().unwrap_or_default();
