@@ -17,6 +17,9 @@ mod transform;
 mod trim;
 mod unescape;
 
+#[cfg(test)]
+mod test_helpers;
+
 macro_rules! string_error {
     (
     $streams:expr,
@@ -179,25 +182,25 @@ impl RegexError {
             Compile(pattern, e) => {
                 string_error!(
                     streams,
-                    "%ls: Regular expression compile error: %ls\n",
+                    "%s: Regular expression compile error: %s\n",
                     cmd,
                     &WString::from(e.error_message())
                 );
-                string_error!(streams, "%ls: %ls\n", cmd, pattern);
-                string_error!(streams, "%ls: %*ls\n", cmd, e.offset().unwrap_or(0), "^");
+                string_error!(streams, "%s: %s\n", cmd, pattern);
+                string_error!(streams, "%s: %*s\n", cmd, e.offset().unwrap_or(0), "^");
             }
             InvalidCaptureGroupName(name) => {
                 streams.err.append(wgettext_fmt!(
-                    "Modification of read-only variable \"%ls\" is not allowed\n",
+                    "Modification of read-only variable \"%s\" is not allowed\n",
                     name
                 ));
             }
             InvalidEscape(pattern) => {
                 string_error!(
                     streams,
-                    "%ls",
+                    "%s",
                     sprintf!(
-                        "%ls: Invalid escape sequence in pattern \"%ls\"\n",
+                        "%s: Invalid escape sequence in pattern \"%s\"\n",
                         cmd,
                         pattern
                     )
@@ -290,7 +293,7 @@ fn width_without_escapes(ins: &wstr, start_pos: usize) -> usize {
 
 /// Empirically determined.
 /// This is probably down to some pipe buffer or some such,
-/// but too small means we need to call `read(2)` and str2wcstring a lot.
+/// but too small means we need to call `read(2)` and bytes2wcstring a lot.
 const STRING_CHUNK_SIZE: usize = 1024;
 fn arguments<'iter, 'args>(
     args: &'iter [&'args wstr],
