@@ -36,7 +36,7 @@ impl StringSubCommand<'_> for Pad {
             'c' => {
                 let [pad_char] = arg.unwrap().as_char_slice() else {
                     return Err(invalid_args!(
-                        "%ls: Padding should be a character '%ls'\n",
+                        "%s: Padding should be a character '%s'\n",
                         name,
                         arg
                     ));
@@ -44,7 +44,7 @@ impl StringSubCommand<'_> for Pad {
                 let pad_char_width = fish_wcwidth(*pad_char);
                 if pad_char_width <= 0 {
                     return Err(invalid_args!(
-                        "%ls: Invalid padding character of width zero '%ls'\n",
+                        "%s: Invalid padding character of width zero '%s'\n",
                         name,
                         arg
                     ));
@@ -56,7 +56,7 @@ impl StringSubCommand<'_> for Pad {
             'w' => {
                 self.width = fish_wcstol(arg.unwrap())?
                     .try_into()
-                    .map_err(|_| invalid_args!("%ls: Invalid width value '%ls'\n", name, arg))?
+                    .map_err(|_| invalid_args!("%s: Invalid width value '%s'\n", name, arg))?
             }
             'C' => self.center = true,
             _ => return Err(StringError::UnknownOption),
@@ -85,8 +85,6 @@ impl StringSubCommand<'_> for Pad {
         let pad_width = max_width.max(self.width);
 
         for (input, width) in inputs {
-            use std::iter::repeat;
-
             let total_pad = pad_width - width;
             let (left_pad, right_pad) = match (self.pad_from, self.center) {
                 (Direction::Left, false) => (total_pad, 0),
@@ -95,8 +93,8 @@ impl StringSubCommand<'_> for Pad {
                 (Direction::Right, true) => (total_pad / 2, total_pad - total_pad / 2),
             };
 
-            let chars = |w| repeat(self.char_to_pad).take(w / self.pad_char_width);
-            let spaces = |w| repeat(' ').take(w % self.pad_char_width);
+            let chars = |w| std::iter::repeat_n(self.char_to_pad, w / self.pad_char_width);
+            let spaces = |w| std::iter::repeat_n(' ', w % self.pad_char_width);
             let mut padded: WString = chars(left_pad)
                 .chain(spaces(left_pad))
                 .chain(input.chars())
