@@ -1,5 +1,5 @@
 use super::*;
-use crate::common::{unescape_string, UnescapeStringStyle};
+use crate::common::{UnescapeStringStyle, unescape_string};
 
 #[derive(Default)]
 pub struct Unescape {
@@ -23,7 +23,7 @@ impl StringSubCommand<'_> for Unescape {
                 self.style = arg
                     .unwrap()
                     .try_into()
-                    .map_err(|_| invalid_args!("%ls: Invalid style value '%ls'\n", name, arg))?
+                    .map_err(|_| invalid_args!("%s: Invalid style value '%s'\n", name, arg))?
             }
             _ => return Err(StringError::UnknownOption),
         }
