@@ -1,5 +1,5 @@
 use super::prelude::*;
-use crate::proc::{proc_wait_any, Job, Pid};
+use crate::proc::{Job, Pid, proc_wait_any};
 use crate::signal::SigChecker;
 use crate::wait_handle::{WaitHandleRef, WaitHandleStore};
 
@@ -195,28 +195,21 @@ pub fn wait(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -> Bui
     let optind = w.wopt_index;
     for item in &argv[optind..argc] {
         if iswnumeric(item) {
-            // argument is pid
-            let mpid: i32 = fish_wcstoi(item).unwrap_or(-1);
-            let Some(mpid) = Pid::new(mpid) else {
-                streams.err.append(wgettext_fmt!(
-                    "%ls: '%ls' is not a valid process id\n",
-                    cmd,
-                    item,
-                ));
+            let Ok(pid) = parse_pid(streams, cmd, item) else {
                 continue;
             };
-            if !find_wait_handles(WaitHandleQuery::Pid(mpid), parser, &mut wait_handles) {
+            if !find_wait_handles(WaitHandleQuery::Pid(pid), parser, &mut wait_handles) {
                 streams.err.append(wgettext_fmt!(
-                    "%ls: Could not find a job with process id '%d'\n",
+                    "%s: Could not find a job with process ID '%d'\n",
                     cmd,
-                    mpid,
+                    pid,
                 ));
             }
         } else {
             // argument is process name
             if !find_wait_handles(WaitHandleQuery::ProcName(item), parser, &mut wait_handles) {
                 streams.err.append(wgettext_fmt!(
-                    "%ls: Could not find child processes with the name '%ls'\n",
+                    "%s: Could not find child processes with the name '%s'\n",
                     cmd,
                     item,
                 ));
