@@ -5,29 +5,26 @@ use crate::expand::{
     PROCESS_EXPAND_SELF, PROCESS_EXPAND_SELF_STR, VARIABLE_EXPAND, VARIABLE_EXPAND_SINGLE,
 };
 use crate::fallback::fish_wcwidth;
-use crate::future_feature_flags::{feature_test, FeatureFlag};
+use crate::future_feature_flags::{FeatureFlag, feature_test};
 use crate::global_safety::AtomicRef;
 use crate::global_safety::RelaxedAtomicBool;
 use crate::key;
-use crate::libc::MB_CUR_MAX;
 use crate::parse_util::parse_util_escape_string_with_quote;
 use crate::terminal::Output;
 use crate::termsize::Termsize;
 use crate::wchar::{decode_byte_from_char, encode_byte_to_char, prelude::*};
-use crate::wcstringutil::wcs2string_callback;
+use crate::wcstringutil::wcs2bytes_callback;
 use crate::wildcard::{ANY_CHAR, ANY_STRING, ANY_STRING_RECURSIVE};
-use crate::wutil::encoding::{mbrtowc, wcrtomb, zero_mbstate, AT_LEAST_MB_LEN_MAX};
 use crate::wutil::fish_iswalnum;
 use bitflags::bitflags;
-use libc::{SIGTTOU, SIG_IGN, STDIN_FILENO};
+use libc::{SIG_IGN, SIGTTOU, STDIN_FILENO};
 use once_cell::sync::OnceCell;
 use std::cell::{Cell, RefCell};
 use std::env;
-use std::ffi::{CStr, CString, OsStr, OsString};
+use std::ffi::{CStr, CString, OsString};
 use std::mem;
 use std::ops::{Deref, DerefMut};
 use std::os::unix::prelude::*;
-use std::path::{Path, PathBuf};
 use std::sync::atomic::{AtomicI32, AtomicU32, Ordering};
 use std::sync::{Arc, MutexGuard};
 use std::time;
@@ -188,7 +185,7 @@ fn escape_string_script(input: &wstr, flags: EscapeFlags) -> WString {
     let no_quoted = flags.contains(EscapeFlags::NO_QUOTED);
     let no_tilde = flags.contains(EscapeFlags::NO_TILDE);
     let no_qmark = feature_test(FeatureFlag::qmark_noglob);
-    let symbolic = flags.contains(EscapeFlags::SYMBOLIC) && MB_CUR_MAX() > 1;
+    let symbolic = flags.contains(EscapeFlags::SYMBOLIC);
 
     assert!(
         !symbolic || !escape_printables,
@@ -207,7 +204,7 @@ fn escape_string_script(input: &wstr, flags: EscapeFlags) -> WString {
 
     let mut out = WString::new();
 
-    for c in input.chars() {
+    for (i, c) in input.chars().enumerate() {
         if let Some(val) = decode_byte_from_char(c) {
             out += "\\X";
 
@@ -311,7 +308,12 @@ fn escape_string_script(input: &wstr, flags: EscapeFlags) -> WString {
                 if c == '$' {
                     dollars += 1;
                 }
-                let char_is_normal = (c == '~' && no_tilde) || (c == '?' && no_qmark);
+                let char_is_normal = (c == '~' && no_tilde)
+                    || (c == '?' && no_qmark)
+                    || (c == '#'
+                        && i.checked_sub(1)
+                            .map(|previ| input.char_at(previ).is_alphanumeric())
+                            .unwrap_or_default());
                 if !char_is_normal {
                     need_escape = true;
                     if escape_printables {
@@ -384,7 +386,7 @@ fn byte_to_hex(byte: u8) -> (char, char) {
 
 /// Escape a string in a fashion suitable for using as a URL. Store the result in out_str.
 fn escape_string_url(input: &wstr) -> WString {
-    let narrow = wcs2string(input);
+    let narrow = wcs2bytes(input);
     let mut out = WString::new();
     for byte in narrow.into_iter() {
         if (byte & 0x80) == 0 {
@@ -407,7 +409,7 @@ fn escape_string_url(input: &wstr) -> WString {
 /// Escape a string in a fashion suitable for using as a fish var name. Store the result in out_str.
 fn escape_string_var(input: &wstr) -> WString {
     let mut prev_was_hex_encoded = false;
-    let narrow = wcs2string(input);
+    let narrow = wcs2bytes(input);
     let mut out = WString::new();
     for c in narrow.into_iter() {
         let ch: char = c.into();
@@ -788,7 +790,7 @@ fn unescape_string_url(input: &wstr) -> Option<WString> {
         i += 1
     }
 
-    Some(str2wcstring(&result))
+    Some(bytes2wcstring(&result))
 }
 
 /// Reverse the effects of `escape_string_var()`. By definition the string should consist of just
@@ -829,7 +831,7 @@ fn unescape_string_var(input: &wstr) -> Option<WString> {
         i += 1;
     }
 
-    Some(str2wcstring(&result))
+    Some(bytes2wcstring(&result))
 }
 
 /// Given a string starting with a backslash, read the escape as if it is unquoted, appending
@@ -847,8 +849,9 @@ pub fn read_unquoted_escape(
     let mut result_char_or_none: Option<char> = None;
 
     let mut errored = false;
-    let mut in_pos = 1; // in_pos always tracks the next character to read (and therefore the number
-                        // of characters read so far)
+    // in_pos always tracks the next character to read
+    // (and therefore the number of characters read so far)
+    let mut in_pos = 1;
 
     // For multibyte \X sequences.
     let mut byte_buff: Vec<u8> = vec![];
@@ -997,7 +1000,7 @@ pub fn read_unquoted_escape(
         }
 
         if !byte_buff.is_empty() {
-            result.push_utfstr(&str2wcstring(&byte_buff));
+            result.push_utfstr(&bytes2wcstring(&byte_buff));
         }
 
         break;
@@ -1029,22 +1032,17 @@ pub fn shell_modes() -> MutexGuard<'static, libc::termios> {
     crate::reader::SHELL_MODES.lock().unwrap()
 }
 
-/// The character to use where the text has been truncated. Is an ellipsis on unicode system and a $
-/// on other systems.
+/// The character to use where the text has been truncated.
 pub fn get_ellipsis_char() -> char {
-    char::from_u32(ELLIPSIS_CHAR.load(Ordering::Relaxed)).unwrap()
+    '\u{2026}' // ('…')
 }
 
-static ELLIPSIS_CHAR: AtomicU32 = AtomicU32::new(0);
-
 /// The character or string to use where text has been truncated (ellipsis if possible, otherwise
 /// ...)
 pub fn get_ellipsis_str() -> &'static wstr {
-    ELLIPSIS_STRING.load()
+    L!("\u{2026}")
 }
 
-static ELLIPSIS_STRING: AtomicRef<wstr> = AtomicRef::new(&L!(""));
-
 /// Character representing an omitted newline at the end of text.
 pub fn get_omitted_newline_str() -> &'static wstr {
     OMITTED_NEWLINE_STR.load()
@@ -1068,6 +1066,10 @@ pub static PROFILING_ACTIVE: RelaxedAtomicBool = RelaxedAtomicBool::new(false);
 /// Name of the current program. Should be set at startup. Used by the debug function.
 pub static PROGRAM_NAME: OnceCell<&'static wstr> = OnceCell::new();
 
+pub fn get_program_name() -> &'static wstr {
+    PROGRAM_NAME.get().unwrap()
+}
+
 /// MS Windows tty devices do not currently have either a read or write timestamp - those respective
 /// fields of `struct stat` are always set to the current time, which means we can't rely on them.
 /// In this case, we assume no external program has written to the terminal behind our back, making
@@ -1092,84 +1094,54 @@ pub static EMPTY_STRING: WString = WString::new();
 /// todo!("Maybe remove the box? It is only needed for get_bg_context.")
 pub type CancelChecker = Box<dyn Fn() -> bool>;
 
-/// Converts the narrow character string \c in into its wide equivalent, and return it.
-///
-/// The string may contain embedded nulls.
-///
-/// This function encodes illegal character sequences in a reversible way using the private use
-/// area.
-pub fn str2wcstring(inp: &[u8]) -> WString {
-    if inp.is_empty() {
+/// Encodes the bytes in `input` into a [`WString`], encoding non-UTF-8 bytes into private-use-area
+/// code-points. Bytes which would be parsed into our reserved PUA range are encoded individually,
+/// to allow for correct round-tripping.
+pub fn bytes2wcstring(mut input: &[u8]) -> WString {
+    if input.is_empty() {
         return WString::new();
     }
 
     let mut result = WString::new();
-    result.reserve(inp.len());
-    let mut pos = 0;
-    let mut state = zero_mbstate();
-    while pos < inp.len() {
-        // Append any initial sequence of ascii characters.
-        // Note we do not support character sets which are not supersets of ASCII.
-        let ascii_prefix_length = count_ascii_prefix(&inp[pos..]);
-        result.push_str(std::str::from_utf8(&inp[pos..pos + ascii_prefix_length]).unwrap());
-        pos += ascii_prefix_length;
-        assert!(pos <= inp.len(), "Position overflowed length");
-        if pos == inp.len() {
-            break;
-        }
 
-        // We have found a non-ASCII character.
-        let mut ret = 0;
-        let mut c = '\0';
-
-        let use_encode_direct = if inp[pos] & 0xF8 == 0xF8 {
-            // Protect against broken mbrtowc() implementations which attempt to encode UTF-8
-            // sequences longer than four bytes (e.g., OS X Snow Leopard).
-            // TODO This check used to be conditionally compiled only on affected platforms.
-            true
-        } else {
-            let mut codepoint = u32::from(c);
-            ret = unsafe {
-                mbrtowc(
-                    std::ptr::addr_of_mut!(codepoint),
-                    std::ptr::addr_of!(inp[pos]).cast(),
-                    inp.len() - pos,
-                    &mut state,
-                )
-            };
-            match char::from_u32(codepoint) {
-                Some(codepoint) => {
-                    c = codepoint;
-                    // Determine whether to encode this character with our crazy scheme.
-                    fish_reserved_codepoint(c)
-                    ||
-                    // Incomplete sequence.
-                    ret == 0_usize.wrapping_sub(2)
-                    ||
-                    // Invalid data.
-                    ret == 0_usize.wrapping_sub(1)
-                    ||
-                    // Other error codes? Terrifying, should never happen.
-                    ret > inp.len() - pos
+    fn append_escaped_str(output: &mut WString, input: &str) {
+        for (i, c) in input.char_indices() {
+            if fish_reserved_codepoint(c) {
+                for byte in &input.as_bytes()[i..i + c.len_utf8()] {
+                    output.push(encode_byte_to_char(*byte));
                 }
-                None => true,
+            } else {
+                output.push(c);
             }
-        };
+        }
+    }
 
-        if use_encode_direct {
-            c = encode_byte_to_char(inp[pos]);
-            result.push(c);
-            pos += 1;
-            state = zero_mbstate();
-        } else if ret == 0 {
-            // embedded null byte!
-            result.push('\0');
-            pos += 1;
-            state = zero_mbstate();
-        } else {
-            // normal case
-            result.push(c);
-            pos += ret;
+    while !input.is_empty() {
+        match std::str::from_utf8(input) {
+            Ok(parsed_str) => {
+                append_escaped_str(&mut result, parsed_str);
+                // The entire remaining input could be parsed, so we are done.
+                break;
+            }
+            Err(e) => {
+                let (valid, after_valid) = input.split_at(e.valid_up_to());
+                // SAFETY: The previous `str::from_utf8` call established that the prefix `valid`
+                // is valid UTF-8. This prefix may be empty.
+                let parsed_str = unsafe { std::str::from_utf8_unchecked(valid) };
+                append_escaped_str(&mut result, parsed_str);
+                // The length of the prefix of `after_valid` which is invalid UTF-8.
+                // The remaining bytes of `input` (if any) will be parsed in subsequent iterations
+                // of the loop, starting from the first byte that starts a valid UTF-8-encoded codepoint.
+                // `error_len` can return `None`, if it sees a byte sequence that could be the
+                // prefix of a valid code-point encoding at the end of the byte slice.
+                // This is useful when the input is chunked, but we don't do that, so in this case
+                // we use our custom encoding for all remaining bytes (at most 3).
+                let error_len = e.error_len().unwrap_or(after_valid.len());
+                for byte in &after_valid[..error_len] {
+                    result.push(encode_byte_to_char(*byte));
+                }
+                input = &after_valid[error_len..];
+            }
         }
     }
     result
@@ -1186,12 +1158,12 @@ pub fn truncate_at_nul(input: &wstr) -> &wstr {
 
 pub fn cstr2wcstring(input: &[u8]) -> WString {
     let input = CStr::from_bytes_until_nul(input).unwrap().to_bytes();
-    str2wcstring(input)
+    bytes2wcstring(input)
 }
 
 pub(crate) fn charptr2wcstring(input: *const libc::c_char) -> WString {
     let input: &[u8] = unsafe { CStr::from_ptr(input).to_bytes() };
-    str2wcstring(input)
+    bytes2wcstring(input)
 }
 
 /// Returns a newly allocated multibyte character string equivalent of the specified wide character
@@ -1199,13 +1171,13 @@ pub(crate) fn charptr2wcstring(input: *const libc::c_char) -> WString {
 ///
 /// This function decodes illegal character sequences in a reversible way using the private use
 /// area.
-pub fn wcs2string(input: &wstr) -> Vec<u8> {
+pub fn wcs2bytes(input: &wstr) -> Vec<u8> {
     if input.is_empty() {
         return vec![];
     }
 
     let mut result = vec![];
-    wcs2string_appending(&mut result, input);
+    wcs2bytes_appending(&mut result, input);
     result
 }
 
@@ -1215,11 +1187,11 @@ pub fn wcs2osstring(input: &wstr) -> OsString {
     }
 
     let mut result = vec![];
-    wcs2string_appending(&mut result, input);
+    wcs2bytes_appending(&mut result, input);
     OsString::from_vec(result)
 }
 
-/// Same as [`wcs2string`]. Meant to be used when we need a zero-terminated string to feed legacy APIs.
+/// Same as [`wcs2bytes`]. Meant to be used when we need a zero-terminated string to feed legacy APIs.
 /// Note: if `input` contains any interior NUL bytes, the result will be truncated at the first!
 pub fn wcs2zstring(input: &wstr) -> CString {
     if input.is_empty() {
@@ -1227,7 +1199,7 @@ pub fn wcs2zstring(input: &wstr) -> CString {
     }
 
     let mut vec = Vec::with_capacity(input.len() + 1);
-    wcs2string_callback(input, |buff| {
+    wcs2bytes_callback(input, |buff| {
         vec.extend_from_slice(buff);
         true
     });
@@ -1246,40 +1218,19 @@ pub fn wcs2zstring(input: &wstr) -> CString {
     }
 }
 
-/// Like wcs2string, but appends to `receiver` instead of returning a new string.
-pub fn wcs2string_appending(output: &mut Vec<u8>, input: &wstr) {
+/// Like [`wcs2bytes`], but appends to `output` instead of returning a new string.
+pub fn wcs2bytes_appending(output: &mut Vec<u8>, input: &wstr) {
     output.reserve(input.len());
-    wcs2string_callback(input, |buff| {
+    wcs2bytes_callback(input, |buff| {
         output.extend_from_slice(buff);
         true
     });
 }
 
-/// Return the count of initial characters in `in` which are ASCII.
-fn count_ascii_prefix(inp: &[u8]) -> usize {
-    // The C++ version had manual vectorization.
-    inp.iter().take_while(|c| c.is_ascii()).count()
-}
-
-// Check if we are running in the test mode, where we should suppress error output
-pub const TESTS_PROGRAM_NAME: &wstr = L!("(ignore)");
-
-/// Hack to not print error messages in the tests. Do not call this from functions in this module
-/// like `debug()`. It is only intended to suppress diagnostic noise from testing things like the
-/// fish parser where we expect a lot of diagnostic messages due to testing error conditions.
-pub fn should_suppress_stderr_for_tests() -> bool {
-    PROGRAM_NAME
-        .get()
-        .map(|p| p == TESTS_PROGRAM_NAME)
-        .unwrap_or_default()
-}
-
 /// Stored in blocks to reference the file which created the block.
 pub type FilenameRef = Arc<WString>;
 
-/// This function should be called after calling `setlocale()` to perform fish specific locale
-/// initialization.
-pub fn fish_setlocale() {
+pub fn init_special_chars_once() {
     // Helper to make a static reference to a static &'wstr, from a string literal.
     // This is necessary to store them in global atomics, as these can't handle fat pointers.
     macro_rules! LL {
@@ -1289,51 +1240,25 @@ pub fn fish_setlocale() {
         }};
     }
 
-    // Use various Unicode symbols if they can be encoded using the current locale, else a simple
-    // ASCII char alternative. All of the can_be_encoded() invocations should return the same
-    // true/false value since the code points are in the BMP but we're going to be paranoid. This
-    // is also technically wrong if we're not in a Unicode locale but we expect (or hope)
-    // can_be_encoded() will return false in that case.
-    if can_be_encoded('\u{2026}') {
-        ELLIPSIS_CHAR.store(u32::from('\u{2026}'), Ordering::Relaxed);
-        ELLIPSIS_STRING.store(LL!("\u{2026}"));
-    } else {
-        ELLIPSIS_CHAR.store(u32::from('$'), Ordering::Relaxed); // "horizontal ellipsis"
-        ELLIPSIS_STRING.store(LL!("..."));
-    }
-
     if is_windows_subsystem_for_linux(WSL::Any) {
         // neither of \u23CE and \u25CF can be displayed in the default fonts on Windows, though
         // they can be *encoded* just fine. Use alternative glyphs.
         OMITTED_NEWLINE_STR.store(LL!("\u{00b6}")); // "pilcrow"
-        OBFUSCATION_READ_CHAR.store(u32::from('\u{2022}'), Ordering::Relaxed); // "bullet"
+        OBFUSCATION_READ_CHAR.store(u32::from('\u{2022}'), Ordering::Relaxed); // "bullet" (•)
     } else if is_console_session() {
         OMITTED_NEWLINE_STR.store(LL!("^J"));
         OBFUSCATION_READ_CHAR.store(u32::from('*'), Ordering::Relaxed);
     } else {
-        if can_be_encoded('\u{23CE}') {
-            OMITTED_NEWLINE_STR.store(LL!("\u{23CE}")); // "return symbol" (⏎)
-        } else {
-            OMITTED_NEWLINE_STR.store(LL!("^J"));
-        }
+        OMITTED_NEWLINE_STR.store(LL!("\u{23CE}")); // "return symbol" (⏎)
         OBFUSCATION_READ_CHAR.store(
-            u32::from(if can_be_encoded('\u{25CF}') {
-                '\u{25CF}' // "black circle"
-            } else {
-                '#'
-            }),
+            u32::from(
+                '\u{25CF}', // "black circle" (●)
+            ),
             Ordering::Relaxed,
         );
     }
 }
 
-/// Test if the character can be encoded using the current locale.
-fn can_be_encoded(wc: char) -> bool {
-    let mut converted = [0 as libc::c_char; AT_LEAST_MB_LEN_MAX];
-    let mut state = zero_mbstate();
-    unsafe { wcrtomb(converted.as_mut_ptr(), wc as u32, &mut state) != 0_usize.wrapping_sub(1) }
-}
-
 /// Call read, blocking and repeating on EINTR. Exits on EAGAIN.
 /// Return the number of bytes read, or 0 on EOF, or an error.
 pub fn read_blocked(fd: RawFd, buf: &mut [u8]) -> nix::Result<usize> {
@@ -1414,7 +1339,7 @@ macro_rules! write_to_output {
 pub fn reformat_for_screen(msg: &wstr, termsize: &Termsize) -> WString {
     let mut buff = WString::new();
 
-    let screen_width = termsize.width;
+    let screen_width = isize::try_from(termsize.width()).unwrap();
     if screen_width != 0 {
         let mut start = 0;
         let mut pos = start;
@@ -1445,7 +1370,7 @@ pub fn reformat_for_screen(msg: &wstr, termsize: &Termsize) -> WString {
                 if line_width != 0 {
                     buff.push('\n');
                 }
-                buff += &sprintf!("%ls-\n", token)[..];
+                buff += &sprintf!("%s-\n", token)[..];
                 line_width = 0;
             } else {
                 // Print the token.
@@ -1646,31 +1571,6 @@ pub fn valid_var_name(s: &wstr) -> bool {
     !s.is_empty() && s.chars().all(valid_var_name_char)
 }
 
-/// Get the absolute path to the fish executable itself
-pub fn get_executable_path(argv0: impl AsRef<Path>) -> PathBuf {
-    if let Ok(path) = std::env::current_exe() {
-        if path.exists() {
-            return path;
-        }
-
-        // When /proc/self/exe points to a file that was deleted (or overwritten on update!)
-        // then linux adds a " (deleted)" suffix.
-        // If that's not a valid path, let's remove that awkward suffix.
-        if path.as_os_str().as_bytes().ends_with(b" (deleted)") {
-            return path;
-        }
-
-        if let (Some(filename), Some(parent)) = (path.file_name(), path.parent()) {
-            if let Some(filename) = filename.to_str() {
-                let corrected_filename = OsStr::new(filename.strip_suffix(" (deleted)").unwrap());
-                return parent.join(corrected_filename);
-            }
-        }
-        return path;
-    }
-    argv0.as_ref().to_owned()
-}
-
 /// A wrapper around Cell which supports modifying the contents, scoped to a region of code.
 /// This provides a somewhat nicer API than ScopedRefCell because you can directly modify the value,
 /// instead of requiring an accessor function which returns a mutable reference to a field.
@@ -2090,8 +1990,349 @@ macro_rules! env_stack_set_from_env {
             $vars.set_one(
                 L!($var_name),
                 $crate::env::EnvMode::GLOBAL,
-                $crate::common::str2wcstring(var.as_bytes()),
+                $crate::common::bytes2wcstring(var.as_bytes()),
             );
         }
     }};
 }
+
+/// The highest character number of character to try and escape.
+#[cfg(test)]
+pub const ESCAPE_TEST_CHAR: usize = 4000;
+
+#[cfg(test)]
+mod tests {
+    use super::{
+        ENCODE_DIRECT_BASE, ENCODE_DIRECT_END, ESCAPE_TEST_CHAR, EscapeFlags, EscapeStringStyle,
+        ScopeGuard, ScopedCell, ScopedRefCell, UnescapeStringStyle, bytes2wcstring, escape_string,
+        truncate_at_nul, unescape_string, wcs2bytes,
+    };
+    use crate::util::{get_rng_seed, get_seeded_rng};
+    use crate::wchar::{L, WString, wstr};
+    use rand::{Rng, RngCore};
+
+    #[test]
+    fn test_escape_string() {
+        let regex = |input| escape_string(input, EscapeStringStyle::Regex);
+
+        // plain text should not be needlessly escaped
+        assert_eq!(regex(L!("hello world!")), L!("hello world!"));
+
+        // all the following are intended to be ultimately matched literally - even if they
+        // don't look like that's the intent - so we escape them.
+        assert_eq!(regex(L!(".ext")), L!("\\.ext"));
+        assert_eq!(regex(L!("{word}")), L!("\\{word\\}"));
+        assert_eq!(regex(L!("hola-mundo")), L!("hola\\-mundo"));
+        assert_eq!(
+            regex(L!("$17.42 is your total?")),
+            L!("\\$17\\.42 is your total\\?")
+        );
+        assert_eq!(
+            regex(L!("not really escaped\\?")),
+            L!("not really escaped\\\\\\?")
+        );
+    }
+
+    #[test]
+    pub fn test_unescape_sane() {
+        const TEST_CASES: &[(&wstr, &wstr)] = &[
+            (L!("abcd"), L!("abcd")),
+            (L!("'abcd'"), L!("abcd")),
+            (L!("'abcd\\n'"), L!("abcd\\n")),
+            (L!("\"abcd\\n\""), L!("abcd\\n")),
+            (L!("\"abcd\\n\""), L!("abcd\\n")),
+            (L!("\\143"), L!("c")),
+            (L!("'\\143'"), L!("\\143")),
+            (L!("\\n"), L!("\n")), // \n normally becomes newline
+        ];
+
+        for (input, expected) in TEST_CASES {
+            let Some(output) = unescape_string(input, UnescapeStringStyle::default()) else {
+                panic!("Failed to unescape string {input:?}");
+            };
+
+            assert_eq!(
+                output, *expected,
+                "In unescaping {input:?}, expected {expected:?} but got {output:?}\n"
+            );
+        }
+    }
+
+    #[test]
+    fn test_escape_var() {
+        const TEST_CASES: &[(&wstr, &wstr)] = &[
+            (L!(" a"), L!("_20_a")),
+            (L!("a B "), L!("a_20_42_20_")),
+            (L!("a b "), L!("a_20_b_20_")),
+            (L!(" B"), L!("_20_42_")),
+            (L!(" f"), L!("_20_f")),
+            (L!(" 1"), L!("_20_31_")),
+            (L!("a\nghi_"), L!("a_0A_ghi__")),
+        ];
+
+        for (input, expected) in TEST_CASES {
+            let output = escape_string(input, EscapeStringStyle::Var);
+
+            assert_eq!(
+                output, *expected,
+                "In escaping {input:?} with style var, expected {expected:?} but got {output:?}\n"
+            );
+        }
+    }
+
+    fn escape_test(escape_style: EscapeStringStyle, unescape_style: UnescapeStringStyle) {
+        let seed: u128 = 92348567983274852905629743984572;
+        let mut rng = get_seeded_rng(seed);
+
+        let mut random_string = WString::new();
+        let mut escaped_string;
+        for _ in 0..(ESCAPE_TEST_COUNT as u32) {
+            random_string.clear();
+            let length = rng.gen_range(0..=(2 * ESCAPE_TEST_LENGTH));
+            for _ in 0..length {
+                random_string
+                    .push(char::from_u32((rng.next_u32() % ESCAPE_TEST_CHAR as u32) + 1).unwrap());
+            }
+
+            escaped_string = escape_string(&random_string, escape_style);
+            let Some(unescaped_string) = unescape_string(&escaped_string, unescape_style) else {
+                let slice = escaped_string.as_char_slice();
+                panic!("Failed to unescape string {slice:?}");
+            };
+            assert_eq!(
+                random_string, unescaped_string,
+                "Escaped and then unescaped string {random_string:?}, but got back a different string {unescaped_string:?}. The intermediate escape looked like {escaped_string:?}."
+            );
+        }
+    }
+
+    #[test]
+    fn test_escape_random_script() {
+        escape_test(EscapeStringStyle::default(), UnescapeStringStyle::default());
+    }
+
+    #[test]
+    fn test_escape_random_var() {
+        escape_test(EscapeStringStyle::Var, UnescapeStringStyle::Var);
+    }
+
+    #[test]
+    fn test_escape_random_url() {
+        escape_test(EscapeStringStyle::Url, UnescapeStringStyle::Url);
+    }
+
+    #[test]
+    fn test_escape_no_printables() {
+        // Verify that ESCAPE_NO_PRINTABLES also escapes backslashes so we don't regress on issue #3892.
+        let random_string = L!("line 1\\n\nline 2").to_owned();
+        let escaped_string = escape_string(
+            &random_string,
+            EscapeStringStyle::Script(EscapeFlags::NO_PRINTABLES | EscapeFlags::NO_QUOTED),
+        );
+        let Some(unescaped_string) =
+            unescape_string(&escaped_string, UnescapeStringStyle::default())
+        else {
+            panic!("Failed to unescape string <{escaped_string}>");
+        };
+
+        assert_eq!(
+            random_string, unescaped_string,
+            "Escaped and then unescaped string '{random_string}', but got back a different string '{unescaped_string}'"
+        );
+    }
+
+    /// The number of tests to run.
+    const ESCAPE_TEST_COUNT: usize = 20_000;
+    /// The average length of strings to unescape.
+    const ESCAPE_TEST_LENGTH: usize = 100;
+
+    /// Helper to convert a narrow string to a sequence of hex digits.
+    fn bytes2hex(input: &[u8]) -> String {
+        let mut output = "".to_string();
+        for byte in input {
+            output += &format!("0x{:2X} ", *byte);
+        }
+        output
+    }
+
+    /// Test wide/narrow conversion by creating random strings and verifying that the original
+    /// string comes back through double conversion.
+    #[test]
+    fn test_convert() {
+        let seed = get_rng_seed();
+        let mut rng = get_seeded_rng(seed);
+        let mut origin = Vec::new();
+
+        for _ in 0..ESCAPE_TEST_COUNT {
+            let length: usize = rng.gen_range(0..=(2 * ESCAPE_TEST_LENGTH));
+            origin.resize(length, 0);
+            rng.fill_bytes(&mut origin);
+
+            let w = bytes2wcstring(&origin[..]);
+            let n = wcs2bytes(&w);
+            assert_eq!(
+                origin,
+                n,
+                "Conversion cycle of string:\n{:4} chars: {}\n\
+                    produced different string:\n\
+                    {:4} chars: {}\n
+                    Use this seed to reproduce: {}",
+                origin.len(),
+                &bytes2hex(&origin),
+                n.len(),
+                &bytes2hex(&n),
+                seed,
+            );
+        }
+    }
+
+    /// Verify correct behavior with embedded nulls.
+    #[test]
+    fn test_convert_nulls() {
+        let input = L!("AAA\0BBB");
+        let out_str = wcs2bytes(input);
+        assert_eq!(
+            input.chars().collect::<Vec<_>>(),
+            std::str::from_utf8(&out_str)
+                .unwrap()
+                .chars()
+                .collect::<Vec<_>>()
+        );
+
+        let out_wstr = bytes2wcstring(&out_str);
+        assert_eq!(input, &out_wstr);
+    }
+
+    /// Verify that ASCII narrow->wide conversions are correct.
+    #[test]
+    fn test_convert_ascii() {
+        let mut s = vec![b'\0'; 4096];
+        for (i, c) in s.iter_mut().enumerate() {
+            *c = u8::try_from(i % 10).unwrap() + b'0';
+        }
+
+        // Test a variety of alignments.
+        for left in 0..16 {
+            for right in 0..16 {
+                let len = s.len() - left - right;
+                let input = &s[left..left + len];
+                let wide = bytes2wcstring(input);
+                let narrow = wcs2bytes(&wide);
+                assert_eq!(narrow, input);
+            }
+        }
+
+        // Put some non-ASCII bytes in and ensure it all still works.
+        for i in 0..s.len() {
+            let saved = s[i];
+            s[i] = 0xF7;
+            assert_eq!(wcs2bytes(&bytes2wcstring(&s)), s);
+            s[i] = saved;
+        }
+    }
+
+    /// fish uses the private-use range to encode bytes that are not valid UTF-8.
+    /// If the input decodes to these private-use codepoints,
+    /// then fish should also use the direct encoding for those bytes.
+    /// Verify that characters in the private use area are correctly round-tripped. See #7723.
+    #[test]
+    fn test_convert_private_use() {
+        for c in ENCODE_DIRECT_BASE..ENCODE_DIRECT_END {
+            // A `char` represents an Unicode scalar value, which takes up at most 4 bytes when encoded in UTF-8.
+            // TODO MSRV(1.92?) replace 4 by `char::MAX_LEN_UTF8` once that's available in our MSRV.
+            // https://doc.rust-lang.org/std/primitive.char.html#associatedconstant.MAX_LEN_UTF8
+            let mut converted = [0_u8; 4];
+            let s = c.encode_utf8(&mut converted).as_bytes();
+            // Ask fish to decode this via bytes2wcstring.
+            // bytes2wcstring should notice that the decoded form collides with its private use
+            // and encode it directly.
+            let ws = bytes2wcstring(s);
+
+            // Each byte should be encoded directly, and round tripping should work.
+            assert_eq!(ws.len(), s.len());
+            assert_eq!(wcs2bytes(&ws), s);
+        }
+    }
+
+    #[test]
+    fn test_scoped_cell() {
+        let cell = ScopedCell::new(42);
+
+        {
+            let _guard = cell.scoped_mod(|x| *x += 1);
+            assert_eq!(cell.get(), 43);
+        }
+
+        assert_eq!(cell.get(), 42);
+    }
+
+    #[test]
+    fn test_scoped_refcell() {
+        #[derive(Debug, PartialEq, Clone)]
+        struct Data {
+            x: i32,
+            y: i32,
+        }
+
+        let cell = ScopedRefCell::new(Data { x: 1, y: 2 });
+
+        {
+            let _guard = cell.scoped_set(10, |d| &mut d.x);
+            assert_eq!(cell.borrow().x, 10);
+        }
+        assert_eq!(cell.borrow().x, 1);
+
+        {
+            let _guard = cell.scoped_replace(Data { x: 42, y: 99 });
+            assert_eq!(*cell.borrow(), Data { x: 42, y: 99 });
+        }
+        assert_eq!(*cell.borrow(), Data { x: 1, y: 2 });
+    }
+
+    #[test]
+    fn test_scope_guard() {
+        let relaxed = std::sync::atomic::Ordering::Relaxed;
+        let counter = std::sync::atomic::AtomicUsize::new(0);
+        {
+            let guard = ScopeGuard::new(123, |arg| {
+                assert_eq!(arg, 123);
+                counter.fetch_add(1, relaxed);
+            });
+            assert_eq!(counter.load(relaxed), 0);
+            std::mem::drop(guard);
+            assert_eq!(counter.load(relaxed), 1);
+        }
+        // commit also invokes the callback.
+        {
+            let guard = ScopeGuard::new(123, |arg| {
+                assert_eq!(arg, 123);
+                counter.fetch_add(1, relaxed);
+            });
+            assert_eq!(counter.load(relaxed), 1);
+            ScopeGuard::commit(guard);
+            assert_eq!(counter.load(relaxed), 2);
+        }
+    }
+
+    #[test]
+    fn test_truncate_at_nul() {
+        assert_eq!(truncate_at_nul(L!("abc\0def")), L!("abc"));
+        assert_eq!(truncate_at_nul(L!("abc")), L!("abc"));
+        assert_eq!(truncate_at_nul(L!("\0abc")), L!(""));
+    }
+}
+
+#[cfg(feature = "benchmark")]
+#[cfg(test)]
+mod bench {
+    extern crate test;
+    use crate::common::bytes2wcstring;
+    use test::Bencher;
+
+    #[bench]
+    fn bench_convert_ascii(b: &mut Bencher) {
+        let s: [u8; 128 * 1024] = std::array::from_fn(|i| b'0' + u8::try_from(i % 10).unwrap());
+        b.bytes = u64::try_from(s.len()).unwrap();
+        b.iter(|| bytes2wcstring(&s));
+    }
+}
