@@ -1,20 +1,19 @@
-use super::environment_impl::{
-    colon_split, uvars, EnvMutex, EnvMutexGuard, EnvScopedImpl, EnvStackImpl, ModResult,
-    UVAR_SCOPE_IS_GLOBAL,
-};
 use super::ElectricVar;
-use crate::abbrs::{abbrs_get_set, Abbreviation, Position};
+use super::environment_impl::{
+    EnvMutex, EnvMutexGuard, EnvScopedImpl, EnvStackImpl, ModResult, UVAR_SCOPE_IS_GLOBAL,
+    colon_split, uvars,
+};
+use crate::abbrs::{Abbreviation, Position, abbrs_get_set};
 use crate::builtins::shared::{BuiltinResult, SUCCESS};
-use crate::common::{str2wcstring, unescape_string, wcs2zstring, UnescapeStringStyle};
+use crate::common::{UnescapeStringStyle, bytes2wcstring, unescape_string, wcs2zstring};
 use crate::env::config_paths::ConfigPaths;
 use crate::env::{EnvMode, EnvVar, Statuses};
 use crate::env_dispatch::{env_dispatch_init, env_dispatch_var_change};
 use crate::event::Event;
 use crate::flog::FLOG;
 use crate::global_safety::RelaxedAtomicBool;
-use crate::input::{init_input, FISH_BIND_MODE_VAR};
-use crate::libc::{stdout_stream, C_PATH_BSHELL, _PATH_BSHELL};
-use crate::nix::{geteuid, getpid, isatty};
+use crate::input::{FISH_BIND_MODE_VAR, init_input};
+use crate::nix::{geteuid, getpid};
 use crate::null_terminated_array::OwningNullTerminatedArray;
 use crate::path::{
     path_emit_config_directory_messages, path_get_cache, path_get_config, path_get_data,
@@ -26,36 +25,30 @@ use crate::universal_notifier::default_notifier;
 use crate::wchar::prelude::*;
 use crate::wcstringutil::join_strings;
 use crate::wutil::{fish_wcstol, wgetcwd, wgettext};
-use std::sync::atomic::Ordering;
 
-use libc::{c_int, confstr, uid_t, STDOUT_FILENO, _IONBF};
+use libc::{c_int, uid_t};
 use once_cell::sync::{Lazy, OnceCell};
 use std::collections::HashMap;
 use std::ffi::CStr;
-use std::io::Write;
 use std::mem::MaybeUninit;
 use std::os::unix::prelude::*;
+use std::path::PathBuf;
 use std::sync::Arc;
 
 /// Set when a universal variable has been modified but not yet been written to disk via sync().
 static UVARS_LOCALLY_MODIFIED: RelaxedAtomicBool = RelaxedAtomicBool::new(false);
 
 /// Return values for `EnvStack::set()`.
-#[derive(Clone, Copy, Debug, Eq, PartialEq)]
+#[derive(Clone, Copy, Debug, Default, Eq, PartialEq)]
 pub enum EnvStackSetResult {
-    Ok,       // The variable was set successfully.
+    #[default]
+    Ok, // The variable was set successfully.
     Perm,     // The variable is read-only.
     Scope,    // Variable cannot be set in the given scope.
     Invalid,  // The variable's value is invalid (e.g. umask).
     NotFound, // The variable was not found (only possible when removing a variable).
 }
 
-impl Default for EnvStackSetResult {
-    fn default() -> Self {
-        EnvStackSetResult::Ok
-    }
-}
-
 impl From<EnvStackSetResult> for BuiltinResult {
     fn from(r: EnvStackSetResult) -> Self {
         match r {
@@ -135,8 +128,7 @@ pub struct EnvDyn {
 }
 
 impl EnvDyn {
-    // Exposed for testing.
-    pub fn new(inner: Box<dyn Environment + Send + Sync>) -> Self {
+    fn new(inner: Box<dyn Environment + Send + Sync>) -> Self {
         Self { inner }
     }
 }
@@ -442,13 +434,9 @@ fn get_hostname_identifier() -> Option<WString> {
     let mut b = [0 as libc::c_char; HOSTNAME_LEN + 1];
     if unsafe { libc::gethostname(b.as_mut_ptr(), b.len()) } == 0 {
         let cstr = unsafe { CStr::from_ptr(b.as_ptr()) };
-        let res = str2wcstring(cstr.to_bytes());
+        let res = bytes2wcstring(cstr.to_bytes());
 
-        if res.is_empty() {
-            None
-        } else {
-            Some(res)
-        }
+        if res.is_empty() { None } else { Some(res) }
     } else {
         None
     }
@@ -519,7 +507,7 @@ fn setup_user(vars: &EnvStack) {
                         vars.set_one(
                             L!("HOME"),
                             EnvMode::GLOBAL | EnvMode::EXPORT,
-                            str2wcstring(s.to_bytes()),
+                            bytes2wcstring(s.to_bytes()),
                         );
                     } else {
                         vars.set_empty(L!("HOME"), EnvMode::GLOBAL | EnvMode::EXPORT);
@@ -544,7 +532,7 @@ fn setup_user(vars: &EnvStack) {
     if retval == 0 && !result.is_null() {
         let userinfo = unsafe { userinfo.assume_init() };
         let s = unsafe { CStr::from_ptr(userinfo.pw_name) };
-        let uname = str2wcstring(s.to_bytes());
+        let uname = bytes2wcstring(s.to_bytes());
         vars.set_one(L!("USER"), EnvMode::GLOBAL | EnvMode::EXPORT, uname);
         // Only change $HOME if it's empty, so we allow e.g. `HOME=(mktemp -d)`.
         // This is okay with common `su` and `sudo` because they set $HOME.
@@ -554,7 +542,7 @@ fn setup_user(vars: &EnvStack) {
                 vars.set_one(
                     L!("HOME"),
                     EnvMode::GLOBAL | EnvMode::EXPORT,
-                    str2wcstring(s.to_bytes()),
+                    bytes2wcstring(s.to_bytes()),
                 );
             } else {
                 // We cannot get $HOME. This triggers warnings for history and config.fish already,
@@ -569,26 +557,25 @@ fn setup_user(vars: &EnvStack) {
 }
 
 pub(crate) static FALLBACK_PATH: Lazy<&[WString]> = Lazy::new(|| {
-    use crate::libc::_CS_PATH;
-    // _CS_PATH: colon-separated paths to find POSIX utilities
-    let buf_size = unsafe { confstr(_CS_PATH(), std::ptr::null_mut(), 0) };
-    Box::leak(
-        (if buf_size > 0 {
-            let mut buf = vec![b'\0' as libc::c_char; buf_size];
-            unsafe { confstr(_CS_PATH(), buf.as_mut_ptr(), buf_size) };
-            let buf = buf;
-            // safety: buf should contain a null-byte, and is not mutable unless we move ownership
-            let cstr = unsafe { CStr::from_ptr(buf.as_ptr()) };
-            colon_split(&[str2wcstring(cstr.to_bytes())])
-        } else {
-            vec![
-                WString::from_str(env!("PREFIX")) + L!("/bin"),
-                L!("/usr/bin").to_owned(),
-                L!("/bin").to_owned(),
-            ]
-        })
-        .into_boxed_slice(),
-    )
+    // _CS_PATH: colon-separated paths to find POSIX utilities. Same as USER_CS_PATH.
+    let cs_path = libc::_CS_PATH;
+
+    let buf_size = unsafe { libc::confstr(cs_path, std::ptr::null_mut(), 0) };
+    let paths: Vec<WString> = if buf_size > 0 {
+        let mut buf = vec![b'\0' as libc::c_char; buf_size];
+        unsafe { libc::confstr(cs_path, buf.as_mut_ptr(), buf_size) };
+        let buf = buf;
+        // safety: buf should contain a null-byte, and is not mutable unless we move ownership
+        let cstr = unsafe { CStr::from_ptr(buf.as_ptr()) };
+        colon_split(&[bytes2wcstring(cstr.to_bytes())])
+    } else {
+        vec![
+            WString::from_str(env!("PREFIX")) + L!("/bin"),
+            WString::from_str("/usr/bin"),
+            WString::from_str("/bin"),
+        ]
+    };
+    Box::leak(paths.into_boxed_slice())
 });
 
 /// Make sure the PATH variable contains something.
@@ -612,7 +599,7 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
     let vars = EnvStack::globals();
 
     let env_iter: Vec<_> = std::env::vars_os()
-        .map(|(k, v)| (str2wcstring(k.as_bytes()), str2wcstring(v.as_bytes())))
+        .map(|(k, v)| (bytes2wcstring(k.as_bytes()), bytes2wcstring(v.as_bytes())))
         .collect();
 
     let mut inherited_vars = HashMap::new();
@@ -646,45 +633,27 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
     setup_user(vars);
 
     if let Some(paths) = paths {
-        #[cfg(feature = "embed-data")]
-        vars.set_empty(FISH_DATADIR_VAR, EnvMode::GLOBAL);
-        #[cfg(not(feature = "embed-data"))]
-        {
-            let datadir = str2wcstring(paths.data.as_os_str().as_bytes());
-            vars.set_one(FISH_DATADIR_VAR, EnvMode::GLOBAL, datadir.clone());
-
-            if default_paths {
-                let mut scstr = datadir;
-                // This is generated by PathBuf.join() everywhere currently
-                assert!(!scstr.ends_with("/"));
-                scstr.push_str("/functions");
-                vars.set_one(L!("fish_function_path"), EnvMode::GLOBAL, scstr);
-            }
-        }
-
-        vars.set_one(
-            FISH_SYSCONFDIR_VAR,
-            EnvMode::GLOBAL,
-            str2wcstring(paths.sysconf.as_os_str().as_bytes()),
-        );
-
-        #[cfg(feature = "embed-data")]
-        vars.set_empty(FISH_HELPDIR_VAR, EnvMode::GLOBAL);
-        #[cfg(not(feature = "embed-data"))]
-        vars.set_one(
-            FISH_HELPDIR_VAR,
-            EnvMode::GLOBAL,
-            str2wcstring(paths.doc.as_os_str().as_bytes()),
-        );
-        if let Some(bp) = &paths.bin {
-            vars.set_one(
-                FISH_BIN_DIR,
+        let set_path = |key: &wstr, maybe_path: Option<&PathBuf>| {
+            vars.set(
+                key,
                 EnvMode::GLOBAL,
-                str2wcstring(bp.as_os_str().as_bytes()),
+                maybe_path
+                    .map(|path| vec![bytes2wcstring(path.as_os_str().as_bytes())])
+                    .unwrap_or_default(),
             );
-        } else {
-            vars.set_empty(FISH_BIN_DIR, EnvMode::GLOBAL);
         };
+
+        set_path(FISH_SYSCONFDIR_VAR, Some(&paths.sysconf));
+        set_path(FISH_BIN_DIR, paths.bin.as_ref());
+
+        let datadir = paths.data.as_ref();
+        set_path(FISH_DATADIR_VAR, datadir);
+        if !cfg!(feature = "embed-data") && default_paths {
+            let mut scstr = datadir.unwrap().clone();
+            scstr.push("functions");
+            set_path(L!("fish_function_path"), Some(&scstr));
+        }
+        set_path(FISH_HELPDIR_VAR, paths.doc.as_ref());
     }
 
     let user_config_dir = path_get_config();
@@ -719,7 +688,7 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
     vars.set_one(L!("CMD_DURATION"), EnvMode::UNEXPORT, "0".into());
 
     // Set up the version variable.
-    let version = str2wcstring(crate::BUILD_VERSION.as_bytes());
+    let version = bytes2wcstring(crate::BUILD_VERSION.as_bytes());
     vars.set_one(L!("version"), EnvMode::GLOBAL, version.clone());
     vars.set_one(L!("FISH_VERSION"), EnvMode::GLOBAL, version);
 
@@ -736,7 +705,7 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
         let nshlvl_str = if let Some(shlvl_var) = std::env::var_os("SHLVL") {
             // TODO: Figure out how to handle invalid numbers better. Shouldn't we issue a
             // diagnostic?
-            match fish_wcstol(&str2wcstring(shlvl_var.as_os_str().as_bytes())) {
+            match fish_wcstol(&bytes2wcstring(shlvl_var.as_os_str().as_bytes())) {
                 Ok(shlvl_i) if shlvl_i >= 0 => (shlvl_i + 1).to_wstring(),
                 _ => L!("1").to_owned(),
             }
@@ -750,7 +719,7 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
             vars.set_one(
                 L!("SHLVL"),
                 EnvMode::GLOBAL | EnvMode::EXPORT,
-                str2wcstring(shlvl_var.as_os_str().as_bytes()),
+                bytes2wcstring(shlvl_var.as_os_str().as_bytes()),
             );
         }
     }
@@ -764,7 +733,7 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
     // (see #7636)
     let incoming_pwd_cstr = std::env::var_os("PWD");
     let incoming_pwd = incoming_pwd_cstr
-        .map(|s| str2wcstring(s.as_os_str().as_bytes()))
+        .map(|s| bytes2wcstring(s.as_os_str().as_bytes()))
         .unwrap_or_default();
     if !incoming_pwd.is_empty()
         && incoming_pwd.char_at(0) == '/'
@@ -778,10 +747,14 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
     // Initialize termsize variables.
     let termsize = termsize::SHARED_CONTAINER.initialize(vars as &dyn Environment);
     if vars.get_unless_empty(L!("COLUMNS")).is_none() {
-        vars.set_one(L!("COLUMNS"), EnvMode::GLOBAL, termsize.width.to_wstring());
+        vars.set_one(
+            L!("COLUMNS"),
+            EnvMode::GLOBAL,
+            termsize.width().to_wstring(),
+        );
     }
     if vars.get_unless_empty(L!("LINES")).is_none() {
-        vars.set_one(L!("LINES"), EnvMode::GLOBAL, termsize.height.to_wstring());
+        vars.set_one(L!("LINES"), EnvMode::GLOBAL, termsize.height().to_wstring());
     }
 
     // Set fish_bind_mode to "default".
@@ -856,15 +829,94 @@ pub fn env_init(paths: Option<&ConfigPaths>, do_uvars: bool, default_paths: bool
     }
 }
 
-/// Various things we need to initialize at run-time that don't really fit any of the other init
-/// routines.
-pub fn misc_init() {
-    // If stdout is open on a tty ensure stdio is unbuffered. That's because those functions might
-    // be intermixed with `write()` calls and we need to ensure the writes are not reordered. See
-    // issue #3748.
-    if isatty(STDOUT_FILENO) {
-        let _ = std::io::stdout().flush();
-        unsafe { libc::setvbuf(stdout_stream(), std::ptr::null_mut(), _IONBF, 0) };
+#[cfg(test)]
+mod tests {
+    use super::{EnvMode, EnvStack, Environment};
+    use crate::tests::prelude::*;
+    use crate::wchar::prelude::*;
+
+    #[test]
+    #[serial]
+    fn test_env_snapshot() {
+        let _cleanup = test_init();
+        std::fs::create_dir_all("test/fish_env_snapshot_test/").unwrap();
+        let parser = TestParser::new();
+        let vars = parser.vars();
+        parser.pushd("test/fish_env_snapshot_test/");
+        vars.push(true);
+        let before_pwd = vars.get(L!("PWD")).unwrap().as_string();
+        vars.set_one(
+            L!("test_env_snapshot_var"),
+            EnvMode::default(),
+            L!("before").to_owned(),
+        );
+        let snapshot = vars.snapshot();
+        vars.set_one(L!("PWD"), EnvMode::default(), L!("/newdir").to_owned());
+        vars.set_one(
+            L!("test_env_snapshot_var"),
+            EnvMode::default(),
+            L!("after").to_owned(),
+        );
+        vars.set_one(
+            L!("test_env_snapshot_var_2"),
+            EnvMode::default(),
+            L!("after").to_owned(),
+        );
+
+        // vars should be unaffected by the snapshot
+        assert_eq!(vars.get(L!("PWD")).unwrap().as_string(), L!("/newdir"));
+        assert_eq!(
+            vars.get(L!("test_env_snapshot_var")).unwrap().as_string(),
+            L!("after")
+        );
+        assert_eq!(
+            vars.get(L!("test_env_snapshot_var_2")).unwrap().as_string(),
+            L!("after")
+        );
+
+        // snapshot should have old values of vars
+        assert_eq!(snapshot.get(L!("PWD")).unwrap().as_string(), before_pwd);
+        assert_eq!(
+            snapshot
+                .get(L!("test_env_snapshot_var"))
+                .unwrap()
+                .as_string(),
+            L!("before")
+        );
+        assert_eq!(snapshot.get(L!("test_env_snapshot_var_2")), None);
+
+        // snapshots see global var changes except for perproc like PWD
+        vars.set_one(
+            L!("test_env_snapshot_var_3"),
+            EnvMode::GLOBAL,
+            L!("reallyglobal").to_owned(),
+        );
+        assert_eq!(
+            vars.get(L!("test_env_snapshot_var_3")).unwrap().as_string(),
+            L!("reallyglobal")
+        );
+        assert_eq!(
+            snapshot
+                .get(L!("test_env_snapshot_var_3"))
+                .unwrap()
+                .as_string(),
+            L!("reallyglobal")
+        );
+
+        vars.pop();
+        parser.popd();
+    }
+
+    // Can't push/pop from globals.
+    #[test]
+    #[should_panic]
+    fn test_no_global_push() {
+        EnvStack::globals().push(true);
+    }
+
+    #[test]
+    #[should_panic]
+    fn test_no_global_pop() {
+        EnvStack::globals().pop();
     }
-    _PATH_BSHELL.store(unsafe { C_PATH_BSHELL().cast_mut() }, Ordering::SeqCst);
 }
