@@ -1,20 +1,21 @@
 use crate::common::wcs2zstring;
 use crate::env::{
-    is_read_only, ElectricVar, EnvMode, EnvStackSetResult, EnvVar, EnvVarFlags, Statuses, VarTable,
-    ELECTRIC_VARIABLES, PATH_ARRAY_SEP,
+    ELECTRIC_VARIABLES, ElectricVar, EnvMode, EnvStackSetResult, EnvVar, EnvVarFlags,
+    PATH_ARRAY_SEP, Statuses, VarTable, is_read_only,
 };
 use crate::env_universal_common::EnvUniversal;
 use crate::flog::FLOG;
 use crate::global_safety::RelaxedAtomicBool;
-use crate::history::{history_session_id_from_var, History};
+use crate::history::{History, history_session_id_from_var};
 use crate::kill::kill_entries;
+use crate::nix::umask;
 use crate::null_terminated_array::OwningNullTerminatedArray;
 use crate::reader::{commandline_get_state, reader_status_count};
 use crate::threads::{is_forked_child, is_main_thread};
 use crate::wchar::prelude::*;
 use crate::wutil::fish_wcstol_radix;
 
-use lazy_static::lazy_static;
+use once_cell::sync::Lazy;
 use std::cell::{RefCell, UnsafeCell};
 use std::collections::HashSet;
 use std::ffi::CString;
@@ -26,7 +27,7 @@ use std::ops::{Deref, DerefMut};
 use portable_atomic::AtomicU64;
 #[cfg(target_has_atomic = "64")]
 use std::sync::atomic::AtomicU64;
-use std::sync::{atomic::Ordering, Arc, Mutex, MutexGuard};
+use std::sync::{Arc, Mutex, MutexGuard, atomic::Ordering};
 
 /// Getter for universal variables.
 /// This is typically initialized in env_init(), and is considered empty before then.
@@ -54,7 +55,7 @@ pub fn colon_split<T: AsRef<wstr>>(val: &[T]) -> Vec<WString> {
 
 /// Return true if a variable should become a path variable by default. See #436.
 fn variable_should_auto_pathvar(name: &wstr) -> bool {
-    name.ends_with("PATH")
+    name.ends_with("PATH") || name == "LANGUAGE"
 }
 
 /// We cache our null-terminated export list. However an exported variable may change for lots of
@@ -84,8 +85,7 @@ fn set_umask(list_val: &[WString]) -> EnvStackSetResult {
         return EnvStackSetResult::Invalid;
     }
     // Do not actually create a umask variable. On env_stack_t::get() it will be calculated.
-    // SAFETY: umask cannot fail.
-    unsafe { libc::umask(mask as libc::mode_t) };
+    umask(mask as libc::mode_t);
     EnvStackSetResult::Ok
 }
 
@@ -201,16 +201,28 @@ impl EnvNode {
     }
 }
 
+// RefCell except we promise it can be used as Sync.
+// Safety: in order to do anything with this, the caller must be holding ENV_LOCK.
+struct EnvNodeSyncCell(RefCell<EnvNode>);
+
+impl EnvNodeSyncCell {
+    fn new(node: EnvNode) -> Self {
+        Self(RefCell::new(node))
+    }
+}
+
+unsafe impl Sync for EnvNodeSyncCell {}
+
 /// EnvNodeRef is a reference to an EnvNode. It may be shared between different environments.
-/// The type Arc<RefCell<...>> may look suspicious, but all accesses to the EnvNode are protected by a global lock.
+/// All accesses to the EnvNode are protected by a global lock.
 #[derive(Clone)]
-struct EnvNodeRef(Arc<RefCell<EnvNode>>);
+struct EnvNodeRef(Arc<EnvNodeSyncCell>);
 
 impl Deref for EnvNodeRef {
     type Target = RefCell<EnvNode>;
 
     fn deref(&self) -> &Self::Target {
-        &self.0
+        &self.0.0
     }
 }
 
@@ -219,7 +231,7 @@ impl EnvNodeRef {
         // Accesses are protected by the global lock.
         #[allow(unknown_lints)]
         #[allow(clippy::arc_with_non_send_sync)]
-        EnvNodeRef(Arc::new(RefCell::new(EnvNode {
+        EnvNodeRef(Arc::new(EnvNodeSyncCell::new(EnvNode {
             env: VarTable::new(),
             new_scope: is_new_scope,
             export_gen: 0,
@@ -248,9 +260,6 @@ impl EnvNodeRef {
     }
 }
 
-// Safety: in order to do anything with an EnvNodeRef, the caller must be holding ENV_LOCK.
-unsafe impl Sync for EnvNodeRef {}
-
 /// Helper to iterate over a chain of EnvNodeRefs.
 struct EnvNodeIter {
     current: Option<EnvNodeRef>,
@@ -276,10 +285,7 @@ impl Iterator for EnvNodeIter {
     }
 }
 
-lazy_static! {
-    // All accesses to the EnvNode are protected by a global lock.
-    static ref GLOBAL_NODE: EnvNodeRef = EnvNodeRef::new(false, None);
-}
+static GLOBAL_NODE: Lazy<EnvNodeRef> = Lazy::new(|| EnvNodeRef::new(false, None));
 
 /// Recursive helper to snapshot a series of nodes.
 fn copy_node_chain(node: &EnvNodeRef) -> EnvNodeRef {
@@ -293,7 +299,7 @@ fn copy_node_chain(node: &EnvNodeRef) -> EnvNodeRef {
     };
     #[allow(unknown_lints)]
     #[allow(clippy::arc_with_non_send_sync)]
-    EnvNodeRef(Arc::new(RefCell::new(new_node)))
+    EnvNodeRef(Arc::new(EnvNodeSyncCell::new(new_node)))
 }
 
 /// A struct wrapping up parser-local variables. These are conceptually variables that differ in
@@ -402,10 +408,9 @@ impl EnvScopedImpl {
             // value. Thus we have to call it twice, to reset the value. The env_lock protects
             // against races. Guess what the umask is; if we guess right we don't need to reset it.
             let guess: libc::mode_t = 0o022;
-            // Safety: umask cannot error.
-            let res: libc::mode_t = unsafe { libc::umask(guess) };
+            let res: libc::mode_t = umask(guess);
             if res != guess {
-                unsafe { libc::umask(res) };
+                umask(res);
             }
             Some(EnvVar::new_from_name(L!("umask"), sprintf!("0%0.3o", res)))
         } else {
@@ -580,8 +585,8 @@ impl EnvScopedImpl {
 
         let mut cursor = self.export_array_generations.iter().fuse();
         let mut mismatch = false;
-        self.enumerate_generations(|gen| {
-            if cursor.next().cloned() != Some(gen) {
+        self.enumerate_generations(|r#gen| {
+            if cursor.next().cloned() != Some(r#gen) {
                 mismatch = true;
             }
         });
@@ -619,12 +624,12 @@ impl EnvScopedImpl {
         Self::get_exported(&self.globals, &mut vals);
         Self::get_exported(&self.locals, &mut vals);
 
-        let uni = uvars().get_names(true, false);
-        for key in uni {
-            let var = uvars().get(&key).unwrap();
-            // Only insert if not already present, as uvars have lowest precedence.
-            // TODO: a longstanding bug is that an unexported local variable will not mask an exported uvar.
-            vals.entry(key).or_insert(var);
+        for (key, var) in uvars().get_table() {
+            if var.exports() {
+                // Only insert if not already present, as uvars have lowest precedence.
+                // TODO: a longstanding bug is that an unexported local variable will not mask an exported uvar.
+                vals.entry(key.clone()).or_insert(var.clone());
+            }
         }
 
         // Dorky way to add our single exported computed variable.
@@ -652,7 +657,7 @@ impl EnvScopedImpl {
 
             // Have to pull this into a local to satisfy the borrow checker.
             let mut generations = std::mem::take(&mut self.export_array_generations);
-            self.enumerate_generations(|gen| generations.push(gen));
+            self.enumerate_generations(|r#gen| generations.push(r#gen));
             self.export_array_generations = generations;
         }
         return self.export_array.as_ref().unwrap().clone();
@@ -763,8 +768,8 @@ impl EnvStackImpl {
                 // This is distinct from the unspecified scope,
                 // which is the global scope if no function exists.
                 let mut node = self.base.locals.clone();
-                while node.next().is_some() {
-                    node = node.next().unwrap();
+                while let Some(next_node) = node.next() {
+                    node = next_node;
                     // The first node that introduces a new scope is ours.
                     // If this doesn't happen, we go on until we've reached the
                     // topmost local scope.
@@ -831,8 +836,8 @@ impl EnvStackImpl {
                 result.status = remove_from_chain(&mut self.base.locals, key);
             } else if query.function {
                 let mut node = self.base.locals.clone();
-                while node.next().is_some() {
-                    node = node.next().unwrap();
+                while let Some(next_node) = node.next() {
+                    node = next_node;
                     if node.borrow().new_scope {
                         break;
                     }
@@ -1134,7 +1139,7 @@ impl<T> EnvMutex<T> {
         }
     }
 
-    pub fn lock(&self) -> EnvMutexGuard<T> {
+    pub fn lock(&self) -> EnvMutexGuard<'_, T> {
         let guard = ENV_LOCK.lock().unwrap();
         // Safety: we have the global lock.
         let value = unsafe { &mut *self.inner.get() };
@@ -1150,23 +1155,29 @@ impl<T> EnvMutex<T> {
 unsafe impl<T> Sync for EnvMutex<T> {}
 unsafe impl<T> Send for EnvMutex<T> {}
 
-#[test]
-fn test_colon_split() {
-    assert_eq!(colon_split(&[L!("foo")]), &[L!("foo")]);
-    assert_eq!(
-        colon_split(&[L!("foo:bar:baz")]),
-        &[L!("foo"), L!("bar"), L!("baz")]
-    );
-    assert_eq!(
-        colon_split(&[L!("foo:bar"), L!("baz")]),
-        &[L!("foo"), L!("bar"), L!("baz")]
-    );
-    assert_eq!(
-        colon_split(&[L!("foo:bar"), L!("baz")]),
-        &[L!("foo"), L!("bar"), L!("baz")]
-    );
-    assert_eq!(
-        colon_split(&[L!("1:"), L!("2:"), L!(":3:")]),
-        &[L!("1"), L!(""), L!("2"), L!(""), L!(""), L!("3"), L!("")]
-    );
+#[cfg(test)]
+mod tests {
+    use super::colon_split;
+    use crate::wchar::prelude::*;
+
+    #[test]
+    fn test_colon_split() {
+        assert_eq!(colon_split(&[L!("foo")]), &[L!("foo")]);
+        assert_eq!(
+            colon_split(&[L!("foo:bar:baz")]),
+            &[L!("foo"), L!("bar"), L!("baz")]
+        );
+        assert_eq!(
+            colon_split(&[L!("foo:bar"), L!("baz")]),
+            &[L!("foo"), L!("bar"), L!("baz")]
+        );
+        assert_eq!(
+            colon_split(&[L!("foo:bar"), L!("baz")]),
+            &[L!("foo"), L!("bar"), L!("baz")]
+        );
+        assert_eq!(
+            colon_split(&[L!("1:"), L!("2:"), L!(":3:")]),
+            &[L!("1"), L!(""), L!("2"), L!(""), L!(""), L!("3"), L!("")]
+        );
+    }
 }
