@@ -1,44 +1,23 @@
-mod config_paths;
+pub mod config_paths;
 pub mod environment;
 mod environment_impl;
 pub mod var;
-pub use config_paths::CONFIG_PATHS;
 
 use crate::common::ToCString;
 pub use environment::*;
-use std::sync::{
-    atomic::{AtomicBool, AtomicUsize},
-    Mutex,
-};
+use std::sync::{Mutex, atomic::AtomicUsize};
 pub use var::*;
 
-/// Limit `read` to 100 MiB (bytes, not wide chars) by default. This can be overridden with the
+/// Limit `read` to 1 GiB (bytes, not wide chars) by default. This can be overridden with the
 /// `fish_read_limit` variable.
-pub const DEFAULT_READ_BYTE_LIMIT: usize = 100 * 1024 * 1024;
+pub const DEFAULT_READ_BYTE_LIMIT: usize = 1024 * 1024 * 1024;
 
 /// The actual `read` limit in effect, defaulting to [`DEFAULT_READ_BYTE_LIMIT`] but overridable
 /// with `$fish_read_limit`.
 pub static READ_BYTE_LIMIT: AtomicUsize = AtomicUsize::new(DEFAULT_READ_BYTE_LIMIT);
 
-/// The curses `cur_term` TERMINAL pointer has been set up.
-pub static CURSES_INITIALIZED: AtomicBool = AtomicBool::new(false);
-
-/// Does the terminal have the "eat new line" glitch.
-pub static TERM_HAS_XN: AtomicBool = AtomicBool::new(false);
-
 static SETENV_LOCK: Mutex<()> = Mutex::new(());
 
-mod ffi {
-    extern "C" {
-        pub fn setenv(
-            name: *const libc::c_char,
-            value: *const libc::c_char,
-            overwrite: libc::c_int,
-        );
-        pub fn unsetenv(name: *const libc::c_char);
-    }
-}
-
 /// Sets an environment variable after obtaining a lock, to try and improve the safety of
 /// environment variables.
 ///
@@ -49,7 +28,7 @@ pub fn setenv_lock<S1: ToCString, S2: ToCString>(name: S1, value: S2, overwrite:
     let value = value.to_cstring();
     let _lock = SETENV_LOCK.lock();
     unsafe {
-        self::ffi::setenv(name.as_ptr(), value.as_ptr(), libc::c_int::from(overwrite));
+        libc::setenv(name.as_ptr(), value.as_ptr(), libc::c_int::from(overwrite));
     }
 }
 
@@ -59,6 +38,6 @@ pub fn unsetenv_lock<S: ToCString>(name: S) {
     let name = name.to_cstring();
     let _lock = SETENV_LOCK.lock();
     unsafe {
-        self::ffi::unsetenv(name.as_ptr());
+        libc::unsetenv(name.as_ptr());
     }
 }
