@@ -1,35 +1,36 @@
+use crate::common::init_special_chars_once;
 use crate::complete::complete_invalidate_path;
-use crate::env::{setenv_lock, unsetenv_lock, EnvMode, EnvStack, Environment};
 use crate::env::{DEFAULT_READ_BYTE_LIMIT, READ_BYTE_LIMIT};
+use crate::env::{EnvMode, EnvStack, Environment, setenv_lock, unsetenv_lock};
 use crate::flog::FLOG;
 use crate::input_common::{update_wait_on_escape_ms, update_wait_on_sequence_key_ms};
+use crate::locale::{invalidate_numeric_locale, set_libc_locales};
 use crate::reader::{
     reader_change_cursor_end_mode, reader_change_cursor_selection_mode, reader_change_history,
     reader_schedule_prompt_repaint, reader_set_autosuggestion_enabled, reader_set_transient_prompt,
 };
 use crate::screen::{
-    screen_set_midnight_commander_hack, IS_DUMB, LAYOUT_CACHE_SHARED, ONLY_GRAYSCALE,
+    IS_DUMB, LAYOUT_CACHE_SHARED, ONLY_GRAYSCALE, screen_set_midnight_commander_hack,
 };
-use crate::terminal::use_terminfo;
 use crate::terminal::ColorSupport;
+use crate::terminal::use_terminfo;
 use crate::tty_handoff::xtversion;
 use crate::wchar::prelude::*;
 use crate::wutil::fish_wcstoi;
 use crate::{function, terminal};
-use std::borrow::Cow;
 use std::collections::HashMap;
-use std::ffi::{CStr, CString};
-use std::ptr;
 use std::sync::atomic::{AtomicBool, Ordering};
 
 /// List of all locale environment variable names that might trigger (re)initializing of the locale
 /// subsystem. These are only the variables we're possibly interested in.
-#[rustfmt::skip]
-const LOCALE_VARIABLES: [&wstr; 10] = [
-    L!("LANG"),       L!("LANGUAGE"), L!("LC_ALL"),
-    L!("LC_COLLATE"), L!("LC_CTYPE"), L!("LC_MESSAGES"),
-    L!("LC_NUMERIC"), L!("LC_TIME"),  L!("LOCPATH"),
-    L!("fish_allow_singlebyte_locale"),
+const LOCALE_VARIABLES: [&wstr; 7] = [
+    L!("LANG"),
+    L!("LANGUAGE"),
+    L!("LC_ALL"),
+    L!("LC_MESSAGES"),
+    L!("LC_NUMERIC"),
+    L!("LC_TIME"),
+    L!("LOCPATH"),
 ];
 
 #[rustfmt::skip]
@@ -144,8 +145,8 @@ fn handle_timezone(var_name: &wstr, vars: &EnvStack) {
         unsetenv_lock(var_name);
     }
 
-    extern "C" {
-        fn tzset();
+    unsafe extern "C" {
+        unsafe fn tzset();
     }
 
     unsafe {
@@ -174,8 +175,6 @@ pub fn guess_emoji_width(vars: &EnvStack) {
         .map(|v| v.as_string())
         .unwrap_or_else(WString::new);
 
-    #[allow(renamed_and_removed_lints)] // for old clippy
-    #[allow(clippy::blocks_in_if_conditions)] // for old clippy
     if xtversion().unwrap_or(L!("")).starts_with(L!("iTerm2 ")) {
         // iTerm2 now defaults to Unicode 9 sizes for anything after macOS 10.12
         FISH_EMOJI_WIDTH.store(2, Ordering::Relaxed);
@@ -300,8 +299,6 @@ fn handle_tz_change(var_name: &wstr, vars: &EnvStack) {
 
 fn handle_locale_change(vars: &EnvStack) {
     init_locale(vars);
-    // We need to re-guess emoji width because the locale might have changed to a multibyte one.
-    guess_emoji_width(vars);
 }
 
 fn handle_term_change(vars: &EnvStack) {
@@ -368,6 +365,7 @@ pub fn env_dispatch_init(vars: &EnvStack) {
 /// Runs the subset of dispatch functions that need to be called at startup.
 fn run_inits(vars: &EnvStack) {
     init_locale(vars);
+    init_special_chars_once();
     init_terminal(vars);
     guess_emoji_width(vars);
     update_wait_on_escape_ms(vars);
@@ -432,9 +430,9 @@ fn update_fish_color_support(vars: &EnvStack) {
         );
     } else {
         supports_24bit = !is_xterm_16color
-            && !vars
+            && vars
                 .get_unless_empty(L!("TERM_PROGRAM"))
-                .is_some_and(|term| term.as_list()[0] == "Apple_Terminal");
+                .is_none_or(|term| term.as_list()[0] != "Apple_Terminal");
         FLOG!(
             term_support,
             "True-color support",
@@ -459,7 +457,7 @@ fn init_terminal(vars: &EnvStack) {
     let term = vars.get(L!("TERM"));
     let term = term
         .as_ref()
-        .and_then(|v| v.as_list().get(0))
+        .and_then(|v| v.as_list().first())
         .map(|v| v.as_utfstr())
         .unwrap_or(L!(""));
 
@@ -503,21 +501,6 @@ pub fn read_terminfo_database(vars: &EnvStack) {
 fn init_locale(vars: &EnvStack) {
     let _guard = crate::locale::LOCALE_LOCK.lock().unwrap();
 
-    #[rustfmt::skip]
-    const UTF8_LOCALES: &[&str] = &[
-        "C.UTF-8", "en_US.UTF-8", "en_GB.UTF-8", "de_DE.UTF-8", "C.utf8", "UTF-8",
-    ];
-
-    let old_msg_locale: CString = {
-        let old = unsafe { libc::setlocale(libc::LC_MESSAGES, ptr::null()) };
-        assert_ne!(old, ptr::null_mut());
-        // We have to make a copy because the subsequent setlocale() call to change the locale will
-        // invalidate the pointer from this setlocale() call.
-
-        // safety: `old` is not a null-pointer, and should be a reference to the currently set locale
-        unsafe { CStr::from_ptr(old.cast()) }.to_owned()
-    };
-
     for var_name in LOCALE_VARIABLES {
         let var = vars
             .getf_unless_empty(var_name, EnvMode::EXPORT)
@@ -531,76 +514,15 @@ fn init_locale(vars: &EnvStack) {
         }
     }
 
-    let user_locale = {
-        let loc_ptr = unsafe { libc::setlocale(libc::LC_ALL, b"\0".as_ptr().cast()) };
-        if loc_ptr.is_null() {
-            FLOG!(env_locale, "user has an invalid locale configured");
-            None
-        } else {
-            // safety: setlocale did not return a null-pointer, so it is a valid pointer
-            Some(unsafe { CStr::from_ptr(loc_ptr) })
-        }
-    };
-
-    // Try to get a multibyte-capable encoding.
-    // A "C" locale is broken for our purposes: any wchar function will break on it. So we try
-    // *really, really, really hard* to not have one.
-    let fix_locale = vars
-        .get_unless_empty(L!("fish_allow_singlebyte_locale"))
-        .map(|v| v.as_string())
-        .map(|allow_c| !crate::wcstringutil::bool_from_string(&allow_c))
-        .unwrap_or(true);
-
-    if fix_locale && crate::libc::MB_CUR_MAX() == 1 {
-        FLOG!(env_locale, "Have singlebyte locale, trying to fix.");
-        for locale in UTF8_LOCALES {
-            {
-                let locale = CString::new(locale.to_owned()).unwrap();
-                // this can fail, that is fine
-                unsafe { libc::setlocale(libc::LC_CTYPE, locale.as_ptr()) };
-            }
-            if crate::libc::MB_CUR_MAX() > 1 {
-                FLOG!(env_locale, "Fixed locale:", locale);
-                break;
-            }
-        }
-
-        if crate::libc::MB_CUR_MAX() == 1 {
-            FLOG!(env_locale, "Failed to fix locale.");
-        }
+    // Safety: we hold the locale lock.
+    if !unsafe {
+        set_libc_locales(/*log_ok=*/ true)
+    } {
+        FLOG!(env_locale, "user has an invalid locale configured");
     }
 
-    // We *always* use a C-locale for numbers because we want '.' (except for in printf).
-    let loc_ptr = unsafe { libc::setlocale(libc::LC_NUMERIC, b"C\0".as_ptr().cast()) };
-    // should never fail, the C locale should always be defined
-    assert_ne!(loc_ptr, ptr::null_mut());
-
-    // See that we regenerate our special locale for numbers
-    crate::locale::invalidate_numeric_locale();
-    crate::common::fish_setlocale();
-    FLOG!(
-        env_locale,
-        "init_locale() setlocale():",
-        user_locale
-            .map(CStr::to_string_lossy)
-            .unwrap_or(Cow::Borrowed("(null)"))
-    );
-
-    let new_msg_loc_ptr = unsafe { libc::setlocale(libc::LC_MESSAGES, std::ptr::null()) };
-    // should never fail
-    assert_ne!(new_msg_loc_ptr, ptr::null_mut());
-    // safety: we just asserted it is not a null-pointer.
-    let new_msg_locale = unsafe { CStr::from_ptr(new_msg_loc_ptr) };
-    FLOG!(
-        env_locale,
-        "Old LC_MESSAGES locale:",
-        old_msg_locale.to_string_lossy()
-    );
-    FLOG!(
-        env_locale,
-        "New LC_MESSAGES locale:",
-        new_msg_locale.to_string_lossy()
-    );
+    // Invalidate the cached numeric locale.
+    invalidate_numeric_locale();
 
     #[cfg(feature = "localize-messages")]
     crate::wutil::gettext::update_locale_from_env(vars);
@@ -611,7 +533,6 @@ pub fn use_posix_spawn() -> bool {
 }
 
 /// Whether or not we are running on an OS where we allow ourselves to use `posix_spawn()`.
-#[allow(clippy::needless_bool)] // for old clippy
 const fn allow_use_posix_spawn() -> bool {
     // OpenBSD's posix_spawn returns status 127 instead of erroring with ENOEXEC when faced with a
     // shebang-less script. Disable posix_spawn on OpenBSD.
