@@ -4,19 +4,20 @@
 // performed have been massive.
 
 use crate::builtins::shared::{
-    builtin_run, ErrorCode, STATUS_CMD_ERROR, STATUS_CMD_UNKNOWN, STATUS_NOT_EXECUTABLE,
-    STATUS_READ_TOO_MUCH,
+    ErrorCode, STATUS_CMD_ERROR, STATUS_CMD_UNKNOWN, STATUS_NOT_EXECUTABLE, STATUS_READ_TOO_MUCH,
+    builtin_run,
 };
 use crate::common::{
-    exit_without_destructors, str2wcstring, truncate_at_nul, wcs2string, wcs2zstring, write_loop,
-    ScopeGuard,
+    ScopeGuard, bytes2wcstring, exit_without_destructors, truncate_at_nul, wcs2bytes, wcs2zstring,
+    write_loop,
 };
-use crate::env::{EnvMode, EnvStack, Environment, Statuses, READ_BYTE_LIMIT};
+use crate::env::{EnvMode, EnvStack, Environment, READ_BYTE_LIMIT, Statuses};
 #[cfg(FISH_USE_POSIX_SPAWN)]
 use crate::env_dispatch::use_posix_spawn;
 use crate::fds::make_fd_blocking;
-use crate::fds::{make_autoclose_pipes, open_cloexec, PIPE_ERROR};
+use crate::fds::{PIPE_ERROR, make_autoclose_pipes, open_cloexec};
 use crate::flog::{FLOG, FLOGF};
+use crate::fork_exec::PATH_BSHELL;
 use crate::fork_exec::blocked_signals_for_job;
 use crate::fork_exec::postfork::{
     child_setup_process, execute_fork, execute_setpgid, report_setpgid_error,
@@ -29,19 +30,18 @@ use crate::io::{
     BufferedOutputStream, FdOutputStream, IoBufferfill, IoChain, IoClose, IoMode, IoPipe,
     IoStreams, OutputStream, SeparatedBuffer, StringOutputStream,
 };
-use crate::libc::_PATH_BSHELL;
 use crate::nix::{getpid, isatty};
 use crate::null_terminated_array::OwningNullTerminatedArray;
 use crate::parser::{Block, BlockId, BlockType, EvalRes, Parser};
 #[cfg(FISH_USE_POSIX_SPAWN)]
 use crate::proc::Pid;
 use crate::proc::{
-    hup_jobs, is_interactive_session, jobs_requiring_warning_on_exit, no_exec,
-    print_exit_warning_for_jobs, InternalProc, Job, JobGroupRef, ProcStatus, Process, ProcessType,
+    InternalProc, Job, JobGroupRef, ProcStatus, Process, ProcessType, hup_jobs,
+    is_interactive_session, jobs_requiring_warning_on_exit, no_exec, print_exit_warning_for_jobs,
 };
 use crate::reader::{reader_run_count, safe_restore_term_mode};
-use crate::redirection::{dup2_list_resolve_chain, Dup2List};
-use crate::threads::{iothread_perform_cant_wait, is_forked_child};
+use crate::redirection::{Dup2List, dup2_list_resolve_chain};
+use crate::threads::{ThreadPool, is_forked_child};
 use crate::trace::trace_if_enabled_with_args;
 use crate::tty_handoff::TtyHandoff;
 use crate::wchar::prelude::*;
@@ -50,7 +50,7 @@ use crate::wutil::{fish_wcstol, perror};
 use errno::{errno, set_errno};
 use libc::{
     EACCES, ENOENT, ENOEXEC, ENOTDIR, EPIPE, EXIT_FAILURE, EXIT_SUCCESS, STDERR_FILENO,
-    STDIN_FILENO, STDOUT_FILENO,
+    STDIN_FILENO, STDOUT_FILENO, c_char,
 };
 use nix::fcntl::OFlag;
 use nix::sys::stat;
@@ -60,8 +60,20 @@ use std::mem::MaybeUninit;
 use std::num::NonZeroU32;
 use std::os::fd::{AsRawFd, OwnedFd, RawFd};
 use std::slice;
-use std::sync::atomic::Ordering;
-use std::sync::{atomic::AtomicUsize, Arc};
+use std::sync::{
+    Arc, OnceLock,
+    atomic::{AtomicUsize, Ordering},
+};
+
+/// The singleton shared exec thread pool.
+/// This is used to write the output of internal processes (e.g. builtins)
+/// to their target fds.
+/// TODO: this IO could be multiplexed using FdMonitor.
+fn exec_thread_pool() -> &'static Arc<ThreadPool> {
+    static EXEC_THREAD_POOL: OnceLock<Arc<ThreadPool>> = OnceLock::new();
+    // Use an unbounded queue because otherwise we risk deadlock.
+    EXEC_THREAD_POOL.get_or_init(|| ThreadPool::new(1, usize::MAX))
+}
 
 /// Execute the processes specified by `j` in the parser \p.
 /// On a true return, the job was successfully launched and the parser will take responsibility for
@@ -212,7 +224,7 @@ pub fn exec_job(parser: &Parser, job: &Job, block_io: IoChain) -> bool {
 
     FLOGF!(
         exec_job_exec,
-        "Executed job %d from command '%ls'",
+        "Executed job %d from command '%s'",
         job.job_id(),
         job.command()
     );
@@ -289,11 +301,7 @@ pub fn exec_subshell_for_expand(
         true,
     );
     // Only return an error code if we should break expansion.
-    if break_expand {
-        ret
-    } else {
-        Ok(())
-    }
+    if break_expand { ret } else { Ok(()) }
 }
 
 /// Number of calls to fork() or posix_spawn().
@@ -401,19 +409,20 @@ fn safe_launch_process(
     if err.0 == ENOEXEC && is_thompson_shell_script(actual_cmd) {
         // Construct new argv.
         // We must not allocate memory, so only 128 args are supported.
-        const maxargs: usize = 128;
+        const MAXARGS: usize = 128;
         let nargs = argv.len();
         let argv = unsafe { slice::from_raw_parts(argv.get(), nargs) };
-        if nargs <= maxargs {
+        if nargs <= MAXARGS {
             // +1 for /bin/sh, +1 for terminating nullptr
-            let mut argv2 = [std::ptr::null(); 1 + maxargs + 1];
-            argv2[0] = _PATH_BSHELL.load(Ordering::Relaxed);
+            let mut argv2 = [std::ptr::null(); 1 + MAXARGS + 1];
+            let bshell = PATH_BSHELL.as_ptr() as *const c_char;
+            argv2[0] = bshell as *mut c_char;
             argv2[1..argv.len() + 1].copy_from_slice(argv);
             // The command to call should use the full path,
             // not what we would pass as argv0.
             argv2[1] = actual_cmd.as_ptr();
             unsafe {
-                libc::execve(_PATH_BSHELL.load(Ordering::Relaxed), &argv2[0], envv.get());
+                libc::execve(bshell, &argv2[0], envv.get());
             }
         }
     }
@@ -573,7 +582,7 @@ fn run_internal_process(p: &Process, outdata: Vec<u8>, errdata: Vec<u8>, ios: &I
 
     FLOGF!(
         proc_internal_proc,
-        "Created internal proc %llu to write output for proc '%ls'",
+        "Created internal proc %u to write output for proc '%s'",
         internal_proc.get_id(),
         p.argv0().unwrap()
     );
@@ -604,7 +613,7 @@ fn run_internal_process(p: &Process, outdata: Vec<u8>, errdata: Vec<u8>, ios: &I
     // builtin_run provide this directly, rather than setting it in the process.
     f.success_status = p.status();
 
-    iothread_perform_cant_wait(move || {
+    exec_thread_pool().perform(move || {
         let mut status = f.success_status;
         if !f.skip_out() {
             if let Err(err) = write_loop(&f.src_outfd, &f.outdata) {
@@ -645,7 +654,7 @@ fn run_internal_process_or_short_circuit(
         if p.is_last_in_job {
             FLOGF!(
                 exec_job_status,
-                "Set status of job %d (%ls) to %d using short circuit",
+                "Set status of job %d (%s) to %d using short circuit",
                 j.job_id(),
                 j.preview(),
                 p.status().status_value()
@@ -750,15 +759,16 @@ fn fork_child_for_process(
     }
 
     // We are the parent. Record the pid and store the pgid for the job if it should lead the pgroup.
-    p.set_pid(Pid::new(pid).unwrap());
+    let pid = Pid::new(pid);
+    p.set_pid(pid);
     if matches!(pgroup_policy, PgroupPolicy::Lead) {
-        job.group().set_pgid(Pid::new(pid).unwrap());
+        job.group().set_pgid(pid);
     }
 
     let count = FORK_COUNT.fetch_add(1, Ordering::Relaxed) + 1;
     FLOGF!(
         exec_fork,
-        "Fork #%d, pid %d fork external command for '%ls'",
+        "Fork #%d, pid %d fork external command for '%s'",
         count,
         pid,
         p.argv0().unwrap()
@@ -824,8 +834,8 @@ fn handle_builtin_output(
     assert!(p.is_builtin(), "Process is not a builtin");
 
     // Figure out any data remaining to write. We may have none, in which case we can short-circuit.
-    let outbuff = wcs2string(out.contents());
-    let errbuff = wcs2string(err.contents());
+    let outbuff = wcs2bytes(out.contents());
+    let errbuff = wcs2bytes(err.contents());
 
     // Some historical behavior.
     if !outbuff.is_empty() {
@@ -894,7 +904,7 @@ fn exec_external_command(
         };
         FLOGF!(
             exec_fork,
-            "Fork #%d, pid %d: spawn external command '%s' from '%ls'",
+            "Fork #%d, pid %d: spawn external command '%s' from '%s'",
             count,
             pid,
             p.actual_cmd,
@@ -904,14 +914,15 @@ fn exec_external_command(
         );
 
         // these are all things do_fork() takes care of normally (for forked processes):
-        p.set_pid(Pid::new(pid).unwrap());
+        let pid = Pid::new(pid);
+        p.set_pid(pid);
         if p.leads_pgrp {
-            j.group().set_pgid(Pid::new(pid).unwrap());
+            j.group().set_pgid(pid);
             // posix_spawn should in principle set the pgid before returning.
             // In glibc, posix_spawn uses fork() and the pgid group is set on the child side;
             // therefore the parent may not have seen it be set yet.
             // Ensure it gets set. See #4715, also https://github.com/Microsoft/WSL/issues/2997.
-            execute_setpgid(pid, pid, true /* is parent */);
+            execute_setpgid(pid.as_pid_t(), pid.as_pid_t(), true /* is parent */);
         }
         return Ok(());
     }
@@ -947,7 +958,11 @@ fn function_prepare_environment(
     // 2. inherited variables
     // 3. argv
 
+    let mut overwrite_argv = false;
     for (idx, named_arg) in props.named_arguments.iter().enumerate() {
+        if named_arg == L!("argv") {
+            overwrite_argv = true
+        };
         if idx < argv.len() {
             vars.set_one(named_arg, EnvMode::LOCAL | EnvMode::USER, argv[idx].clone());
         } else {
@@ -956,10 +971,15 @@ fn function_prepare_environment(
     }
 
     for (key, value) in &*props.inherit_vars {
+        if key == L!("argv") {
+            overwrite_argv = true
+        };
         vars.set(key, EnvMode::LOCAL | EnvMode::USER, value.clone());
     }
 
-    vars.set_argv(argv);
+    if !overwrite_argv {
+        vars.set_argv(argv);
+    }
     fb
 }
 
@@ -989,7 +1009,7 @@ fn get_performer_for_block_node(p: &Process, job: &Job, io_chain: &IoChain) -> B
     let node = node.clone();
     Box::new(move |parser: &Parser, _out, _err| {
         parser
-            .eval_node(&node, &io_chain, job_group.as_ref(), BlockType::top)
+            .eval_node(&node, &io_chain, job_group.as_ref(), BlockType::top, false)
             .status
     })
 }
@@ -1016,7 +1036,7 @@ fn get_performer_for_function(
     let Some(props) = function::get_props(p.argv0().unwrap()) else {
         FLOG!(
             error,
-            wgettext_fmt!("Unknown function '%ls'", p.argv0().unwrap())
+            wgettext_fmt!("Unknown function '%s'", p.argv0().unwrap())
         );
         return Err(());
     };
@@ -1025,7 +1045,13 @@ fn get_performer_for_function(
         // Pull out the job list from the function.
         let fb = function_prepare_environment(parser, argv, &props);
         let body_node = props.func_node.child_ref(|n| &n.jobs);
-        let mut res = parser.eval_node(&body_node, &io_chain, job_group.as_ref(), BlockType::top);
+        let mut res = parser.eval_node(
+            &body_node,
+            &io_chain,
+            job_group.as_ref(),
+            BlockType::top,
+            false,
+        );
         function_restore_environment(parser, fb);
 
         // If the function did not execute anything, treat it as success.
@@ -1328,7 +1354,9 @@ fn exec_process_in_job(
         }
         ProcessType::Exec => {
             // We should have handled exec up above.
-            panic!("process_type_t::exec process found in pipeline, where it should never be. Aborting.");
+            panic!(
+                "process_type_t::exec process found in pipeline, where it should never be. Aborting."
+            );
         }
     }
 }
@@ -1361,7 +1389,7 @@ fn get_deferred_process(j: &Job) -> Option<usize> {
 }
 
 /// Given that we failed to execute process `failed_proc` in job `job`, mark that process and
-/// every subsequent process in the pipeline as aborted before launch.
+/// every subsequent process in the pipeline as aborted before launch.
 fn abort_pipeline_from(job: &Job, offset: usize) {
     for p in job.processes().iter().skip(offset) {
         p.mark_aborted_before_launch();
@@ -1403,7 +1431,7 @@ fn populate_subshell_output(lst: &mut Vec<WString>, buffer: &SeparatedBuffer, sp
         let data = &elem.contents;
         if elem.is_explicitly_separated() {
             // Just append this one.
-            lst.push(str2wcstring(data));
+            lst.push(bytes2wcstring(data));
             continue;
         }
 
@@ -1421,7 +1449,7 @@ fn populate_subshell_output(lst: &mut Vec<WString>, buffer: &SeparatedBuffer, sp
                 // If it's not found, just use the end.
                 let stop = stop.map(|rel| cursor + rel).unwrap_or(data.len());
                 // Stop now points at the first character we do not want to copy.
-                lst.push(str2wcstring(&data[cursor..stop]));
+                lst.push(bytes2wcstring(&data[cursor..stop]));
 
                 // If we hit a separator, skip over it; otherwise we're at the end.
                 cursor = stop + if hit_separator { 1 } else { 0 };
@@ -1429,7 +1457,7 @@ fn populate_subshell_output(lst: &mut Vec<WString>, buffer: &SeparatedBuffer, sp
         } else {
             // We're not splitting output, but we still want to trim off a trailing newline.
             let trailing_newline = if data.last() == Some(&b'\n') { 1 } else { 0 };
-            lst.push(str2wcstring(&data[..data.len() - trailing_newline]));
+            lst.push(bytes2wcstring(&data[..data.len() - trailing_newline]));
         }
     }
 }
@@ -1478,7 +1506,7 @@ fn exec_subshell_internal(
 
     let mut io_chain = IoChain::new();
     io_chain.push(bufferfill.clone());
-    let eval_res = parser.eval_with(cmd, &io_chain, job_group, BlockType::subst);
+    let eval_res = parser.eval_with(cmd, &io_chain, job_group, BlockType::subst, false);
     let buffer = IoBufferfill::finish(bufferfill);
     if buffer.discarded() {
         *break_expand = true;
