@@ -30,8 +30,8 @@ static WC_LOOKUP_TABLE: Lazy<WcLookupTable> = Lazy::new(WcLookupTable::new);
 /// A safe wrapper around the system `wcwidth()` function
 #[cfg(not(cygwin))]
 pub fn wcwidth(c: char) -> isize {
-    extern "C" {
-        pub fn wcwidth(c: libc::wchar_t) -> libc::c_int;
+    unsafe extern "C" {
+        pub unsafe fn wcwidth(c: libc::wchar_t) -> libc::c_int;
     }
 
     const _: () = assert!(std::mem::size_of::<libc::wchar_t>() >= std::mem::size_of::<char>());
@@ -162,21 +162,26 @@ pub fn wcscasecmp_fuzzy(lhs: &wstr, rhs: &wstr, canonicalize: fn(char) -> char)
     lhs.cmp(rhs)
 }
 
-#[test]
-fn test_wcscasecmp() {
+#[cfg(test)]
+mod tests {
+    use super::wcscasecmp;
+    use crate::wchar::prelude::*;
     use std::cmp::Ordering;
 
-    // Comparison with empty
-    assert_eq!(wcscasecmp(L!("a"), L!("")), Ordering::Greater);
-    assert_eq!(wcscasecmp(L!(""), L!("a")), Ordering::Less);
-    assert_eq!(wcscasecmp(L!(""), L!("")), Ordering::Equal);
+    #[test]
+    fn test_wcscasecmp() {
+        // Comparison with empty
+        assert_eq!(wcscasecmp(L!("a"), L!("")), Ordering::Greater);
+        assert_eq!(wcscasecmp(L!(""), L!("a")), Ordering::Less);
+        assert_eq!(wcscasecmp(L!(""), L!("")), Ordering::Equal);
 
-    // Basic comparison
-    assert_eq!(wcscasecmp(L!("A"), L!("a")), Ordering::Equal);
-    assert_eq!(wcscasecmp(L!("B"), L!("a")), Ordering::Greater);
-    assert_eq!(wcscasecmp(L!("A"), L!("B")), Ordering::Less);
+        // Basic comparison
+        assert_eq!(wcscasecmp(L!("A"), L!("a")), Ordering::Equal);
+        assert_eq!(wcscasecmp(L!("B"), L!("a")), Ordering::Greater);
+        assert_eq!(wcscasecmp(L!("A"), L!("B")), Ordering::Less);
 
-    // Multi-byte comparison
-    assert_eq!(wcscasecmp(L!("İ"), L!("i\u{307}")), Ordering::Equal);
-    assert_eq!(wcscasecmp(L!("ia"), L!("İa")), Ordering::Less);
+        // Multi-byte comparison
+        assert_eq!(wcscasecmp(L!("İ"), L!("i\u{307}")), Ordering::Equal);
+        assert_eq!(wcscasecmp(L!("ia"), L!("İa")), Ordering::Less);
+    }
 }
