@@ -1,15 +1,15 @@
 use crate::{
-    common::{str2wcstring, wcs2osstring, wcs2zstring},
+    FLOG, FLOGF,
+    common::{bytes2wcstring, wcs2osstring, wcs2zstring},
     fds::wopen_cloexec,
-    path::{path_remoteness, DirRemoteness},
+    path::{DirRemoteness, path_remoteness},
     wchar::prelude::*,
     wutil::{
-        file_id_for_file, file_id_for_path, wdirname, wrename, wunlink, FileId, INVALID_FILE_ID,
+        FileId, INVALID_FILE_ID, file_id_for_file, file_id_for_path, wdirname, wrename, wunlink,
     },
-    FLOG, FLOGF,
 };
 use errno::errno;
-use libc::{c_int, fchown, flock, LOCK_EX, LOCK_SH};
+use libc::{LOCK_EX, LOCK_SH, c_int};
 use nix::{fcntl::OFlag, sys::stat::Mode};
 use std::{
     ffi::CString,
@@ -32,7 +32,7 @@ fn fish_mkstemp_cloexec(name_template: CString) -> std::io::Result<(File, CStrin
     };
     #[cfg(apple)]
     let fd = {
-        use libc::{FD_CLOEXEC, F_SETFD};
+        use libc::{F_SETFD, FD_CLOEXEC};
         let fd = unsafe { libc::mkstemp(name) };
         if fd != -1 {
             unsafe { libc::fcntl(fd, F_SETFD, FD_CLOEXEC) };
@@ -56,11 +56,7 @@ pub fn create_temporary_file(name_template: &wstr) -> std::io::Result<(File, WSt
                 _ => {
                     FLOG!(
                         error,
-                        wgettext_fmt!(
-                            "Unable to create temporary file '%ls': %s",
-                            name_template,
-                            e
-                        )
+                        wgettext_fmt!("Unable to create temporary file '%s': %s", name_template, e)
                     );
 
                     return Err(e);
@@ -68,7 +64,7 @@ pub fn create_temporary_file(name_template: &wstr) -> std::io::Result<(File, WSt
             },
         }
     };
-    Ok((fd, str2wcstring(c_string_template.to_bytes())))
+    Ok((fd, bytes2wcstring(c_string_template.to_bytes())))
 }
 
 /// Use this struct for all accesses to file which need mutual exclusion.
@@ -119,7 +115,7 @@ impl LockingMode {
             Self::Exclusive(WriteMethod::Append) => {
                 OFlag::O_WRONLY | OFlag::O_APPEND | OFlag::O_CREAT
             }
-            Self::Exclusive(WriteMethod::RenameIntoPlace) => OFlag::O_RDONLY | OFlag::O_CREAT,
+            Self::Exclusive(WriteMethod::RenameIntoPlace) => OFlag::O_RDWR | OFlag::O_CREAT,
         }
     }
 }
@@ -132,8 +128,8 @@ impl LockedFile {
     /// Two modes of modification are supported:
     /// - Appending
     /// - Writing to a temporary file which is then renamed into place.
-    /// File flags are derived from the [`LockingMode`].
-    /// `file_name` should just be a name, not a full path.
+    ///   File flags are derived from the [`LockingMode`].
+    ///   `file_name` should just be a name, not a full path.
     pub fn new(locking_mode: LockingMode, file_path: &wstr) -> std::io::Result<Self> {
         let dir_path = wdirname(file_path);
 
@@ -149,12 +145,16 @@ impl LockedFile {
         let dir_fd = wopen_cloexec(dir_path, OFlag::O_RDONLY, Mode::empty())?;
 
         // Try locking the directory. Retry if locking was interrupted.
-        while unsafe { flock(dir_fd.as_raw_fd(), locking_mode.flock_op()) } == -1 {
+        while unsafe { libc::flock(dir_fd.as_raw_fd(), locking_mode.flock_op()) } == -1 {
             let err = std::io::Error::last_os_error();
             if err.kind() != std::io::ErrorKind::Interrupted {
                 return Err(err);
             }
         }
+        #[cfg(cygwin)]
+        // Prevents error 14 "Bad Addr" on Windows (#11933). If the error happens
+        // the process appears deadlocked
+        std::thread::sleep(std::time::Duration::from_nanos(1));
 
         // Open the data file
         let data_file = wopen_cloexec(file_path, locking_mode.file_flags(), LOCKED_FILE_MODE)?;
@@ -196,6 +196,7 @@ pub fn fsync(file: &File) -> std::io::Result<()> {
             -1 => {
                 let os_error = std::io::Error::last_os_error();
                 if os_error.kind() != std::io::ErrorKind::Interrupted {
+                    FLOGF!(synced_file_access, "fsync failed: %s", os_error);
                     return Err(os_error);
                 }
             }
@@ -212,14 +213,13 @@ pub fn fsync(file: &File) -> std::io::Result<()> {
 /// If the file does not exist this function will return an error.
 pub fn lock_and_load<F, UserData>(path: &wstr, load: F) -> std::io::Result<(FileId, UserData)>
 where
-    F: Fn(&File) -> std::io::Result<UserData>,
+    F: Fn(&File, FileId) -> std::io::Result<UserData>,
 {
     match LockedFile::new(LockingMode::Shared, path) {
         Ok(locked_file) => {
-            return Ok((
-                file_id_for_file(locked_file.get()),
-                load(locked_file.get())?,
-            ));
+            let file_id = file_id_for_file(locked_file.get());
+            let user_data = load(locked_file.get(), file_id.clone())?;
+            return Ok((file_id, user_data));
         }
         Err(e) => {
             FLOGF!(
@@ -245,11 +245,11 @@ where
     // Fallback implementation for situations where locking is unavailable.
     let max_attempts = 1000;
     for _ in 0..max_attempts {
-        let initial_file_id = file_id_for_path(path);
         // If we cannot open the file, there is nothing we can do,
         // so just return immediately.
         let file = wopen_cloexec(path, OFlag::O_RDONLY, Mode::empty())?;
-        let loaded_data = match load(&file) {
+        let initial_file_id = file_id_for_file(&file);
+        let loaded_data = match load(&file, initial_file_id.clone()) {
             Ok(update_data) => update_data,
             Err(_) => {
                 // Retry if load function failed. Because we do not hold a lock, this might be
@@ -265,7 +265,9 @@ where
         // If the file id did not change, we assume that we loaded a consistent state.
         return Ok((final_file_id, loaded_data));
     }
-    Err(std::io::Error::new(std::io::ErrorKind::Other, "Failed to update the file. Locking is disabled, and the fallback code did not succeed within the permissible number of attempts."))
+    Err(std::io::Error::other(
+        "Failed to update the file. Locking is disabled, and the fallback code did not succeed within the permissible number of attempts.",
+    ))
 }
 
 pub struct PotentialUpdate<UserData> {
@@ -285,11 +287,12 @@ pub struct PotentialUpdate<UserData> {
 ///
 /// - `path`: The path to the file which should be updated.
 /// - `rewrite`: The function which handles reading from the file and writing to a temporary file.
-/// The first argument is for the file to read from, the second for the temporary file to write to.
-/// On success, the value returned by `rewrite` is included in this functions return value. Be
-/// careful about side effects of `rewrite`. It might get executed multiple times. Try to avoid
-/// side effects and instead extract any data you might need and return them on success.
-/// Then, apply the desired side effects once this function has returned successfully.
+///   The first argument is for the file to read from, the second for the temporary file to
+///   write to.  On success, the value returned by `rewrite` is included in this functions
+///   return value. Be careful about side effects of `rewrite`. It might get executed multiple
+///   times. Try to avoid side effects and instead extract any data you might need and return
+///   them on success.  Then, apply the desired side effects once this function has returned
+///   successfully.
 ///
 /// # Return value
 ///
@@ -315,12 +318,11 @@ where
         // did, it would be tricky to set the permissions correctly. (bash doesn't get this
         // case right either).
         if let Ok(md) = old_file.metadata() {
-            // TODO(MSRV): Consider replacing with std::os::unix::fs::fchown when MSRV >= 1.73
-            if unsafe { fchown(new_file.as_raw_fd(), md.uid(), md.gid()) } == -1 {
+            if let Err(e) = std::os::unix::fs::fchown(new_file, Some(md.uid()), Some(md.gid())) {
                 FLOG!(
                     synced_file_access,
                     "Error when changing ownership of file:",
-                    errno::errno()
+                    e
                 );
             }
             if let Err(e) = new_file.set_permissions(md.permissions()) {
@@ -473,7 +475,9 @@ where
             // (If we did write.)
             return Ok((final_file_id, potential_update));
         }
-        Err(std::io::Error::new(std::io::ErrorKind::Other, "Failed to update the file. Locking is disabled, and the fallback code did not succeed within the permissible number of attempts."))
+        Err(std::io::Error::other(
+            "Failed to update the file. Locking is disabled, and the fallback code did not succeed within the permissible number of attempts.",
+        ))
     }
 
     const TMP_FILE_SUFFIX: &wstr = L!(".XXXXXX");
