@@ -4,7 +4,7 @@
 
 use crate::ast::{self, Node};
 use crate::autoload::Autoload;
-use crate::common::{assert_sync, escape, valid_func_name, FilenameRef};
+use crate::common::{FilenameRef, assert_sync, escape, valid_func_name};
 use crate::complete::complete_wrap_map;
 use crate::env::{EnvStack, Environment};
 use crate::event::{self, EventDescription};
@@ -13,7 +13,7 @@ use crate::parse_tree::NodeRef;
 use crate::parser::Parser;
 use crate::parser_keywords::parser_keywords_is_reserved;
 use crate::wchar::prelude::*;
-use crate::wutil::{dir_iter::DirIter, sprintf};
+use crate::wutil::dir_iter::DirIter;
 use once_cell::sync::Lazy;
 use std::collections::{HashMap, HashSet};
 use std::num::NonZeroU32;
@@ -240,7 +240,7 @@ pub fn exists_no_autoload(cmd: &wstr) -> bool {
         return true;
     }
 
-    let narrow = crate::common::wcs2string(cmd);
+    let narrow = crate::common::wcs2bytes(cmd);
     if let Ok(cmdstr) = std::str::from_utf8(&narrow) {
         let cmd = "functions/".to_owned() + cmdstr + ".fish";
         crate::autoload::has_asset(&cmd)
@@ -448,10 +448,10 @@ impl FunctionProperties {
             let d = &handler.desc;
             match d {
                 EventDescription::Signal { signal } => {
-                    sprintf!(=> &mut out, " --on-signal %ls", signal.name());
+                    sprintf!(=> &mut out, " --on-signal %s", signal.name());
                 }
                 EventDescription::Variable { name } => {
-                    sprintf!(=> &mut out, " --on-variable %ls", name);
+                    sprintf!(=> &mut out, " --on-variable %s", name);
                 }
                 EventDescription::ProcessExit { pid } => {
                     let pid = pid.map(|p| p.get()).unwrap_or(0);
@@ -465,7 +465,7 @@ impl FunctionProperties {
                     out.push_str(" --on-job-exit caller");
                 }
                 EventDescription::Generic { param } => {
-                    sprintf!(=> &mut out, " --on-event %ls", param);
+                    sprintf!(=> &mut out, " --on-event %s", param);
                 }
                 EventDescription::Any => {
                     panic!("Unexpected event handler type");
@@ -475,8 +475,9 @@ impl FunctionProperties {
 
         let named = &self.named_arguments;
         if !named.is_empty() {
+            sprintf!(=> &mut out, " --argument-names");
             for name in named {
-                sprintf!(=> &mut out, " --argument-names %ls", name);
+                sprintf!(=> &mut out, " %s", name);
             }
         }
 
@@ -490,7 +491,7 @@ impl FunctionProperties {
         for (name, values) in self.inherit_vars.iter() {
             // We don't know what indentation style the function uses,
             // so we do what fish_indent would.
-            sprintf!(=> &mut out, "\n    set -l %ls", name);
+            sprintf!(=> &mut out, "\n    set -l %s", name);
             for arg in values {
                 out.push(' ');
                 out.push_utfstr(&escape(arg));
