@@ -1,28 +1,20 @@
-use crate::common::{escape, get_by_sorted_name, str2wcstring, Named};
+use crate::common::{Named, escape, get_by_sorted_name};
 use crate::env::Environment;
-use crate::event;
 use crate::flog::FLOG;
-// Polyfill for Option::is_none_or(), stabilized in 1.82.0
-#[allow(unused_imports)]
-use crate::future::IsSomeAnd;
 use crate::global_safety::RelaxedAtomicBool;
 use crate::input_common::{
-    match_key_event_to_key, CharEvent, CharInputStyle, ImplicitEvent, InputData, InputEventQueuer,
-    KeyMatchQuality, ReadlineCmd, R_END_INPUT_FUNCTIONS,
+    CharEvent, CharInputStyle, ImplicitEvent, InputEventQueuer, KeyMatchQuality,
+    R_END_INPUT_FUNCTIONS, ReadlineCmd, match_key_event_to_key,
 };
-use crate::key::{self, canonicalize_raw_escapes, ctrl, Key, Modifiers};
-use crate::proc::job_reap;
-use crate::reader::{
-    reader_reading_interrupted, reader_reset_interrupted, reader_schedule_prompt_repaint, Reader,
-};
-use crate::signal::signal_clear_cancel;
-use crate::threads::{assert_is_main_thread, iothread_service_main};
+use crate::key::{self, Key, Modifiers, canonicalize_raw_escapes, ctrl};
+use crate::reader::{Reader, reader_reset_interrupted};
+use crate::threads::assert_is_main_thread;
 use crate::wchar::prelude::*;
 use once_cell::sync::Lazy;
 use std::mem;
 use std::sync::{
-    atomic::{AtomicU32, Ordering},
     Mutex, MutexGuard,
+    atomic::{AtomicU32, Ordering},
 };
 
 pub const FISH_BIND_MODE_VAR: &wstr = L!("fish_bind_mode");
@@ -224,16 +216,6 @@ const fn _assert_sizes_match() {
 }
 const _: () = _assert_sizes_match();
 
-// Keep this function for debug purposes
-// See 031b265
-#[allow(dead_code)]
-pub fn describe_char(c: i32) -> WString {
-    if c > 0 && (c as usize) < R_END_INPUT_FUNCTIONS {
-        return sprintf!("%02x (%ls)", c, INPUT_FUNCTION_METADATA[c as usize].name);
-    }
-    return sprintf!("%02x", c);
-}
-
 /// The input mapping set is the set of mappings from character sequences to commands.
 #[derive(Debug, Default)]
 pub struct InputMappingSet {
@@ -390,68 +372,6 @@ pub fn init_input() {
     }
 }
 
-impl<'a> InputEventQueuer for Reader<'a> {
-    fn get_input_data(&self) -> &InputData {
-        &self.data.input_data
-    }
-
-    fn get_input_data_mut(&mut self) -> &mut InputData {
-        &mut self.data.input_data
-    }
-
-    fn prepare_to_select(&mut self) {
-        // Fire any pending events and reap stray processes, including printing exit status messages.
-        event::fire_delayed(self.parser);
-        if job_reap(self.parser, true) {
-            reader_schedule_prompt_repaint();
-        }
-    }
-
-    fn select_interrupted(&mut self) {
-        // Readline commands may be bound to \cc which also sets the cancel flag.
-        // See #6937, #8125.
-        signal_clear_cancel();
-
-        // Fire any pending events and reap stray processes, including printing exit status messages.
-        let parser = self.parser;
-        event::fire_delayed(parser);
-        if job_reap(parser, true) {
-            reader_schedule_prompt_repaint();
-        }
-
-        // Tell the reader an event occurred.
-        if reader_reading_interrupted(self) != 0 {
-            self.enqueue_interrupt_key();
-            return;
-        }
-        self.push_front(CharEvent::from_check_exit());
-    }
-
-    fn uvar_change_notified(&mut self) {
-        self.parser.sync_uvars_and_fire(true /* always */);
-    }
-
-    fn ioport_notified(&mut self) {
-        iothread_service_main(self);
-    }
-
-    fn paste_start_buffering(&mut self) {
-        self.input_data.paste_buffer = Some(vec![]);
-        self.push_front(CharEvent::from_readline(ReadlineCmd::BeginUndoGroup));
-    }
-
-    fn paste_commit(&mut self) {
-        self.push_front(CharEvent::from_readline(ReadlineCmd::EndUndoGroup));
-        let Some(buffer) = self.input_data.paste_buffer.take() else {
-            return;
-        };
-        self.push_front(CharEvent::Command(sprintf!(
-            "__fish_paste %s",
-            escape(&str2wcstring(&buffer))
-        )));
-    }
-}
-
 /// A struct which allows accumulating input events, or returns them to the queue.
 /// This contains a list of events which have been dequeued, and a current index into that list.
 pub struct EventQueuePeeker<'q, Queuer: InputEventQueuer + ?Sized> {
@@ -1043,3 +963,72 @@ pub fn input_function_get_code(name: &wstr) -> Option<ReadlineCmd> {
     // a binary search for the matching name.
     get_by_sorted_name(name, INPUT_FUNCTION_METADATA).map(|md| md.code)
 }
+
+#[cfg(test)]
+mod tests {
+    use super::{DEFAULT_BIND_MODE, EventQueuePeeker, InputMappingSet, KeyNameStyle};
+    use crate::env::EnvStack;
+    use crate::input_common::{CharEvent, InputData, InputEventQueuer, KeyEvent};
+    use crate::key::Key;
+    use crate::wchar::prelude::*;
+
+    struct TestInputEventQueuer {
+        input_data: InputData,
+    }
+
+    impl InputEventQueuer for TestInputEventQueuer {
+        fn get_input_data(&self) -> &InputData {
+            &self.input_data
+        }
+        fn get_input_data_mut(&mut self) -> &mut InputData {
+            &mut self.input_data
+        }
+    }
+
+    #[test]
+    fn test_input() {
+        let vars = EnvStack::new();
+        let mut input = TestInputEventQueuer {
+            input_data: InputData::new(i32::MAX, None), // value doesn't matter since we don't read from it
+        };
+        // Ensure sequences are order independent. Here we add two bindings where the first is a prefix
+        // of the second, and then emit the second key list. The second binding should be invoked, not
+        // the first!
+        let prefix_binding: Vec<Key> = "qqqqqqqa".chars().map(Key::from_raw).collect();
+        let mut desired_binding = prefix_binding.clone();
+        desired_binding.push(Key::from_raw('a'));
+
+        let default_mode = || DEFAULT_BIND_MODE.to_owned();
+
+        let mut input_mappings = InputMappingSet::default();
+        input_mappings.add1(
+            prefix_binding,
+            KeyNameStyle::Plain,
+            WString::from_str("up-line"),
+            default_mode(),
+            None,
+            true,
+        );
+        input_mappings.add1(
+            desired_binding.clone(),
+            KeyNameStyle::Plain,
+            WString::from_str("down-line"),
+            default_mode(),
+            None,
+            true,
+        );
+
+        // Push the desired binding to the queue.
+        for key in desired_binding {
+            input
+                .input_data
+                .queue_char(CharEvent::from_key(KeyEvent::from(key)));
+        }
+
+        let mut peeker = EventQueuePeeker::new(&mut input);
+        let mapping = peeker.find_mapping(&vars, &input_mappings);
+        assert!(mapping.is_some());
+        assert!(mapping.unwrap().commands == ["down-line"]);
+        peeker.restart();
+    }
+}
