@@ -1,10 +1,10 @@
 use crate::builtins::shared::{STATUS_CMD_ERROR, STATUS_CMD_OK, STATUS_READ_TOO_MUCH};
-use crate::common::{str2wcstring, wcs2string, EMPTY_STRING};
+use crate::common::{EMPTY_STRING, bytes2wcstring, wcs2bytes};
 use crate::fd_monitor::{Callback, FdMonitor, FdMonitorItemId};
 use crate::fds::{
-    make_autoclose_pipes, make_fd_nonblocking, wopen_cloexec, AutoCloseFd, PIPE_ERROR,
+    AutoCloseFd, PIPE_ERROR, make_autoclose_pipes, make_fd_nonblocking, wopen_cloexec,
 };
-use crate::flog::{should_flog, FLOG, FLOGF};
+use crate::flog::{FLOG, FLOGF, should_flog};
 use crate::nix::isatty;
 use crate::path::path_apply_working_directory;
 use crate::proc::JobGroupRef;
@@ -541,9 +541,9 @@ impl IoChain {
                     let next: &wstr = wdirname(dname);
                     if let Ok(md) = wstat(next) {
                         if !md.is_dir() {
-                            FLOGF!(warning, "Path '%ls' is not a directory", next);
+                            FLOGF!(warning, "Path '%s' is not a directory", next);
                         } else {
-                            FLOGF!(warning, "Path '%ls' does not exist", dname);
+                            FLOGF!(warning, "Path '%s' does not exist", dname);
                         }
                         break;
                     }
@@ -583,10 +583,10 @@ impl IoChain {
                         Err(err) => {
                             if oflags.contains(OFlag::O_EXCL) && err == nix::Error::EEXIST {
                                 FLOGF!(warning, NOCLOB_ERROR, spec.target);
-                            } else if spec.mode != RedirectionMode::try_input {
-                                if should_flog!(warning) {
-                                    print_error(errno::errno().0, &spec.target);
-                                }
+                            } else if spec.mode != RedirectionMode::try_input
+                                && should_flog!(warning)
+                            {
+                                print_error(errno::errno().0, &spec.target);
                             }
                             // If opening a file fails, insert a closed FD instead of the file redirection
                             // and return false. This lets execution potentially recover and at least gives
@@ -631,12 +631,12 @@ impl IoChain {
         }
 
         eprintf!(
-            "Chain %s (%ld items):\n",
+            "Chain %s (%d items):\n",
             format!("{:p}", std::ptr::addr_of!(self)),
             self.0.len()
         );
         for (i, io) in self.0.iter().enumerate() {
-            eprintf!("\t%lu: fd:%d, ", i, io.fd());
+            eprintf!("\t%u: fd:%d, ", i, io.fd());
             io.print();
         }
     }
@@ -727,11 +727,14 @@ impl OutputStream {
         self.append(wstr::from_char_slice(&[c]))
     }
 
+    pub fn append_narrow(&mut self, s: &str) -> bool {
+        self.append(bytes2wcstring(s.as_bytes()))
+    }
     // Append data from a narrow buffer, widening it.
     pub fn append_narrow_buffer(&mut self, buffer: &SeparatedBuffer) -> bool {
         for rhs_elem in buffer.elements() {
             if !self.append_with_separation(
-                &str2wcstring(&rhs_elem.contents),
+                &bytes2wcstring(&rhs_elem.contents),
                 rhs_elem.separation,
                 false,
             ) {
@@ -745,7 +748,7 @@ impl OutputStream {
 impl Output for OutputStream {
     fn write_bytes(&mut self, command_part: &[u8]) {
         // TODO Retry on interrupt.
-        self.append(str2wcstring(command_part));
+        self.append(bytes2wcstring(command_part));
     }
 }
 
@@ -834,7 +837,7 @@ impl BufferedOutputStream {
         Self { buffer }
     }
     fn append(&mut self, s: &wstr) -> bool {
-        self.buffer.append(&wcs2string(s), SeparationType::inferred)
+        self.buffer.append(&wcs2bytes(s), SeparationType::inferred)
     }
     fn append_with_separation(
         &mut self,
@@ -842,7 +845,7 @@ impl BufferedOutputStream {
         typ: SeparationType,
         _want_newline: bool,
     ) -> bool {
-        self.buffer.append(&wcs2string(s), typ)
+        self.buffer.append(&wcs2bytes(s), typ)
     }
     fn flush_and_check_error(&mut self) -> libc::c_int {
         if self.buffer.discarded() {
@@ -910,8 +913,8 @@ impl<'a> IoStreams<'a> {
 }
 
 /// File redirection error message.
-const FILE_ERROR: &wstr = L!("An error occurred while redirecting file '%ls'");
-const NOCLOB_ERROR: &wstr = L!("The file '%ls' already exists");
+const FILE_ERROR: &wstr = L!("An error occurred while redirecting file '%s'");
+const NOCLOB_ERROR: &wstr = L!("The file '%s' already exists");
 
 /// Base open mode to pass to calls to open.
 const OPEN_MASK: Mode = Mode::from_bits_truncate(0o666);
