@@ -1,10 +1,10 @@
 use libc::VERASE;
 
 use crate::{
-    common::{escape_string, EscapeFlags, EscapeStringStyle},
+    common::{EscapeFlags, EscapeStringStyle, escape_string},
     fallback::fish_wcwidth,
     flog::FloggableDebug,
-    future_feature_flags::{test as feature_test, FeatureFlag},
+    future_feature_flags::{FeatureFlag, test as feature_test},
     reader::safe_get_terminal_mode_on_startup,
     wchar::{decode_byte_from_char, prelude::*},
     wutil::fish_wcstoul,
@@ -285,7 +285,7 @@ pub(crate) fn parse_keys(value: &wstr) -> Result<Vec<Key>, WString> {
                             "unknown modifier '%s' in '%s'",
                             modifier,
                             escape_nonprintables(full_key_name)
-                        ))
+                        ));
                     }
                 }
             }
@@ -447,7 +447,7 @@ pub fn char_to_symbol(c: char, is_first_in_token: bool) -> WString {
         // Unmapped key from https://sw.kovidgoyal.net/kitty/keyboard-protocol/#functional-key-definitions
         sprintf!(=> buf, "\\u%04X", u32::from(c));
     } else if fish_wcwidth(c) > 0 {
-        sprintf!(=> buf, "%lc", c);
+        sprintf!(=> buf, "%c", c);
     } else if c <= '\u{FFFF}' {
         // BMP Unicode character
         sprintf!(=> buf, "\\u%04X", u32::from(c));
@@ -456,3 +456,26 @@ pub fn char_to_symbol(c: char, is_first_in_token: bool) -> WString {
     }
     buff
 }
+
+#[cfg(test)]
+mod tests {
+    use crate::key::{self, Key, ctrl, function_key, parse_keys};
+    use crate::wchar::prelude::*;
+
+    #[test]
+    fn test_parse_key() {
+        assert_eq!(
+            parse_keys(L!("escape")),
+            Ok(vec![Key::from_raw(key::Escape)])
+        );
+        assert_eq!(parse_keys(L!("\x1b")), Ok(vec![Key::from_raw(key::Escape)]));
+        assert_eq!(parse_keys(L!("ctrl-a")), Ok(vec![ctrl('a')]));
+        assert_eq!(parse_keys(L!("\x01")), Ok(vec![ctrl('a')]));
+        assert!(parse_keys(L!("f0")).is_err());
+        assert_eq!(
+            parse_keys(L!("f1")),
+            Ok(vec![Key::from_raw(function_key(1))])
+        );
+        assert!(parse_keys(L!("F1")).is_err());
+    }
+}
