@@ -1,28 +1,5 @@
 #![cfg_attr(feature = "benchmark", feature(test))]
 #![allow(non_camel_case_types)]
-#![allow(non_upper_case_globals)]
-#![allow(unknown_lints)]
-#![allow(unstable_name_collisions)]
-#![allow(rustdoc::private_intra_doc_links)]
-#![allow(clippy::bool_assert_comparison)]
-#![allow(clippy::collapsible_if)]
-#![allow(clippy::comparison_chain)]
-#![allow(clippy::derivable_impls)]
-#![allow(clippy::doc_lazy_continuation)]
-#![allow(clippy::doc_overindented_list_items)]
-#![allow(clippy::field_reassign_with_default)]
-#![allow(clippy::get_first)]
-#![allow(clippy::if_same_then_else)]
-#![allow(clippy::incompatible_msrv)]
-#![allow(clippy::len_without_is_empty)]
-#![allow(clippy::manual_is_ascii_check)]
-#![allow(clippy::manual_range_contains)]
-#![allow(clippy::needless_lifetimes)]
-#![allow(clippy::needless_return)]
-#![allow(clippy::new_without_default)]
-#![allow(clippy::option_map_unit_fn)]
-#![allow(clippy::too_many_arguments)]
-#![allow(clippy::uninlined_format_args)]
 
 pub const BUILD_VERSION: &str = env!("FISH_BUILD_VERSION");
 
@@ -53,7 +30,6 @@ pub mod flog;
 pub mod fork_exec;
 pub mod fs;
 pub mod function;
-pub mod future;
 pub mod future_feature_flags;
 pub mod global_safety;
 pub mod highlight;
@@ -64,8 +40,6 @@ pub mod io;
 pub mod job_group;
 pub mod key;
 pub mod kill;
-#[allow(non_snake_case)]
-pub mod libc;
 pub mod locale;
 pub mod nix;
 pub mod null_terminated_array;
@@ -83,10 +57,10 @@ pub mod print_help;
 pub mod proc;
 pub mod re;
 pub mod reader;
-pub mod reader_history_search;
 pub mod redirection;
 pub mod screen;
 pub mod signal;
+pub mod stdx;
 pub mod terminal;
 pub mod termsize;
 pub mod text_face;
