@@ -1,19 +1,16 @@
 //! Pager support.
 
-use std::collections::hash_map::Entry;
 use std::collections::HashMap;
+use std::collections::hash_map::Entry;
 
 use crate::common::{
-    escape_string, get_ellipsis_char, get_ellipsis_str, EscapeFlags, EscapeStringStyle,
+    EscapeFlags, EscapeStringStyle, escape_string, get_ellipsis_char, get_ellipsis_str,
 };
 use crate::complete::Completion;
 use crate::editable_line::EditableLine;
-#[allow(unused_imports)]
-use crate::future::IsSomeAnd;
-use crate::highlight::{highlight_shell, HighlightRole, HighlightSpec};
-use crate::libc::MB_CUR_MAX;
+use crate::highlight::{HighlightRole, HighlightSpec, highlight_shell};
 use crate::operation_context::OperationContext;
-use crate::screen::{wcswidth_rendered, wcwidth_rendered, CharOffset, Line, ScreenData};
+use crate::screen::{CharOffset, Line, ScreenData, wcswidth_rendered, wcwidth_rendered};
 use crate::termsize::Termsize;
 use crate::wchar::prelude::*;
 use crate::wcstringutil::string_fuzzy_match_string;
@@ -273,7 +270,7 @@ impl Pager {
         assert_ne!(rendering.remaining_to_disclose, 1);
         if rendering.remaining_to_disclose > 1 {
             progress_text = wgettext_fmt!(
-                "%lsand %lu more rows",
+                "%sand %u more rows",
                 get_ellipsis_str(),
                 rendering.remaining_to_disclose
             );
@@ -282,7 +279,7 @@ impl Pager {
             // to present things as 1-indexed. We do not add 1 to stop_row or row_count because
             // these are the "past the last value".
             progress_text =
-                wgettext_fmt!("rows %lu to %lu of %lu", start_row + 1, stop_row, row_count);
+                wgettext_fmt!("rows %u to %u of %u", start_row + 1, stop_row, row_count);
         } else if self.search_field_shown && self.completion_infos.is_empty() {
             // Everything is filtered.
             progress_text = wgettext!("(no matches)").to_owned();
@@ -315,9 +312,10 @@ impl Pager {
         let mut search_field_text = self.search_field_line.text().to_owned();
         // Append spaces to make it at least the required width.
         if search_field_text.len() < PAGER_SEARCH_FIELD_WIDTH {
-            search_field_text.extend(
-                std::iter::repeat(' ').take(PAGER_SEARCH_FIELD_WIDTH - search_field_text.len()),
-            );
+            search_field_text.extend(std::iter::repeat_n(
+                ' ',
+                PAGER_SEARCH_FIELD_WIDTH - search_field_text.len(),
+            ));
         }
         let search_field = rendering.screen_data.insert_line_at_index(0);
 
@@ -403,6 +401,7 @@ impl Pager {
     /// \param row_stop the row after the last row to print
     /// \param prefix The string to print before each completion
     /// \param lst The list of completions to print
+    #[allow(clippy::too_many_arguments)]
     fn completion_print(
         &self,
         cols: usize,
@@ -609,7 +608,7 @@ impl Pager {
             // No description, or it won't fit. Just add spaces.
             print_max(
                 offset_in_cmdline,
-                &WString::from_iter(std::iter::repeat(' ').take(desc_remaining)),
+                &WString::from_iter(std::iter::repeat_n(' ', desc_remaining)),
                 bg,
                 desc_remaining,
                 false,
@@ -651,8 +650,8 @@ impl Pager {
 
     // Sets the terminal size.
     pub fn set_term_size(&mut self, ts: &Termsize) {
-        self.available_term_width = usize::try_from(ts.width).unwrap_or_default();
-        self.available_term_height = usize::try_from(ts.height).unwrap_or_default();
+        self.available_term_width = ts.width();
+        self.available_term_height = ts.height();
     }
 
     // Changes the selected completion in the given direction according to the layout of the given
@@ -910,7 +909,7 @@ impl Pager {
         rendering.search_field_line = self.search_field_line.clone();
         for cols in (1..=PAGER_MAX_COLS).rev() {
             // Initially empty rendering.
-            rendering.screen_data.resize(0);
+            rendering.screen_data.clear_lines();
 
             // Determine how many rows we would need if we had 'cols' columns. Then determine how many
             // columns we want from that. For example, say we had 19 completions. We can fit them into 6
@@ -1232,19 +1231,16 @@ fn process_completions_into_infos(lst: &[Completion]) -> Vec<PagerComp> {
                 EscapeFlags::NO_PRINTABLES | EscapeFlags::NO_QUOTED | EscapeFlags::SYMBOLIC,
             ),
         ));
-        if comp.replaces_line()
-            // HACK We want to render a full shell command, with syntax highlighting.  Above we
-            // escape nonprintables, which might make the rendered command longer than the original
-            // completion. In that case we get wrong colors.  However this should only happen in
-            // contrived cases, since our symbolic escaping uses a single character to represent
-            // newline and tab characters; other nonprintables are extremely rare in a command
-            // line. It will only be common for single-byte locales where we don't
-            // use Unicode characters for escaping, so just disable those here.
-            // We should probably fix this by first highlighting the original completion, and
-            // then writing a variant of escape_string() that adjusts highlighting according
-            // so it matches the escaped string.
-            && MB_CUR_MAX() > 1
-        {
+        // HACK We want to render a full shell command, with syntax highlighting.  Above we
+        // escape nonprintables, which might make the rendered command longer than the original
+        // completion. In that case we get wrong colors.  However this should only happen in
+        // contrived cases, since our symbolic escaping uses a single character to represent
+        // newline and tab characters; other nonprintables are extremely rare in a command
+        // line.
+        // We should probably fix this by first highlighting the original completion, and
+        // then writing a variant of escape_string() that adjusts highlighting according
+        // so it matches the escaped string.
+        if comp.replaces_line() {
             highlight_shell(
                 &comp.completion,
                 &mut comp_info.colors,
@@ -1264,3 +1260,219 @@ fn process_completions_into_infos(lst: &[Completion]) -> Vec<PagerComp> {
     }
     result
 }
+
+#[cfg(test)]
+mod tests {
+    use super::{Pager, SelectionMotion};
+    use crate::common::get_ellipsis_char;
+    use crate::complete::{CompleteFlags, Completion};
+    use crate::termsize::Termsize;
+    use crate::tests::prelude::*;
+    use crate::wchar::prelude::*;
+    use crate::wcstringutil::StringFuzzyMatch;
+    use std::num::NonZeroU16;
+
+    #[test]
+    #[serial]
+    fn test_pager_navigation() {
+        let _cleanup = test_init();
+        // Generate 19 strings of width 10. There's 2 spaces between completions, and our term size is
+        // 80; these can therefore fit into 6 columns (6 * 12 - 2 = 70) or 5 columns (58) but not 7
+        // columns (7 * 12 - 2 = 82).
+        //
+        // You can simulate this test by creating 19 files named "file00.txt" through "file_18.txt".
+        let mut completions = vec![];
+        for _ in 0..19 {
+            completions.push(Completion::new(
+                L!("abcdefghij").to_owned(),
+                "".into(),
+                StringFuzzyMatch::exact_match(),
+                CompleteFlags::default(),
+            ));
+        }
+
+        let mut pager = Pager::default();
+        pager.set_completions(&completions, true);
+        pager.set_term_size(&Termsize::defaults());
+        let mut render = pager.render();
+
+        assert_eq!(render.term_width, Some(80));
+        assert_eq!(render.term_height, Some(24));
+
+        let rows = 4;
+        let cols = 5;
+
+        // We have 19 completions. We can fit into 6 columns with 4 rows or 5 columns with 4 rows; the
+        // second one is better and so is what we ought to have picked.
+        assert_eq!(render.rows, rows);
+        assert_eq!(render.cols, cols);
+
+        // Initially expect to have no completion index.
+        assert!(render.selected_completion_idx.is_none());
+
+        // Here are navigation directions and where we expect the selection to be.
+        macro_rules! validate {
+            ($pager:ident, $render:ident, $dir:expr, $sel:expr) => {
+                $pager.select_next_completion_in_direction($dir, &$render);
+                $pager.update_rendering(&mut $render);
+                assert_eq!(
+                    Some($sel),
+                    $render.selected_completion_idx,
+                    "For command {:?}",
+                    $dir
+                );
+            };
+        }
+
+        // Tab completion to get into the list.
+        validate!(pager, render, SelectionMotion::Next, 0);
+        // Westward motion in upper left goes to the last filled column in the last row.
+        validate!(pager, render, SelectionMotion::West, 15);
+        // East goes back.
+        validate!(pager, render, SelectionMotion::East, 0);
+        validate!(pager, render, SelectionMotion::West, 15);
+        validate!(pager, render, SelectionMotion::West, 11);
+        validate!(pager, render, SelectionMotion::East, 15);
+        validate!(pager, render, SelectionMotion::East, 0);
+        // "Next" motion goes down the column.
+        validate!(pager, render, SelectionMotion::Next, 1);
+        validate!(pager, render, SelectionMotion::Next, 2);
+        validate!(pager, render, SelectionMotion::West, 17);
+        validate!(pager, render, SelectionMotion::East, 2);
+        validate!(pager, render, SelectionMotion::East, 6);
+        validate!(pager, render, SelectionMotion::East, 10);
+        validate!(pager, render, SelectionMotion::East, 14);
+        validate!(pager, render, SelectionMotion::East, 18);
+        validate!(pager, render, SelectionMotion::West, 14);
+        validate!(pager, render, SelectionMotion::East, 18);
+        // Eastward motion wraps back to the upper left, westward goes to the prior column.
+        validate!(pager, render, SelectionMotion::East, 3);
+        validate!(pager, render, SelectionMotion::East, 7);
+        validate!(pager, render, SelectionMotion::East, 11);
+        validate!(pager, render, SelectionMotion::East, 15);
+        // Pages.
+        validate!(pager, render, SelectionMotion::PageNorth, 12);
+        validate!(pager, render, SelectionMotion::PageSouth, 15);
+        validate!(pager, render, SelectionMotion::PageNorth, 12);
+        validate!(pager, render, SelectionMotion::East, 16);
+        validate!(pager, render, SelectionMotion::PageSouth, 18);
+        validate!(pager, render, SelectionMotion::East, 3);
+        validate!(pager, render, SelectionMotion::North, 2);
+        validate!(pager, render, SelectionMotion::PageNorth, 0);
+        validate!(pager, render, SelectionMotion::PageSouth, 3);
+    }
+
+    #[test]
+    #[serial]
+    fn test_pager_layout() {
+        let _cleanup = test_init();
+        // These tests are woefully incomplete
+        // They only test the truncation logic for a single completion
+
+        let rendered_line = |pager: &mut Pager, width: u16| {
+            pager.set_term_size(&Termsize::new(
+                NonZeroU16::new(width).unwrap(),
+                Termsize::DEFAULT_HEIGHT,
+            ));
+            let rendering = pager.render();
+            let sd = &rendering.screen_data;
+            assert_eq!(sd.line_count(), 1);
+            let line = sd.line(0);
+            WString::from(Vec::from_iter((0..line.len()).map(|i| line.char_at(i))))
+        };
+        let compute_expected = |expected: &wstr| {
+            let ellipsis_char = get_ellipsis_char();
+            if ellipsis_char != '\u{2026}' {
+                // hack: handle the case where ellipsis is not L'\x2026'
+                expected.replace(L!("\u{2026}"), wstr::from_char_slice(&[ellipsis_char]))
+            } else {
+                expected.to_owned()
+            }
+        };
+
+        macro_rules! validate {
+            ($pager:expr, $width:expr, $expected:expr) => {
+                assert_eq!(
+                    rendered_line($pager, $width),
+                    compute_expected($expected),
+                    "width {}",
+                    $width
+                );
+            };
+        }
+
+        let mut pager = Pager::default();
+
+        // These test cases have equal completions and descriptions
+        let c1s = vec![Completion::new(
+            L!("abcdefghij").to_owned(),
+            L!("1234567890").to_owned(),
+            StringFuzzyMatch::exact_match(),
+            CompleteFlags::default(),
+        )];
+        pager.set_completions(&c1s, true);
+
+        validate!(&mut pager, 26, L!("abcdefghij  (1234567890)"));
+        validate!(&mut pager, 25, L!("abcdefghij  (1234567890)"));
+        validate!(&mut pager, 24, L!("abcdefghij  (1234567890)"));
+        validate!(&mut pager, 23, L!("abcdefghij  (12345678…)"));
+        validate!(&mut pager, 22, L!("abcdefghij  (1234567…)"));
+        validate!(&mut pager, 21, L!("abcdefghij  (123456…)"));
+        validate!(&mut pager, 20, L!("abcdefghij  (12345…)"));
+        validate!(&mut pager, 19, L!("abcdefghij  (1234…)"));
+        validate!(&mut pager, 18, L!("abcdefgh…  (1234…)"));
+        validate!(&mut pager, 17, L!("abcdefg…  (1234…)"));
+        validate!(&mut pager, 16, L!("abcdefg…  (123…)"));
+
+        // These test cases have heavyweight completions
+        let c2s = vec![Completion::new(
+            L!("abcdefghijklmnopqrs").to_owned(),
+            L!("1").to_owned(),
+            StringFuzzyMatch::exact_match(),
+            CompleteFlags::default(),
+        )];
+        pager.set_completions(&c2s, true);
+        validate!(&mut pager, 26, L!("abcdefghijklmnopqrs  (1)"));
+        validate!(&mut pager, 25, L!("abcdefghijklmnopqrs  (1)"));
+        validate!(&mut pager, 24, L!("abcdefghijklmnopqrs  (1)"));
+        validate!(&mut pager, 23, L!("abcdefghijklmnopq…  (1)"));
+        validate!(&mut pager, 22, L!("abcdefghijklmnop…  (1)"));
+        validate!(&mut pager, 21, L!("abcdefghijklmno…  (1)"));
+        validate!(&mut pager, 20, L!("abcdefghijklmn…  (1)"));
+        validate!(&mut pager, 19, L!("abcdefghijklm…  (1)"));
+        validate!(&mut pager, 18, L!("abcdefghijkl…  (1)"));
+        validate!(&mut pager, 17, L!("abcdefghijk…  (1)"));
+        validate!(&mut pager, 16, L!("abcdefghij…  (1)"));
+
+        // These test cases have no descriptions
+        let c3s = vec![Completion::new(
+            L!("abcdefghijklmnopqrst").to_owned(),
+            L!("").to_owned(),
+            StringFuzzyMatch::exact_match(),
+            CompleteFlags::default(),
+        )];
+        pager.set_completions(&c3s, true);
+        validate!(&mut pager, 26, L!("abcdefghijklmnopqrst"));
+        validate!(&mut pager, 25, L!("abcdefghijklmnopqrst"));
+        validate!(&mut pager, 24, L!("abcdefghijklmnopqrst"));
+        validate!(&mut pager, 23, L!("abcdefghijklmnopqrst"));
+        validate!(&mut pager, 22, L!("abcdefghijklmnopqrst"));
+        validate!(&mut pager, 21, L!("abcdefghijklmnopqrst"));
+        validate!(&mut pager, 20, L!("abcdefghijklmnopqrst"));
+        validate!(&mut pager, 19, L!("abcdefghijklmnopqr…"));
+        validate!(&mut pager, 18, L!("abcdefghijklmnopq…"));
+        validate!(&mut pager, 17, L!("abcdefghijklmnop…"));
+        validate!(&mut pager, 16, L!("abcdefghijklmno…"));
+
+        // Newlines in prefix
+        let c4s = vec![Completion::new(
+            L!("Hello").to_owned(),
+            L!("").to_owned(),
+            StringFuzzyMatch::exact_match(),
+            CompleteFlags::default(),
+        )];
+        pager.set_prefix(L!("{\\\n"), false); // }
+        pager.set_completions(&c4s, true);
+        validate!(&mut pager, 30, L!("{\\␊Hello")); // }
+    }
+}
