@@ -1,24 +1,23 @@
 //! Provides the "linkage" between an ast and actual execution structures (job_t, etc.).
 
 use crate::ast::{
-    self, unescape_keyword, BlockStatementHeader, Keyword, Leaf, Node, Statement, Token,
+    self, BlockStatementHeader, Keyword, Leaf, Node, Statement, Token, unescape_keyword,
 };
 use crate::builtins;
 use crate::builtins::shared::{
-    builtin_exists, BUILTIN_ERR_VARNAME, STATUS_CMD_ERROR, STATUS_CMD_OK, STATUS_CMD_UNKNOWN,
-    STATUS_EXPAND_ERROR, STATUS_ILLEGAL_CMD, STATUS_INVALID_ARGS, STATUS_NOT_EXECUTABLE,
-    STATUS_UNMATCHED_WILDCARD,
+    BUILTIN_ERR_VARNAME, STATUS_CMD_ERROR, STATUS_CMD_OK, STATUS_CMD_UNKNOWN, STATUS_EXPAND_ERROR,
+    STATUS_ILLEGAL_CMD, STATUS_INVALID_ARGS, STATUS_NOT_EXECUTABLE, STATUS_UNMATCHED_WILDCARD,
+    builtin_exists,
 };
 use crate::common::{
-    escape, should_suppress_stderr_for_tests, truncate_at_nul, valid_var_name, ScopeGuard,
-    ScopeGuarding, ScopedRefCell,
+    ScopeGuard, ScopeGuarding, ScopedRefCell, escape, truncate_at_nul, valid_var_name,
 };
 use crate::complete::CompletionList;
 use crate::env::{EnvMode, EnvStackSetResult, EnvVar, EnvVarFlags, Environment, Statuses};
 use crate::event::{self, Event};
 use crate::exec::exec_job;
 use crate::expand::{
-    expand_one, expand_string, expand_to_command_and_args, ExpandFlags, ExpandResultCode,
+    ExpandFlags, ExpandResultCode, expand_one, expand_string, expand_to_command_and_args,
 };
 use crate::flog::FLOG;
 use crate::function;
@@ -26,33 +25,33 @@ use crate::io::{IoChain, IoStreams, OutputStream, StringOutputStream};
 use crate::job_group::JobGroup;
 use crate::operation_context::OperationContext;
 use crate::parse_constants::{
-    parse_error_offset_source_start, ParseError, ParseErrorCode, ParseErrorList, ParseKeyword,
-    ParseTokenType, StatementDecoration, CALL_STACK_LIMIT_EXCEEDED_ERR_MSG, ERROR_TIME_BACKGROUND,
+    CALL_STACK_LIMIT_EXCEEDED_ERR_MSG, ERROR_TIME_BACKGROUND,
     FAILED_EXPANSION_VARIABLE_NAME_ERR_MSG, ILLEGAL_FD_ERR_MSG, INFINITE_FUNC_RECURSION_ERR_MSG,
-    WILDCARD_ERR_MSG,
+    ParseError, ParseErrorCode, ParseErrorList, ParseKeyword, ParseTokenType, StatementDecoration,
+    WILDCARD_ERR_MSG, parse_error_offset_source_start,
 };
 use crate::parse_tree::{LineCounter, NodeRef, ParsedSourceRef};
 use crate::parse_util::{
-    parse_util_locate_cmdsubst_range, parse_util_unescape_wildcards,
-    MaybeParentheses::CommandSubstitution,
+    MaybeParentheses::CommandSubstitution, parse_util_locate_cmdsubst_range,
+    parse_util_unescape_wildcards,
 };
 use crate::parser::{Block, BlockData, BlockId, BlockType, LoopStatus, Parser, ProfileItem};
 use crate::parser_keywords::parser_keywords_is_subcommand;
 use crate::path::{path_as_implicit_cd, path_try_get_path};
 use crate::proc::{
-    get_job_control_mode, job_reap, no_exec, ConcreteAssignment, Job, JobControl, JobProperties,
-    JobRef, Process, ProcessType,
+    ConcreteAssignment, Job, JobControl, JobProperties, JobRef, Process, ProcessType,
+    get_job_control_mode, job_reap, no_exec,
 };
 use crate::reader::fish_is_unwinding_for_exit;
 use crate::redirection::{RedirectionMode, RedirectionSpec, RedirectionSpecList};
 use crate::signal::Signal;
 use crate::timer::push_timer;
-use crate::tokenizer::{variable_assignment_equals_pos, PipeOrRedir, TokenType};
+use crate::tokenizer::{PipeOrRedir, TokenType, variable_assignment_equals_pos};
 use crate::trace::{trace_if_enabled, trace_if_enabled_with_args};
 use crate::wchar::prelude::*;
 use crate::wchar_ext::WExt;
 use crate::wildcard::wildcard_match;
-use libc::{c_int, ENOTDIR, EXIT_SUCCESS, STDERR_FILENO, STDOUT_FILENO};
+use libc::{ENOTDIR, EXIT_SUCCESS, STDERR_FILENO, STDOUT_FILENO, c_int};
 use std::io::ErrorKind;
 use std::rc::Rc;
 use std::sync::Arc;
@@ -90,6 +89,9 @@ pub struct ExecutionContext<'a> {
     /// The block IO chain.
     /// For example, in `begin; foo ; end < file.txt` this would have the 'file.txt' IO.
     block_io: IoChain,
+
+    /// Hack to supress non-redirectable stderr in some unit tests.
+    test_only_suppress_stderr: bool,
 }
 
 // Report an error, setting $status to `status`. Always returns
@@ -119,12 +121,14 @@ impl<'a> ExecutionContext<'a> {
         pstree: ParsedSourceRef,
         block_io: IoChain,
         line_counter: &'a ScopedRefCell<LineCounter<ast::JobPipeline>>,
+        test_only_suppress_stderr: bool,
     ) -> Self {
         Self {
             pstree,
             cancel_signal: None,
             line_counter,
             block_io,
+            test_only_suppress_stderr,
         }
     }
 
@@ -242,7 +246,7 @@ impl<'a> ExecutionContext<'a> {
             let backtrace_and_desc = ctx.parser().get_backtrace(&self.pstree().src, error_list);
 
             // Print it.
-            if !should_suppress_stderr_for_tests() {
+            if !self.test_only_suppress_stderr {
                 eprintf!("%s", backtrace_and_desc);
             }
 
@@ -277,7 +281,7 @@ impl<'a> ExecutionContext<'a> {
                         ctx,
                         STATUS_NOT_EXECUTABLE,
                         &statement.command,
-                        "Unknown command. A component of '%ls' is not a directory. Check your $PATH.",
+                        "Unknown command. A component of '%s' is not a directory. Check your $PATH.",
                         cmd
                     );
                 } else {
@@ -286,7 +290,7 @@ impl<'a> ExecutionContext<'a> {
                         ctx,
                         STATUS_NOT_EXECUTABLE,
                         &statement.command,
-                        "Unknown command. A component of '%ls' is not a directory.",
+                        "Unknown command. A component of '%s' is not a directory.",
                         cmd
                     );
                 }
@@ -297,7 +301,7 @@ impl<'a> ExecutionContext<'a> {
                 ctx,
                 STATUS_NOT_EXECUTABLE,
                 &statement.command,
-                "Unknown command. '%ls' exists but is not an executable file.",
+                "Unknown command. '%s' exists but is not an executable file.",
                 cmd
             );
         }
@@ -395,7 +399,7 @@ impl<'a> ExecutionContext<'a> {
         let forbidden_function_name = parent_fn_name;
 
         // Get the first job in the job list.
-        let jc = &jobs.get(0)?;
+        let jc = &jobs.first()?;
         let job = &jc.job;
 
         // Helper to return if a statement is infinitely recursive in this function.
@@ -421,11 +425,7 @@ impl<'a> ExecutionContext<'a> {
                     None,
                 )
                 && &cmd == forbidden_function_name;
-            if forbidden {
-                Some(dc)
-            } else {
-                None
-            }
+            if forbidden { Some(dc) } else { None }
         };
 
         // Check main statement.
@@ -903,7 +903,7 @@ impl<'a> ExecutionContext<'a> {
                 ctx,
                 STATUS_INVALID_ARGS,
                 header.var_name,
-                "%ls: %ls: cannot overwrite read-only variable",
+                "%s: %s: cannot overwrite read-only variable",
                 "for",
                 for_var_name
             );
@@ -1094,7 +1094,7 @@ impl<'a> ExecutionContext<'a> {
                         ctx,
                         STATUS_INVALID_ARGS,
                         &statement.argument,
-                        "switch: Expected at most one argument, got %lu\n",
+                        "switch: Expected at most one argument, got %u\n",
                         switch_values_expanded.len()
                     );
                 }
@@ -1279,14 +1279,16 @@ impl<'a> ExecutionContext<'a> {
             &mut streams,
             &mut shim_arguments,
             NodeRef::new(Arc::clone(self.pstree()), statement),
-        );
+        )
+        .err()
+        .unwrap_or(STATUS_CMD_OK);
 
         ctx.parser().libdata_mut().status_count += 1;
         ctx.parser().set_last_statuses(Statuses::just(err_code));
 
         let errtext = errs.contents();
         if !errtext.is_empty() {
-            report_error!(self, ctx, err_code, header, "%ls", errtext);
+            report_error!(self, ctx, err_code, header, "%s", errtext);
         }
         result
     }
@@ -1419,7 +1421,7 @@ impl<'a> ExecutionContext<'a> {
                         ctx,
                         STATUS_INVALID_ARGS,
                         redir_node,
-                        "Invalid redirection: %ls",
+                        "Invalid redirection: %s",
                         self.node_source(redir_node)
                     );
                 }
@@ -1445,7 +1447,7 @@ impl<'a> ExecutionContext<'a> {
                     ctx,
                     STATUS_INVALID_ARGS,
                     redir_node,
-                    "Invalid redirection target: %ls",
+                    "Invalid redirection target: %s",
                     target
                 );
                 if oper.mode == RedirectionMode::input && {
@@ -1462,7 +1464,9 @@ impl<'a> ExecutionContext<'a> {
                             _ => false,
                         }
                 } {
-                    eprintf!("If you wish to use process substitution, consider the psub command, see: `help psub`\n");
+                    eprintf!(
+                        "If you wish to use process substitution, consider the psub command, see: `help psub`\n"
+                    );
                 }
                 return error_ret;
             }
@@ -1481,7 +1485,7 @@ impl<'a> ExecutionContext<'a> {
                     ctx,
                     STATUS_INVALID_ARGS,
                     redir_node,
-                    "Requested redirection to '%ls', which is not a valid file descriptor",
+                    "Requested redirection to '%s', which is not a valid file descriptor",
                     &spec.target
                 );
             }
@@ -1591,8 +1595,10 @@ impl<'a> ExecutionContext<'a> {
             return result;
         }
 
-        let mut props = JobProperties::default();
-        props.initial_background = job_is_background;
+        let mut props = JobProperties {
+            initial_background: job_is_background,
+            ..Default::default()
+        };
         {
             let parser = ctx.parser();
             let sc = parser.scope();
