@@ -7,8 +7,8 @@ use std::sync::Arc;
 use crate::ast::{self, Ast, JobList, Node};
 use crate::common::{assert_send, assert_sync};
 use crate::parse_constants::{
-    token_type_user_presentable_description, ParseErrorCode, ParseErrorList, ParseKeyword,
-    ParseTokenType, ParseTreeFlags, SourceOffset, SourceRange, SOURCE_OFFSET_INVALID,
+    ParseErrorCode, ParseErrorList, ParseKeyword, ParseTokenType, ParseTreeFlags,
+    SOURCE_OFFSET_INVALID, SourceOffset, SourceRange, token_type_user_presentable_description,
 };
 use crate::tokenizer::TokenizerError;
 use crate::wchar::prelude::*;
@@ -74,7 +74,7 @@ impl ParseToken {
     pub fn describe(&self) -> WString {
         let mut result = self.typ.to_wstr().to_owned();
         if self.keyword != ParseKeyword::None {
-            sprintf!(=> &mut result, " <%ls>", self.keyword.to_wstr())
+            sprintf!(=> &mut result, " <%s>", self.keyword.to_wstr())
         }
         result
     }
@@ -253,6 +253,7 @@ impl<NodeType: Node> LineCounter<NodeType> {
         }
 
         // We want to return the number of newlines at offsets less than the given offset.
+        #[allow(clippy::comparison_chain)] // TODO(MSRV>=1.90) for old clippy
         if offset > self.cached_offset {
             // Add one for every newline we find in the range [cached_offset, offset).
             // The codegen is substantially better when using a char slice than the char iterator.
