@@ -4,11 +4,11 @@
 
 use crate::common::{wcs2osstring, wcs2zstring};
 use crate::env::{EnvMode, EnvStack, Environment, FALLBACK_PATH};
-use crate::expand::{expand_tilde, HOME_DIRECTORY};
+use crate::expand::{HOME_DIRECTORY, expand_tilde};
 use crate::flog::{FLOG, FLOGF};
 use crate::wchar::prelude::*;
 use crate::wutil::{normalize_path, path_normalize_for_cd, waccess, wdirname, wstat};
-use errno::{errno, set_errno, Errno};
+use errno::{Errno, errno, set_errno};
 use libc::{EACCES, ENOENT, ENOTDIR, F_OK, X_OK};
 use once_cell::sync::Lazy;
 use std::ffi::OsStr;
@@ -150,12 +150,12 @@ fn maybe_issue_path_warning(
     if path.is_empty() {
         FLOG!(
             warning_path,
-            wgettext_fmt!("Unable to locate the %ls directory.", which_dir)
+            wgettext_fmt!("Unable to locate the %s directory.", which_dir)
         );
         FLOG!(
             warning_path,
             wgettext_fmt!(
-                "Please set the %ls or HOME environment variable before starting fish.",
+                "Please set the %s or HOME environment variable before starting fish.",
                 xdg_var
             )
         );
@@ -164,7 +164,7 @@ fn maybe_issue_path_warning(
         FLOG!(
             warning_path,
             wgettext_fmt!(
-                "Unable to locate %ls directory derived from $%ls: '%ls'.",
+                "Unable to locate %s directory derived from $%s: '%s'.",
                 which_dir,
                 env_var,
                 path
@@ -177,7 +177,7 @@ fn maybe_issue_path_warning(
         FLOG!(
             warning_path,
             wgettext_fmt!(
-                "Please set $%ls to a directory where you have write access.",
+                "Please set $%s to a directory where you have write access.",
                 env_var
             )
         );
@@ -236,7 +236,7 @@ fn path_check_executable(path: &wstr) -> Result<(), std::io::Error> {
 
 /// Return all the paths that match the given command.
 pub fn path_get_paths(cmd: &wstr, vars: &dyn Environment) -> Vec<WString> {
-    FLOGF!(path, "path_get_paths('%ls')", cmd);
+    FLOGF!(path, "path_get_paths('%s')", cmd);
     let mut paths = vec![];
 
     // If the command has a slash, it must be an absolute or relative path and thus we don't bother
@@ -585,7 +585,7 @@ fn make_base_directory(xdg_var: &wstr, non_xdg_homepath: &wstr) -> BaseDirectory
     // the actual $HOME or $XDG_XXX directories. This prevents the tests from failing and/or stops
     // the tests polluting the user's actual $HOME if a sandbox environment has not been set up.
     {
-        use crate::common::{str2wcstring, BUILD_DIR};
+        use crate::common::{BUILD_DIR, bytes2wcstring};
         use std::path::PathBuf;
 
         let mut build_dir = PathBuf::from(BUILD_DIR);
@@ -599,7 +599,7 @@ fn make_base_directory(xdg_var: &wstr, non_xdg_homepath: &wstr) -> BaseDirectory
         };
 
         return BaseDirectory {
-            path: str2wcstring(build_dir.as_os_str().as_bytes()),
+            path: bytes2wcstring(build_dir.as_os_str().as_bytes()),
             remoteness: DirRemoteness::unknown,
             used_xdg: false,
             err,
@@ -692,51 +692,41 @@ pub fn path_remoteness(path: &wstr) -> DirRemoteness {
             }
         }
     }
-    #[cfg(not(any(target_os = "linux", cygwin)))]
+
+    // NetBSD doesn't have statfs, but MNT_LOCAL works for statvfs.
+    #[cfg(target_os = "netbsd")]
     {
-        fn remoteness_via_statfs<StatFS, Flags>(
-            statfn: unsafe extern "C" fn(*const libc::c_char, *mut StatFS) -> libc::c_int,
-            flagsfn: fn(&StatFS) -> Flags,
-            is_local_flag: u64,
-            path: &std::ffi::CStr,
-        ) -> DirRemoteness
-        where
-            u64: From<Flags>,
-        {
-            if is_local_flag == 0 {
-                return DirRemoteness::unknown;
-            }
-            let mut buf = MaybeUninit::uninit();
-            if unsafe { (statfn)(path.as_ptr(), buf.as_mut_ptr()) } < 0 {
-                return DirRemoteness::unknown;
-            }
-            let buf = unsafe { buf.assume_init() };
-            // statfs::f_flag is hard-coded as 64-bits on 32/64-bit FreeBSD but it's a (4-byte)
-            // long on 32-bit NetBSD.. and always 4-bytes on macOS (even on 64-bit builds).
-            #[allow(clippy::useless_conversion)]
-            if u64::from((flagsfn)(&buf)) & is_local_flag != 0 {
-                DirRemoteness::local
-            } else {
-                DirRemoteness::remote
-            }
+        let mut buf = MaybeUninit::uninit();
+        if unsafe { libc::statvfs(narrow.as_ptr(), buf.as_mut_ptr()) } < 0 {
+            return DirRemoteness::unknown;
+        }
+        let buf = unsafe { buf.assume_init() };
+        #[allow(clippy::useless_conversion)]
+        let flags = buf.f_flag as u64;
+        #[allow(clippy::unnecessary_cast)]
+        if flags & (libc::MNT_LOCAL as u64) != 0 {
+            DirRemoteness::local
+        } else {
+            DirRemoteness::remote
+        }
+    }
+
+    #[cfg(not(any(target_os = "linux", target_os = "netbsd", cygwin)))]
+    {
+        let mut buf = MaybeUninit::uninit();
+        if unsafe { libc::statfs(narrow.as_ptr(), buf.as_mut_ptr()) } < 0 {
+            return DirRemoteness::unknown;
+        }
+        let buf = unsafe { buf.assume_init() };
+        // statfs::f_flags types differ.
+        #[allow(clippy::useless_conversion)]
+        let flags = buf.f_flags as u64;
+        #[allow(clippy::unnecessary_cast)]
+        if flags & (libc::MNT_LOCAL as u64) != 0 {
+            DirRemoteness::local
+        } else {
+            DirRemoteness::remote
         }
-        // ST_LOCAL is a flag to statvfs, which is itself standardized.
-        // In practice the only system to define it is NetBSD.
-        #[cfg(target_os = "netbsd")]
-        let remoteness = remoteness_via_statfs(
-            libc::statvfs,
-            |stat: &libc::statvfs| stat.f_flag,
-            crate::libc::ST_LOCAL(),
-            &narrow,
-        );
-        #[cfg(not(target_os = "netbsd"))]
-        let remoteness = remoteness_via_statfs(
-            libc::statfs,
-            |stat: &libc::statfs| stat.f_flags,
-            crate::libc::MNT_LOCAL(),
-            &narrow,
-        );
-        remoteness
     }
 }
 
@@ -777,55 +767,61 @@ pub fn append_path_component(path: &mut WString, component: &wstr) {
     }
 }
 
-#[test]
-fn test_path_make_canonical() {
-    let mut path = L!("//foo//////bar/").to_owned();
-    path_make_canonical(&mut path);
-    assert_eq!(path, "/foo/bar");
+#[cfg(test)]
+mod tests {
+    use super::{path_apply_working_directory, path_make_canonical, paths_are_equivalent};
+    use crate::wchar::prelude::*;
 
-    path = L!("/").to_owned();
-    path_make_canonical(&mut path);
-    assert_eq!(path, "/");
-}
+    #[test]
+    fn test_path_make_canonical() {
+        let mut path = L!("//foo//////bar/").to_owned();
+        path_make_canonical(&mut path);
+        assert_eq!(path, "/foo/bar");
 
-#[test]
-fn test_path() {
-    let mut path = L!("//foo//////bar/").to_owned();
-    path_make_canonical(&mut path);
-    assert_eq!(&path, L!("/foo/bar"));
+        path = L!("/").to_owned();
+        path_make_canonical(&mut path);
+        assert_eq!(path, "/");
+    }
 
-    path = L!("/").to_owned();
-    path_make_canonical(&mut path);
-    assert_eq!(&path, L!("/"));
+    #[test]
+    fn test_path() {
+        let mut path = L!("//foo//////bar/").to_owned();
+        path_make_canonical(&mut path);
+        assert_eq!(&path, L!("/foo/bar"));
 
-    path = L!("/home/fishuser/").to_owned();
-    path_make_canonical(&mut path);
-    assert_eq!(&path, L!("/home/fishuser"));
+        path = L!("/").to_owned();
+        path_make_canonical(&mut path);
+        assert_eq!(&path, L!("/"));
 
-    assert!(!paths_are_equivalent(L!("/foo/bar/baz"), L!("foo/bar/baz")));
-    assert!(paths_are_equivalent(
-        L!("///foo///bar/baz"),
-        L!("/foo/bar////baz//")
-    ));
-    assert!(paths_are_equivalent(L!("/foo/bar/baz"), L!("/foo/bar/baz")));
-    assert!(paths_are_equivalent(L!("/"), L!("/")));
+        path = L!("/home/fishuser/").to_owned();
+        path_make_canonical(&mut path);
+        assert_eq!(&path, L!("/home/fishuser"));
 
-    assert_eq!(
-        path_apply_working_directory(L!("abc"), L!("/def/")),
-        L!("/def/abc")
-    );
-    assert_eq!(
-        path_apply_working_directory(L!("abc/"), L!("/def/")),
-        L!("/def/abc/")
-    );
-    assert_eq!(
-        path_apply_working_directory(L!("/abc/"), L!("/def/")),
-        L!("/abc/")
-    );
-    assert_eq!(
-        path_apply_working_directory(L!("/abc"), L!("/def/")),
-        L!("/abc")
-    );
-    assert!(path_apply_working_directory(L!(""), L!("/def/")).is_empty());
-    assert_eq!(path_apply_working_directory(L!("abc"), L!("")), L!("abc"));
+        assert!(!paths_are_equivalent(L!("/foo/bar/baz"), L!("foo/bar/baz")));
+        assert!(paths_are_equivalent(
+            L!("///foo///bar/baz"),
+            L!("/foo/bar////baz//")
+        ));
+        assert!(paths_are_equivalent(L!("/foo/bar/baz"), L!("/foo/bar/baz")));
+        assert!(paths_are_equivalent(L!("/"), L!("/")));
+
+        assert_eq!(
+            path_apply_working_directory(L!("abc"), L!("/def/")),
+            L!("/def/abc")
+        );
+        assert_eq!(
+            path_apply_working_directory(L!("abc/"), L!("/def/")),
+            L!("/def/abc/")
+        );
+        assert_eq!(
+            path_apply_working_directory(L!("/abc/"), L!("/def/")),
+            L!("/abc/")
+        );
+        assert_eq!(
+            path_apply_working_directory(L!("/abc"), L!("/def/")),
+            L!("/abc")
+        );
+        assert!(path_apply_working_directory(L!(""), L!("/def/")).is_empty());
+        assert_eq!(path_apply_working_directory(L!("abc"), L!("")), L!("abc"));
+    }
 }
