@@ -4,7 +4,7 @@
 
 use crate::ast;
 use crate::common::{
-    charptr2wcstring, escape, is_windows_subsystem_for_linux, timef, Timepoint, WSL,
+    Timepoint, WSL, charptr2wcstring, escape, is_windows_subsystem_for_linux, timef,
 };
 use crate::env::Statuses;
 use crate::event::{self, Event};
@@ -16,16 +16,16 @@ use crate::parse_tree::NodeRef;
 use crate::parser::{Block, Parser};
 use crate::reader::{fish_is_unwinding_for_exit, reader_schedule_prompt_repaint};
 use crate::redirection::RedirectionSpecList;
-use crate::signal::{signal_set_handlers_once, Signal};
-use crate::topic_monitor::{topic_monitor_principal, GenerationsList, Topic};
+use crate::signal::{Signal, signal_set_handlers_once};
+use crate::topic_monitor::{GenerationsList, Topic, topic_monitor_principal};
 use crate::wait_handle::{InternalJobId, WaitHandle, WaitHandleRef, WaitHandleStore};
 use crate::wchar::prelude::*;
 use crate::wchar_ext::ToWString;
 use crate::wutil::{wbasename, wperror};
 use libc::{
-    EXIT_SUCCESS, SIGABRT, SIGBUS, SIGCONT, SIGFPE, SIGHUP, SIGILL, SIGINT, SIGKILL, SIGPIPE,
-    SIGQUIT, SIGSEGV, SIGSYS, SIGTTOU, SIG_DFL, SIG_IGN, WCONTINUED, WEXITSTATUS, WIFCONTINUED,
-    WIFEXITED, WIFSIGNALED, WIFSTOPPED, WNOHANG, WTERMSIG, WUNTRACED, _SC_CLK_TCK,
+    _SC_CLK_TCK, EXIT_SUCCESS, SIG_DFL, SIG_IGN, SIGABRT, SIGBUS, SIGCONT, SIGFPE, SIGHUP, SIGILL,
+    SIGINT, SIGKILL, SIGPIPE, SIGQUIT, SIGSEGV, SIGSYS, SIGTTOU, WCONTINUED, WEXITSTATUS,
+    WIFCONTINUED, WIFEXITED, WIFSIGNALED, WIFSTOPPED, WNOHANG, WTERMSIG, WUNTRACED,
 };
 use once_cell::sync::Lazy;
 #[cfg(not(target_has_atomic = "64"))]
@@ -274,10 +274,13 @@ pub struct Pid(NonZeroU32);
 
 impl Pid {
     #[inline(always)]
-    pub fn new(pid: i32) -> Option<Pid> {
-        // Construct a pid from an i32, which must be at least zero.
-        assert!(pid >= 0, "Pid must be at least zero");
-        NonZeroU32::new(pid as u32).map(Pid)
+    pub fn new(pid: i32) -> Self {
+        Self(
+            u32::try_from(pid)
+                .ok()
+                .and_then(NonZeroU32::new)
+                .expect("PID must be greater than zero"),
+        )
     }
     #[inline(always)]
     pub fn get(&self) -> i32 {
@@ -439,7 +442,7 @@ impl Process {
 
     /// Returns `argv[0]`.
     pub fn argv0(&self) -> Option<&wstr> {
-        self.argv.get(0).map(|s| s.as_utfstr())
+        self.argv.first().map(|s| s.as_utfstr())
     }
 
     /// Returns the status.
@@ -518,7 +521,7 @@ impl Process {
     }
 
     /// Create a wait handle for the process.
-    /// As a process does not know its job id, we pass it in.
+    /// As a process does not know its job ID, we pass it in.
     /// Note this will return null if the process is not waitable (has no pid).
     pub fn make_wait_handle(&self, jid: InternalJobId) -> Option<WaitHandleRef> {
         let pid = self.pid()?;
@@ -646,7 +649,7 @@ impl Job {
     }
 
     /// Returns a truncated version of the job string. Used when a message has already been emitted
-    /// containing the full job string and job id, but using the job id alone would be confusing
+    /// containing the full job string and job ID, but using the job ID alone would be confusing
     /// due to reuse of freed job ids. Prevents overloading the debug comments with the full,
     /// untruncated job string when we don't care what the job is, only which of the currently
     /// running jobs it is.
@@ -782,7 +785,7 @@ impl Job {
     pub fn continue_job(&self, parser: &Parser) {
         FLOGF!(
             proc_job_run,
-            "Run job %d (%ls), %ls, %ls",
+            "Run job %d (%s), %s, %s",
             self.job_id(),
             self.command(),
             if self.is_completed() {
@@ -822,7 +825,7 @@ impl Job {
         if !self.signal(SIGCONT) {
             FLOGF!(
                 proc_pgroup,
-                "Failed to send SIGCONT to procs in job %ls",
+                "Failed to send SIGCONT to procs in job %s",
                 self.command()
             );
             return false;
@@ -888,11 +891,7 @@ impl Job {
             return None;
         }
         st.status = if self.flags().negate {
-            if laststatus == 0 {
-                1
-            } else {
-                0
-            }
+            if laststatus == 0 { 1 } else { 0 }
         } else {
             laststatus
         };
@@ -991,7 +990,7 @@ pub fn print_exit_warning_for_jobs(jobs: &JobList) {
         // Unwrap safety: we can't have a background job that doesn't have an external process and
         // external processes always have a pid set.
         printf!(
-            "%6d  %ls\n",
+            "%6d  %s\n",
             j.external_procs().next().and_then(|p| p.pid()).unwrap(),
             j.command()
         );
@@ -1234,9 +1233,10 @@ fn process_mark_finished_children(parser: &Parser, block_ok: bool) {
                     WNOHANG | WUNTRACED | WCONTINUED,
                 )
             };
-            let Some(pid) = Pid::new(pid) else {
+            if pid == 0 {
                 continue;
-            };
+            }
+            let pid = Pid::new(pid);
             assert!(pid == proc.pid().unwrap(), "Unexpected waitpid() return");
 
             // The process has stopped or exited! Update its status.
@@ -1251,7 +1251,7 @@ fn process_mark_finished_children(parser: &Parser, block_ok: bool) {
             if status.normal_exited() || status.signal_exited() {
                 FLOGF!(
                     proc_reap_external,
-                    "Reaped external process '%ls' (pid %d, status %d)",
+                    "Reaped external process '%s' (pid %d, status %d)",
                     proc.argv0().unwrap(),
                     pid,
                     proc.status().status_value()
@@ -1260,7 +1260,7 @@ fn process_mark_finished_children(parser: &Parser, block_ok: bool) {
                 assert!(status.stopped() || status.continued());
                 FLOGF!(
                     proc_reap_external,
-                    "External process '%ls' (pid %d, %s)",
+                    "External process '%s' (pid %d, %s)",
                     proc.argv0().unwrap(),
                     proc.pid().unwrap(),
                     if proc.status().stopped() {
@@ -1305,7 +1305,7 @@ fn process_mark_finished_children(parser: &Parser, block_ok: bool) {
             handle_child_status(j, proc, status);
             FLOGF!(
                 proc_reap_internal,
-                "Reaped internal process '%ls' (id %llu, status %d)",
+                "Reaped internal process '%s' (id %u, status %d)",
                 proc.argv0().unwrap(),
                 internal_proc.get_id(),
                 proc.status().status_value(),
