@@ -5,8 +5,8 @@ use crate::common::exit_without_destructors;
 use crate::event::{enqueue_signal, is_signal_observed};
 use crate::nix::getpid;
 use crate::reader::{reader_handle_sigint, reader_sighup, safe_restore_term_mode};
-use crate::termsize::TermsizeContainer;
-use crate::topic_monitor::{topic_monitor_principal, Generation, GenerationsList, Topic};
+use crate::termsize::safe_termsize_invalidate_tty;
+use crate::topic_monitor::{Generation, GenerationsList, Topic, topic_monitor_principal};
 use crate::tty_handoff::{safe_deactivate_tty_protocols, safe_mark_tty_invalid};
 use crate::wchar::prelude::*;
 use crate::wutil::{fish_wcstoi, perror};
@@ -78,7 +78,7 @@ extern "C" fn fish_signal_handler(
     match sig {
         libc::SIGWINCH => {
             // Respond to a winch signal by telling the termsize container.
-            TermsizeContainer::handle_winch();
+            safe_termsize_invalidate_tty();
         }
         libc::SIGHUP => {
             // Exit unless the signal was trapped.
@@ -310,13 +310,13 @@ pub fn signal_unblock_all() {
 /// A Sigchecker can be used to check if a SIGINT (or SIGHUP) has been delivered.
 pub struct SigChecker {
     topic: Topic,
-    gen: Generation,
+    r#gen: Generation,
 }
 
 impl SigChecker {
     /// Create a new checker for the given topic.
     pub fn new(topic: Topic) -> Self {
-        let mut res = SigChecker { topic, gen: 0 };
+        let mut res = SigChecker { topic, r#gen: 0 };
         // Call check() to update our generation.
         res.check();
         res
@@ -331,9 +331,9 @@ impl SigChecker {
     /// was created.
     pub fn check(&mut self) -> bool {
         let tm = topic_monitor_principal();
-        let gen = tm.generation_for_topic(self.topic);
-        let changed = self.gen != gen;
-        self.gen = gen;
+        let r#gen = tm.generation_for_topic(self.topic);
+        let changed = self.r#gen != r#gen;
+        self.r#gen = r#gen;
         changed
     }
 
@@ -341,7 +341,7 @@ impl SigChecker {
     pub fn wait(&self) {
         let tm = topic_monitor_principal();
         let gens = GenerationsList::invalid();
-        gens.set(self.topic, self.gen);
+        gens.set(self.topic, self.r#gen);
         tm.check(&gens, true /* wait */);
     }
 }
@@ -417,7 +417,17 @@ const SIGNAL_TABLE : &[LookupEntry] = &[
     #[cfg(any(apple, bsd))]
     signal_entry!(SIGINFO, SIGINFO_DESC),
 
-    #[cfg(target_os = "linux")]
+    #[cfg(all(
+        target_os = "linux",
+        not(any(
+            target_arch = "mips",
+            target_arch = "mips32r6",
+            target_arch = "mips64",
+            target_arch = "mips64r6",
+            target_arch = "sparc",
+            target_arch = "sparc64",
+        ))
+    ))]
     signal_entry!(SIGSTKFLT, SIGSTKFLT_DESC),
 
     #[cfg(target_os = "linux")]
@@ -557,42 +567,46 @@ impl From<Signal> for NonZeroI32 {
     }
 }
 
-// Need to use add_test for wgettext support.
+#[cfg(test)]
+mod tests {
+    use super::Signal;
+    use crate::wchar::prelude::*;
 
-#[test]
-fn test_signal_name() {
-    let sig = Signal::new(libc::SIGINT);
-    assert_eq!(sig.name(), "SIGINT");
-}
+    #[test]
+    fn test_signal_name() {
+        let sig = Signal::new(libc::SIGINT);
+        assert_eq!(sig.name(), "SIGINT");
+    }
 
-#[rustfmt::skip]
-#[test]
-fn test_signal_parse() {
-    assert_eq!(Signal::parse(L!("SIGHUP")), Some(Signal::new(libc::SIGHUP)));
-    assert_eq!(Signal::parse(L!("sigwinch")), Some(Signal::new(libc::SIGWINCH)));
-    assert_eq!(Signal::parse(L!("TSTP")), Some(Signal::new(libc::SIGTSTP)));
-    assert_eq!(Signal::parse(L!("TstP")), Some(Signal::new(libc::SIGTSTP)));
-    assert_eq!(Signal::parse(L!("sigCONT")), Some(Signal::new(libc::SIGCONT)));
-    assert_eq!(Signal::parse(L!("SIGFOO")), None);
-    assert_eq!(Signal::parse(L!("")), None);
-    assert_eq!(Signal::parse(L!("SIG")), None);
-    assert_eq!(Signal::parse(L!("سلام")), None);
+    #[rustfmt::skip]
+    #[test]
+    fn test_signal_parse() {
+        assert_eq!(Signal::parse(L!("SIGHUP")), Some(Signal::new(libc::SIGHUP)));
+        assert_eq!(Signal::parse(L!("sigwinch")), Some(Signal::new(libc::SIGWINCH)));
+        assert_eq!(Signal::parse(L!("TSTP")), Some(Signal::new(libc::SIGTSTP)));
+        assert_eq!(Signal::parse(L!("TstP")), Some(Signal::new(libc::SIGTSTP)));
+        assert_eq!(Signal::parse(L!("sigCONT")), Some(Signal::new(libc::SIGCONT)));
+        assert_eq!(Signal::parse(L!("SIGFOO")), None);
+        assert_eq!(Signal::parse(L!("")), None);
+        assert_eq!(Signal::parse(L!("SIG")), None);
+        assert_eq!(Signal::parse(L!("سلام")), None);
 
-    assert_eq!(Signal::parse(&libc::SIGINT.to_wstring()), Some(Signal::new(libc::SIGINT)));
-    assert_eq!(Signal::parse(L!("0")), None);
-    assert_eq!(Signal::parse(L!("-0")), None);
-    assert_eq!(Signal::parse(L!("-1")), None);
-}
+        assert_eq!(Signal::parse(&libc::SIGINT.to_wstring()), Some(Signal::new(libc::SIGINT)));
+        assert_eq!(Signal::parse(L!("0")), None);
+        assert_eq!(Signal::parse(L!("-0")), None);
+        assert_eq!(Signal::parse(L!("-1")), None);
+    }
 
-#[test]
-#[cfg(any(target_os = "freebsd", target_os = "netbsd", target_os = "openbsd"))]
-/// Verify bsd feature is detected on the known BSDs, which gives us greater confidence it'll work
-/// for the unknown ones too. We don't need to do this for Linux and macOS because we're using
-/// rust's native OS targeting for those.
-fn bsd_signals() {
-    assert_eq!(Signal::parse(L!("SIGEMT")), Some(Signal::new(libc::SIGEMT)));
-    assert_eq!(
-        Signal::parse(L!("SIGINFO")),
-        Some(Signal::new(libc::SIGINFO))
-    );
+    #[test]
+    #[cfg(any(target_os = "freebsd", target_os = "netbsd", target_os = "openbsd"))]
+    /// Verify bsd feature is detected on the known BSDs, which gives us greater confidence it'll work
+    /// for the unknown ones too. We don't need to do this for Linux and macOS because we're using
+    /// rust's native OS targeting for those.
+    fn bsd_signals() {
+        assert_eq!(Signal::parse(L!("SIGEMT")), Some(Signal::new(libc::SIGEMT)));
+        assert_eq!(
+            Signal::parse(L!("SIGINFO")),
+            Some(Signal::new(libc::SIGINFO))
+        );
+    }
 }
