@@ -1,13 +1,13 @@
 // Generic output functions.
+use crate::FLOGF;
 use crate::color::{Color, Color24};
 use crate::common::ToCString;
-use crate::common::{self, escape_string, wcs2string, wcs2string_appending, EscapeStringStyle};
+use crate::common::{self, EscapeStringStyle, escape_string, wcs2bytes, wcs2bytes_appending};
 use crate::future_feature_flags::{self, FeatureFlag};
 use crate::screen::{is_dumb, only_grayscale};
 use crate::text_face::{TextFace, TextStyling, UnderlineStyle};
 use crate::threads::MainThread;
 use crate::wchar::prelude::*;
-use crate::FLOGF;
 use bitflags::bitflags;
 use std::cell::{RefCell, RefMut};
 use std::env;
@@ -16,9 +16,9 @@ use std::ops::{Deref, DerefMut};
 use std::os::fd::RawFd;
 use std::os::unix::ffi::OsStrExt;
 use std::path::PathBuf;
-use std::sync::atomic::{AtomicU8, Ordering};
 use std::sync::Arc;
 use std::sync::Mutex;
+use std::sync::atomic::{AtomicU8, Ordering};
 
 bitflags! {
     #[derive(Copy, Clone, Default)]
@@ -102,6 +102,7 @@ pub(crate) enum TerminalCommand<'a> {
     // Note that OSC 7 and OSC 52 are written from fish script, and OSC 8 is written in our
     // man pages (via "man_show_urls").
     Osc0WindowTitle(&'a [WString]),
+    Osc1TabTitle(&'a [WString]),
     Osc133CommandStart(&'a wstr),
     Osc133PromptStart,
     Osc133CommandFinished(libc::c_int),
@@ -111,7 +112,6 @@ pub(crate) enum TerminalCommand<'a> {
     ScrollContentUp(usize),
 
     DecsetShowCursor,
-    DecrstMouseTracking,
     DecsetFocusReporting,
     DecrstFocusReporting,
     DecsetBracketedPaste,
@@ -176,14 +176,14 @@ pub(crate) trait Output {
             ModifyOtherKeysDisable => write(self, b"\x1b[>4;0m"),
             ApplicationKeypadModeEnable => write(self, b"\x1b="),
             ApplicationKeypadModeDisable => write(self, b"\x1b>"),
-            Osc0WindowTitle(title) => osc_0_window_title(self, title),
+            Osc0WindowTitle(title) => osc_0_or_1_terminal_title(self, false, title),
+            Osc1TabTitle(title) => osc_0_or_1_terminal_title(self, true, title),
             Osc133PromptStart => osc_133_prompt_start(self),
             Osc133CommandStart(command) => osc_133_command_start(self, command),
             Osc133CommandFinished(s) => osc_133_command_finished(self, s),
             QueryCursorPosition => write(self, b"\x1b[6n"),
             ScrollContentUp(lines) => scroll_content_up(self, lines),
             DecsetShowCursor => write(self, b"\x1b[?25h"),
-            DecrstMouseTracking => write(self, b"\x1b[?1000l"),
             DecsetFocusReporting => write(self, b"\x1b[?1004h"),
             DecrstFocusReporting => write(self, b"\x1b[?1004l"),
             DecsetBracketedPaste => write(self, b"\x1b[?2004h"),
@@ -371,10 +371,10 @@ fn query_kitty_progressive_enhancements(out: &mut impl Output) -> bool {
     true
 }
 
-fn osc_0_window_title(out: &mut impl Output, title: &[WString]) -> bool {
-    out.write_bytes(b"\x1b]0;");
+fn osc_0_or_1_terminal_title(out: &mut impl Output, is_1: bool, title: &[WString]) -> bool {
+    out.write_bytes(if is_1 { b"\x1b]1;" } else { b"\x1b]0;" });
     for title_line in title {
-        out.write_bytes(&wcs2string(title_line));
+        out.write_bytes(&wcs2bytes(title_line));
     }
     out.write_bytes(b"\x07"); // BEL
     true
@@ -505,12 +505,12 @@ impl Outputter {
     /// In order to set the color to normal, three commands may have to be written.
     ///
     /// - First a command to set the color, such as set_a_foreground. This is needed because otherwise
-    /// the previous strings colors might be removed as well.
+    ///   the previous strings colors might be removed as well.
     ///
     /// - After that we write the exit_attribute_mode command to reset all color attributes.
     ///
     /// - Lastly we may need to write set_a_background or set_a_foreground to set the other half of the
-    /// color pair to what it should be.
+    ///   color pair to what it should be.
     pub(crate) fn set_text_face(&mut self, face: TextFace) {
         self.set_text_face_internal(face, true)
     }
@@ -541,14 +541,12 @@ impl Outputter {
             // Only way to exit non-resettable ones is a reset of all attributes.
             self.reset_text_face();
         }
-        if salvage_unreadable {
-            if !bg.is_special() && fg == bg {
-                fg = if bg == Color::WHITE {
-                    Color::BLACK
-                } else {
-                    Color::WHITE
-                };
-            }
+        if salvage_unreadable && !bg.is_special() && fg == bg {
+            fg = if bg == Color::WHITE {
+                Color::BLACK
+            } else {
+                Color::WHITE
+            };
         }
 
         if !fg.is_none() && fg != self.last.fg {
@@ -636,7 +634,7 @@ impl Outputter {
 
     /// Write a wide string.
     pub fn write_wstr(&mut self, str: &wstr) {
-        wcs2string_appending(&mut self.contents, str);
+        wcs2bytes_appending(&mut self.contents, str);
         self.maybe_flush();
     }
 
@@ -863,7 +861,7 @@ pub fn setup() {
             let mut path = PathBuf::from(dir);
             path.push(first_char.clone());
             path.push(t.clone());
-            FLOGF!(term_support, "Trying path '%ls'", path.to_str().unwrap());
+            FLOGF!(term_support, "Trying path '%s'", path.to_str().unwrap());
             if let Ok(db) = terminfo::Database::from_path(path) {
                 return Ok(db);
             }
