@@ -32,7 +32,7 @@ use std::{
 
 use crate::{
     wchar::prelude::*,
-    wutil::{wcstod::wcstod_underscores, wgettext, Error as wcstodError},
+    wutil::{Error as wcstodError, wcstod::wcstod_underscores, wgettext},
 };
 
 #[derive(Clone, Copy)]
@@ -196,11 +196,7 @@ fn maximum(n: &[f64]) -> f64 {
 
         if a == b {
             // treat +0 as larger than -0
-            if a.is_sign_positive() {
-                a
-            } else {
-                b
-            }
+            if a.is_sign_positive() { a } else { b }
         } else if a > b {
             a
         } else {
@@ -220,11 +216,7 @@ fn minimum(n: &[f64]) -> f64 {
 
         if a == b {
             // treat -0 as smaller than +0
-            if a.is_sign_negative() {
-                a
-            } else {
-                b
-            }
+            if a.is_sign_negative() { a } else { b }
         } else if a < b {
             a
         } else {
@@ -286,12 +278,14 @@ const BUILTINS: &[(&wstr, Function)] = &[
         L!("bitxor"),
         Function::Fn2(|a, b| bitwise_op(a, b, BitXor::bitxor)),
     ),
+    #[allow(clippy::incompatible_msrv)]
     (L!("ceil"), Function::Fn1(f64::ceil)),
     (L!("cos"), Function::Fn1(f64::cos)),
     (L!("cosh"), Function::Fn1(f64::cosh)),
     (L!("e"), Function::Constant(E)),
     (L!("exp"), Function::Fn1(f64::exp)),
     (L!("fac"), Function::Fn1(fac)),
+    #[allow(clippy::incompatible_msrv)]
     (L!("floor"), Function::Fn1(f64::floor)),
     (L!("ln"), Function::Fn1(f64::ln)),
     (L!("log"), Function::Fn1(f64::log10)),
@@ -303,6 +297,7 @@ const BUILTINS: &[(&wstr, Function)] = &[
     (L!("npr"), Function::Fn2(npr)),
     (L!("pi"), Function::Constant(PI)),
     (L!("pow"), Function::Fn2(f64::powf)),
+    #[allow(clippy::incompatible_msrv)]
     (L!("round"), Function::Fn1(f64::round)),
     (L!("sin"), Function::Fn1(f64::sin)),
     (L!("sinh"), Function::Fn1(f64::sinh)),
@@ -630,7 +625,7 @@ impl<'s> State<'s> {
         }
     }
 
-    /// <power>  = {("-" | "+")} <base>
+    /// \<power\>  = {("-" | "+")} \<base\>
     fn power(&mut self) -> f64 {
         let mut sign = 1.0;
         while let Token::Infix(op) = self.current {
@@ -647,7 +642,7 @@ impl<'s> State<'s> {
         sign * self.base()
     }
 
-    /// <factor> = <power> {"^" <power>}
+    /// \<factor\> = \<power\> {"^" \<power\>}
     fn factor(&mut self) -> f64 {
         let mut ret = self.power();
 
@@ -659,7 +654,7 @@ impl<'s> State<'s> {
         ret
     }
 
-    /// <term>   = <factor> {("*" | "/" | "%") <factor>}
+    /// \<term\>   = \<factor\> {("*" | "/" | "%") \<factor\>}
     fn term(&mut self) -> f64 {
         let mut ret = self.factor();
         while let Token::Infix(op @ (Operator::Mul | Operator::Div | Operator::Rem)) = self.current
@@ -678,7 +673,7 @@ impl<'s> State<'s> {
         ret
     }
 
-    /// <expr>   = <term> {("+" | "-") <term>}
+    /// \<expr\>   = \<term\> {("+" | "-") \<term\>}
     fn expr(&mut self) -> f64 {
         let mut ret = self.term();
         while let Token::Infix(op @ (Operator::Add | Operator::Sub)) = self.current {
