@@ -1,9 +1,9 @@
+use crate::FLOGF;
 use crate::common::wcs2osstring;
 use crate::env_universal_common::default_vars_path;
 use crate::universal_notifier::UniversalNotifier;
 use crate::wchar::prelude::*;
 use crate::wutil::wdirname;
-use crate::FLOGF;
 use nix::sys::event::{EvFlags, EventFilter, FilterFlag, KEvent, Kqueue};
 use std::fs::File;
 use std::os::fd::AsFd;
@@ -178,33 +178,30 @@ impl UniversalNotifier for KqueueNotifier {
     }
 }
 
-#[test]
-fn test_kqueue_notifiers() {
-    use crate::common::cstr2wcstring;
-    use std::ffi::CStr;
-    use std::fs::remove_dir_all;
-    use std::path::PathBuf;
+#[cfg(test)]
+mod tests {
+    use super::KqueueNotifier;
+    use crate::common::wcs2osstring;
+    use crate::{
+        universal_notifier::{UniversalNotifier, test_helpers::test_notifiers},
+        wchar::WString,
+    };
 
-    let mut template: Box<[u8]> = Box::from(&b"/tmp/fish_kqueue_XXXXXX\0"[..]);
+    #[test]
+    fn test_kqueue_notifiers() {
+        let temp_dir = fish_tempfile::new_dir().unwrap();
+        let fake_uvars_path =
+            WString::from(temp_dir.path().join("fish_variables").to_str().unwrap());
 
-    let temp_dir_ptr = unsafe { libc::mkdtemp(template.as_mut_ptr().cast()) };
-    if temp_dir_ptr.is_null() {
-        panic!("failed to create temp dir");
+        let mut notifiers = Vec::new();
+        for _ in 0..16 {
+            notifiers
+                .push(KqueueNotifier::new_at(&fake_uvars_path).expect("failed to create notifier"));
+        }
+        let notifiers = notifiers
+            .iter()
+            .map(|n| n as &dyn UniversalNotifier)
+            .collect::<Vec<_>>();
+        test_notifiers(&notifiers, Some(&fake_uvars_path));
     }
-    let tmp_dir = unsafe { CStr::from_ptr(temp_dir_ptr) };
-    let fake_uvars_dir = cstr2wcstring(tmp_dir.to_bytes_with_nul());
-    let fake_uvars_path = fake_uvars_dir.clone() + "/fish_variables";
-
-    let mut notifiers = Vec::new();
-    for _ in 0..16 {
-        notifiers
-            .push(KqueueNotifier::new_at(&fake_uvars_path).expect("failed to create notifier"));
-    }
-    let notifiers = notifiers
-        .iter()
-        .map(|n| n as &dyn UniversalNotifier)
-        .collect::<Vec<_>>();
-    super::test_helpers::test_notifiers(&notifiers, Some(&fake_uvars_path));
-
-    let _ = remove_dir_all(PathBuf::from(wcs2osstring(&fake_uvars_dir)));
 }
