@@ -7,17 +7,17 @@ use libc::{c_char, c_int};
 use std::ffi::CString;
 use std::os::fd::{BorrowedFd, RawFd};
 
-extern "C" {
-    fn notify_register_file_descriptor(
+unsafe extern "C" {
+    unsafe fn notify_register_file_descriptor(
         name: *const c_char,
         fd: *mut c_int,
         flags: c_int,
         token: *mut c_int,
     ) -> u32;
 
-    fn notify_post(name: *const c_char) -> u32;
+    unsafe fn notify_post(name: *const c_char) -> u32;
 
-    fn notify_cancel(token: c_int) -> c_int;
+    unsafe fn notify_cancel(token: c_int) -> c_int;
 }
 
 const NOTIFY_STATUS_OK: u32 = 0;
@@ -136,15 +136,21 @@ impl UniversalNotifier for NotifydNotifier {
     }
 }
 
-#[test]
-fn test_notifyd_notifiers() {
-    let mut notifiers = Vec::new();
-    for _ in 0..16 {
-        notifiers.push(NotifydNotifier::new().expect("failed to create notifier"));
+#[cfg(test)]
+mod tests {
+    use super::NotifydNotifier;
+    use crate::universal_notifier::{UniversalNotifier, test_helpers::test_notifiers};
+
+    #[test]
+    fn test_notifyd_notifiers() {
+        let mut notifiers = Vec::new();
+        for _ in 0..16 {
+            notifiers.push(NotifydNotifier::new().expect("failed to create notifier"));
+        }
+        let notifiers = notifiers
+            .iter()
+            .map(|n| n as &dyn UniversalNotifier)
+            .collect::<Vec<_>>();
+        test_notifiers(&notifiers, None);
     }
-    let notifiers = notifiers
-        .iter()
-        .map(|n| n as &dyn UniversalNotifier)
-        .collect::<Vec<_>>();
-    super::test_helpers::test_notifiers(&notifiers, None);
 }
