@@ -6,28 +6,29 @@ use std::collections::HashSet;
 use std::fs;
 
 use crate::common::{
-    char_offset, is_windows_subsystem_for_linux, unescape_string, UnescapeFlags,
-    UnescapeStringStyle, WILDCARD_RESERVED_BASE, WSL,
+    UnescapeFlags, UnescapeStringStyle, WILDCARD_RESERVED_BASE, WSL, char_offset,
+    is_windows_subsystem_for_linux, unescape_string,
 };
 use crate::complete::{CompleteFlags, Completion, CompletionReceiver, PROG_COMPLETE_SEP};
 use crate::expand::ExpandFlags;
 use crate::fallback::wcscasecmp;
-use crate::future_feature_flags::feature_test;
 use crate::future_feature_flags::FeatureFlag;
+use crate::future_feature_flags::feature_test;
 use crate::wchar::prelude::*;
 use crate::wcstringutil::{
-    string_fuzzy_match_string, string_suffixes_string_case_insensitive, CaseSensitivity,
+    CaseSensitivity, string_fuzzy_match_string, string_suffixes_string_case_insensitive,
 };
 use crate::wutil::dir_iter::DirEntryType;
 use crate::wutil::{dir_iter::DirEntry, lwstat, waccess};
-use once_cell::sync::Lazy;
 
-static COMPLETE_EXEC_DESC: Lazy<&wstr> = Lazy::new(|| wgettext!("command"));
-static COMPLETE_EXEC_LINK_DESC: Lazy<&wstr> = Lazy::new(|| wgettext!("command link"));
-static COMPLETE_FILE_DESC: Lazy<&wstr> = Lazy::new(|| wgettext!("file"));
-static COMPLETE_SYMLINK_DESC: Lazy<&wstr> = Lazy::new(|| wgettext!("symlink"));
-static COMPLETE_DIRECTORY_SYMLINK_DESC: Lazy<&wstr> = Lazy::new(|| wgettext!("dir symlink"));
-static COMPLETE_DIRECTORY_DESC: Lazy<&wstr> = Lazy::new(|| wgettext!("directory"));
+localizable_consts!(
+    COMPLETE_EXEC_DESC "command"
+    COMPLETE_EXEC_LINK_DESC "command link"
+    COMPLETE_FILE_DESC "file"
+    COMPLETE_SYMLINK_DESC "symlink"
+    COMPLETE_DIRECTORY_SYMLINK_DESC "dir symlink"
+    COMPLETE_DIRECTORY_DESC "directory"
+);
 
 /// Character representing any character except '/' (slash).
 pub const ANY_CHAR: char = char_offset(WILDCARD_RESERVED_BASE, 0);
@@ -313,18 +314,18 @@ fn file_get_desc(
 
     return if is_link {
         if is_dir {
-            *COMPLETE_DIRECTORY_SYMLINK_DESC
+            wgettext!(COMPLETE_DIRECTORY_SYMLINK_DESC)
         } else if is_executable(filename) {
-            *COMPLETE_EXEC_LINK_DESC
+            wgettext!(COMPLETE_EXEC_LINK_DESC)
         } else {
-            *COMPLETE_SYMLINK_DESC
+            wgettext!(COMPLETE_SYMLINK_DESC)
         }
     } else if is_dir {
-        *COMPLETE_DIRECTORY_DESC
+        wgettext!(COMPLETE_DIRECTORY_DESC)
     } else if is_executable(filename) {
-        *COMPLETE_EXEC_DESC
+        wgettext!(COMPLETE_EXEC_DESC)
     } else {
-        *COMPLETE_FILE_DESC
+        wgettext!(COMPLETE_FILE_DESC)
     };
 }
 
@@ -437,7 +438,7 @@ mod expander {
 
     use crate::{
         path::append_path_component,
-        wutil::{dir_iter::DirIter, normalize_path, DevInode},
+        wutil::{DevInode, dir_iter::DirIter, normalize_path},
     };
 
     use super::*;
