@@ -1,27 +1,23 @@
 pub mod dir_iter;
-pub mod encoding;
 pub mod errors;
 pub mod fileid;
 pub mod gettext;
 mod hex_float;
 #[macro_use]
 pub mod printf;
-#[cfg(test)]
-mod tests;
 pub mod wcstod;
 pub mod wcstoi;
 
 use crate::common::{
-    cstr2wcstring, fish_reserved_codepoint, str2wcstring, wcs2osstring, wcs2string, wcs2zstring,
+    bytes2wcstring, fish_reserved_codepoint, wcs2bytes, wcs2osstring, wcs2zstring,
 };
-use crate::fallback;
-use crate::flog::FLOGF;
-use crate::wchar::{wstr, WString, L};
+use crate::wchar::{L, WString, wstr};
 use crate::wchar_ext::WExt;
-use crate::wcstringutil::{join_strings, wcs2string_callback};
+use crate::wcstringutil::{join_strings, wcs2bytes_callback};
+use crate::{FLOG, fallback};
 use errno::errno;
 pub use gettext::{
-    localizable_consts, localizable_string, wgettext, wgettext_fmt, LocalizableString,
+    LocalizableString, localizable_consts, localizable_string, wgettext, wgettext_fmt,
 };
 use std::ffi::{CStr, OsStr};
 use std::fs::{self, canonicalize};
@@ -31,7 +27,7 @@ use std::os::unix::prelude::*;
 pub use crate::wutil::printf::{eprintf, fprintf, printf, sprintf};
 
 pub use fileid::{
-    file_id_for_file, file_id_for_path, file_id_for_path_narrow, DevInode, FileId, INVALID_FILE_ID,
+    DevInode, FileId, INVALID_FILE_ID, file_id_for_file, file_id_for_path, file_id_for_path_narrow,
 };
 pub use wcstoi::*;
 
@@ -77,7 +73,7 @@ pub fn wunlink(file_name: &wstr) -> libc::c_int {
 }
 
 pub fn wperror(s: &wstr) {
-    let bytes = wcs2string(s);
+    let bytes = wcs2bytes(s);
     // We can't guarantee the string is 100% Unicode (why?), so we don't use std::str::from_utf8()
     let s = OsStr::from_bytes(&bytes).to_string_lossy();
     perror(&s)
@@ -104,24 +100,13 @@ pub fn perror_io(s: &str, e: &io::Error) {
 
 /// Wide character version of getcwd().
 pub fn wgetcwd() -> WString {
-    let mut cwd = [b'\0'; libc::PATH_MAX as usize];
-    let res = unsafe {
-        libc::getcwd(
-            std::ptr::addr_of_mut!(cwd).cast(),
-            std::mem::size_of_val(&cwd),
-        )
-    };
-    if !res.is_null() {
-        return cstr2wcstring(&cwd);
+    match std::env::current_dir() {
+        Ok(cwd) => bytes2wcstring(cwd.into_os_string().as_bytes()),
+        Err(e) => {
+            FLOG!(error, "std::env::current_dir() failed with error:", e);
+            WString::new()
+        }
     }
-
-    FLOGF!(
-        error,
-        "getcwd() failed with errno %d/%s",
-        errno::errno().0,
-        errno::errno().to_string()
-    );
-    WString::new()
 }
 
 /// Wide character version of readlink().
@@ -148,7 +133,7 @@ pub fn wreadlink(file_name: &wstr) -> Option<WString> {
     if nbytes == bufsize {
         return None;
     }
-    Some(str2wcstring(&target_buf[0..nbytes]))
+    Some(bytes2wcstring(&target_buf[0..nbytes]))
 }
 
 /// Wide character realpath. The last path component does not need to be valid. If an error occurs,
@@ -206,7 +191,7 @@ pub fn wrealpath(pathname: &wstr) -> Option<WString> {
         }
     };
 
-    Some(str2wcstring(&real_path))
+    Some(bytes2wcstring(&real_path))
 }
 
 /// Given an input path, "normalize" it:
@@ -257,36 +242,6 @@ pub fn normalize_path(path: &wstr, allow_leading_double_slashes: bool) -> WStrin
     result
 }
 
-#[test]
-fn test_normalize_path() {
-    fn norm_path(path: &wstr) -> WString {
-        normalize_path(path, true)
-    }
-    assert_eq!(norm_path(L!("")), ".");
-    assert_eq!(norm_path(L!("..")), "..");
-    assert_eq!(norm_path(L!("./")), ".");
-    assert_eq!(norm_path(L!("./.")), ".");
-    assert_eq!(norm_path(L!("/")), "/");
-    assert_eq!(norm_path(L!("//")), "//");
-    assert_eq!(norm_path(L!("///")), "/");
-    assert_eq!(norm_path(L!("////")), "/");
-    assert_eq!(norm_path(L!("/.///")), "/");
-    assert_eq!(norm_path(L!(".//")), ".");
-    assert_eq!(norm_path(L!("/.//../")), "/");
-    assert_eq!(norm_path(L!("////abc")), "/abc");
-    assert_eq!(norm_path(L!("/abc")), "/abc");
-    assert_eq!(norm_path(L!("/abc/")), "/abc");
-    assert_eq!(norm_path(L!("/abc/..def/")), "/abc/..def");
-    assert_eq!(norm_path(L!("//abc/../def/")), "//def");
-    assert_eq!(norm_path(L!("abc/../abc/../abc/../abc")), "abc");
-    assert_eq!(norm_path(L!("../../")), "../..");
-    assert_eq!(norm_path(L!("foo/./bar")), "foo/bar");
-    assert_eq!(norm_path(L!("foo/../")), ".");
-    assert_eq!(norm_path(L!("foo/../foo")), "foo");
-    assert_eq!(norm_path(L!("foo/../foo/")), "foo");
-    assert_eq!(norm_path(L!("foo/././bar/.././baz")), "foo/baz");
-}
-
 /// Given an input path `path` and a working directory `wd`, do a "normalizing join" in a way
 /// appropriate for cd. That is, return effectively wd + path while resolving leading ../s from
 /// path. The intent here is to allow 'cd' out of a directory which may no longer exist, without
@@ -318,6 +273,7 @@ pub fn path_normalize_for_cd(wd: &wstr, path: &wstr) -> WString {
 
     // Erase leading . and .. components from path_comps, popping from wd_comps as we go.
     while let Some(comp) = path_comps.peek() {
+        #[allow(clippy::if_same_then_else)]
         if comp.is_empty() || comp == "." {
             path_comps.next();
         } else if comp == ".." && wd_comps.pop_back().is_some() {
@@ -342,98 +298,6 @@ pub fn path_normalize_for_cd(wd: &wstr, path: &wstr) -> WString {
     result
 }
 
-#[cfg(test)]
-mod path_cd_tests {
-    use super::path_normalize_for_cd;
-    use crate::wchar::L;
-
-    #[test]
-    fn relative_path() {
-        let wd = L!("/home/user/");
-        let path = L!("projects");
-        eprintf!("(%ls, %ls)\n", wd, path);
-        assert_eq!(path_normalize_for_cd(wd, path), L!("/home/user/projects"));
-    }
-
-    #[test]
-    fn absolute_path() {
-        let wd = L!("/home/user/");
-        let path = L!("/etc");
-        eprintf!("(%ls, %ls)\n", wd, path);
-        assert_eq!(path_normalize_for_cd(wd, path), L!("/etc"));
-    }
-
-    #[test]
-    fn parent_directory() {
-        let wd = L!("/home/user/projects/");
-        let path = L!("../docs");
-        eprintf!("(%ls, %ls)\n", wd, path);
-        assert_eq!(path_normalize_for_cd(wd, path), L!("/home/user/docs"));
-    }
-
-    #[test]
-    fn current_directory() {
-        let wd = L!("/home/user/");
-        let path = L!("./");
-        eprintf!("(%ls, %ls)\n", wd, path);
-        assert_eq!(path_normalize_for_cd(wd, path), L!("/home/user"));
-    }
-
-    #[test]
-    fn nested_parent_directory() {
-        let wd = L!("/home/user/projects/");
-        let path = L!("../../");
-        eprintf!("(%ls, %ls)\n", wd, path);
-        assert_eq!(path_normalize_for_cd(wd, path), L!("/home"));
-    }
-
-    #[test]
-    fn complex_path() {
-        let wd = L!("/home/user/projects/");
-        let path = L!("./../other/projects/./.././../docs");
-        eprintf!("(%ls, %ls)\n", wd, path);
-        assert_eq!(
-            path_normalize_for_cd(wd, path),
-            L!("/home/user/other/projects/./.././../docs")
-        );
-    }
-
-    #[test]
-    fn root_directory() {
-        let wd = L!("/");
-        let path = L!("..");
-        eprintf!("(%ls, %ls)\n", wd, path);
-        assert_eq!(path_normalize_for_cd(wd, path), L!("/.."));
-    }
-
-    #[test]
-    fn up_to_root_directory() {
-        let wd = L!("/foo/");
-        let path = L!("..");
-        eprintf!("(%ls, %ls)\n", wd, path);
-        assert_eq!(path_normalize_for_cd(wd, path), L!("/"));
-    }
-
-    #[test]
-    fn empty_path() {
-        let wd = L!("/home/user/");
-        let path = L!("");
-        eprintf!("(%ls, %ls)\n", wd, path);
-        assert_eq!(path_normalize_for_cd(wd, path), L!("/home/user/"));
-    }
-
-    #[test]
-    fn trailing_slash() {
-        let wd = L!("/home/user/projects/");
-        let path = L!("docs/");
-        eprintf!("(%ls, %ls)\n", wd, path);
-        assert_eq!(
-            path_normalize_for_cd(wd, path),
-            L!("/home/user/projects/docs/")
-        );
-    }
-}
-
 /// Wide character version of dirname().
 pub fn wdirname(mut path: &wstr) -> &wstr {
     // Do not use system-provided dirname (#7837).
@@ -519,9 +383,9 @@ pub fn write_to_fd(input: &[u8], fd: RawFd) -> nix::Result<usize> {
 /// actually written cannot be obtained.
 pub fn wwrite_to_fd(input: &wstr, fd: RawFd) -> Option<usize> {
     // Accumulate data in a local buffer.
-    let mut accum = [b'\0'; 512];
+    let mut accum = [0u8; 512];
     let mut accumlen = 0;
-    let maxaccum: usize = std::mem::size_of_val(&accum);
+    let accum_capacity = accum.len();
 
     // Helper to perform a write to 'fd', looping as necessary.
     // Return true on success, false on error.
@@ -548,18 +412,16 @@ pub fn wwrite_to_fd(input: &wstr, fd: RawFd) -> Option<usize> {
         true
     };
 
-    let mut success = wcs2string_callback(input, |buff: &[u8]| {
-        if buff.len() + accumlen > maxaccum {
+    let mut success = wcs2bytes_callback(input, |buff: &[u8]| {
+        if buff.len() + accumlen > accum_capacity {
             // We have to flush.
             if !flush_accum(&mut total_written, &accum, &mut accumlen) {
                 return false;
             }
         }
-        if buff.len() + accumlen <= maxaccum {
+        if buff.len() + accumlen <= accum_capacity {
             // Accumulate more.
-            unsafe {
-                std::ptr::copy(&buff[0], &mut accum[accumlen], buff.len());
-            }
+            accum[accumlen..(accumlen + buff.len())].copy_from_slice(buff);
             accumlen += buff.len();
             true
         } else {
@@ -571,11 +433,7 @@ pub fn wwrite_to_fd(input: &wstr, fd: RawFd) -> Option<usize> {
     if success {
         success = flush_accum(&mut total_written, &accum, &mut accumlen);
     }
-    if success {
-        Some(total_written)
-    } else {
-        None
-    }
+    if success { Some(total_written) } else { None }
 }
 
 const PUA1_START: char = '\u{E000}';
@@ -619,12 +477,236 @@ pub fn wstr_offset_in(cursor: &wstr, base: &wstr) -> usize {
     offset as usize
 }
 
-#[test]
-fn test_wstr_offset_in() {
-    use crate::wchar::L;
-    let base = L!("hello world");
-    assert_eq!(wstr_offset_in(&base[6..], base), 6);
-    assert_eq!(wstr_offset_in(&base[0..], base), 0);
-    assert_eq!(wstr_offset_in(&base[6..], &base[6..]), 0);
-    assert_eq!(wstr_offset_in(&base[base.len()..], base), base.len());
+#[cfg(test)]
+mod tests {
+    use super::{normalize_path, wbasename, wdirname, wstr_offset_in, wwrite_to_fd};
+    use crate::common::wcs2bytes;
+    use crate::fds::AutoCloseFd;
+    use crate::tests::prelude::*;
+    use crate::util::get_rng;
+    use crate::wchar::prelude::*;
+    use libc::{O_CREAT, O_RDWR, O_TRUNC, SEEK_SET, c_void};
+    use rand::Rng;
+    use std::{ffi::CString, ptr};
+
+    mod test_path_normalize_for_cd {
+        use super::super::path_normalize_for_cd;
+        use crate::wchar::L;
+
+        #[test]
+        fn relative_path() {
+            let wd = L!("/home/user/");
+            let path = L!("projects");
+            eprintf!("(%s, %s)\n", wd, path);
+            assert_eq!(path_normalize_for_cd(wd, path), L!("/home/user/projects"));
+        }
+
+        #[test]
+        fn absolute_path() {
+            let wd = L!("/home/user/");
+            let path = L!("/etc");
+            eprintf!("(%s, %s)\n", wd, path);
+            assert_eq!(path_normalize_for_cd(wd, path), L!("/etc"));
+        }
+
+        #[test]
+        fn parent_directory() {
+            let wd = L!("/home/user/projects/");
+            let path = L!("../docs");
+            eprintf!("(%s, %s)\n", wd, path);
+            assert_eq!(path_normalize_for_cd(wd, path), L!("/home/user/docs"));
+        }
+
+        #[test]
+        fn current_directory() {
+            let wd = L!("/home/user/");
+            let path = L!("./");
+            eprintf!("(%s, %s)\n", wd, path);
+            assert_eq!(path_normalize_for_cd(wd, path), L!("/home/user"));
+        }
+
+        #[test]
+        fn nested_parent_directory() {
+            let wd = L!("/home/user/projects/");
+            let path = L!("../../");
+            eprintf!("(%s, %s)\n", wd, path);
+            assert_eq!(path_normalize_for_cd(wd, path), L!("/home"));
+        }
+
+        #[test]
+        fn complex_path() {
+            let wd = L!("/home/user/projects/");
+            let path = L!("./../other/projects/./.././../docs");
+            eprintf!("(%s, %s)\n", wd, path);
+            assert_eq!(
+                path_normalize_for_cd(wd, path),
+                L!("/home/user/other/projects/./.././../docs")
+            );
+        }
+
+        #[test]
+        fn root_directory() {
+            let wd = L!("/");
+            let path = L!("..");
+            eprintf!("(%s, %s)\n", wd, path);
+            assert_eq!(path_normalize_for_cd(wd, path), L!("/.."));
+        }
+
+        #[test]
+        fn up_to_root_directory() {
+            let wd = L!("/foo/");
+            let path = L!("..");
+            eprintf!("(%s, %s)\n", wd, path);
+            assert_eq!(path_normalize_for_cd(wd, path), L!("/"));
+        }
+
+        #[test]
+        fn empty_path() {
+            let wd = L!("/home/user/");
+            let path = L!("");
+            eprintf!("(%s, %s)\n", wd, path);
+            assert_eq!(path_normalize_for_cd(wd, path), L!("/home/user/"));
+        }
+
+        #[test]
+        fn trailing_slash() {
+            let wd = L!("/home/user/projects/");
+            let path = L!("docs/");
+            eprintf!("(%s, %s)\n", wd, path);
+            assert_eq!(
+                path_normalize_for_cd(wd, path),
+                L!("/home/user/projects/docs/")
+            );
+        }
+    }
+
+    #[test]
+    fn test_normalize_path() {
+        fn norm_path(path: &wstr) -> WString {
+            normalize_path(path, true)
+        }
+        assert_eq!(norm_path(L!("")), ".");
+        assert_eq!(norm_path(L!("..")), "..");
+        assert_eq!(norm_path(L!("./")), ".");
+        assert_eq!(norm_path(L!("./.")), ".");
+        assert_eq!(norm_path(L!("/")), "/");
+        assert_eq!(norm_path(L!("//")), "//");
+        assert_eq!(norm_path(L!("///")), "/");
+        assert_eq!(norm_path(L!("////")), "/");
+        assert_eq!(norm_path(L!("/.///")), "/");
+        assert_eq!(norm_path(L!(".//")), ".");
+        assert_eq!(norm_path(L!("/.//../")), "/");
+        assert_eq!(norm_path(L!("////abc")), "/abc");
+        assert_eq!(norm_path(L!("/abc")), "/abc");
+        assert_eq!(norm_path(L!("/abc/")), "/abc");
+        assert_eq!(norm_path(L!("/abc/..def/")), "/abc/..def");
+        assert_eq!(norm_path(L!("//abc/../def/")), "//def");
+        assert_eq!(norm_path(L!("abc/../abc/../abc/../abc")), "abc");
+        assert_eq!(norm_path(L!("../../")), "../..");
+        assert_eq!(norm_path(L!("foo/./bar")), "foo/bar");
+        assert_eq!(norm_path(L!("foo/../")), ".");
+        assert_eq!(norm_path(L!("foo/../foo")), "foo");
+        assert_eq!(norm_path(L!("foo/../foo/")), "foo");
+        assert_eq!(norm_path(L!("foo/././bar/.././baz")), "foo/baz");
+    }
+
+    #[test]
+    fn test_wdirname_wbasename() {
+        // path, dir, base
+        struct Test(&'static wstr, &'static wstr, &'static wstr);
+        const testcases: &[Test] = &[
+            Test(L!(""), L!("."), L!(".")),
+            Test(L!("foo//"), L!("."), L!("foo")),
+            Test(L!("foo//////"), L!("."), L!("foo")),
+            Test(L!("/////foo"), L!("/"), L!("foo")),
+            Test(L!("//foo/////bar"), L!("//foo"), L!("bar")),
+            Test(L!("foo/////bar"), L!("foo"), L!("bar")),
+            // Examples given in XPG4.2.
+            Test(L!("/usr/lib"), L!("/usr"), L!("lib")),
+            Test(L!("usr"), L!("."), L!("usr")),
+            Test(L!("/"), L!("/"), L!("/")),
+            Test(L!("."), L!("."), L!(".")),
+            Test(L!(".."), L!("."), L!("..")),
+        ];
+
+        for tc in testcases {
+            let Test(path, tc_dir, tc_base) = *tc;
+            let dir = wdirname(path);
+            assert_eq!(
+                dir, tc_dir,
+                "\npath: {:?}, dir: {:?}, tc.dir: {:?}",
+                path, dir, tc_dir
+            );
+
+            let base = wbasename(path);
+            assert_eq!(
+                base, tc_base,
+                "\npath: {:?}, base: {:?}, tc.base: {:?}",
+                path, base, tc_base
+            );
+        }
+
+        // Ensure strings which greatly exceed PATH_MAX still work (#7837).
+        const PATH_MAX: usize = libc::PATH_MAX as usize;
+        let mut longpath = WString::new();
+        longpath.reserve(PATH_MAX * 2 + 10);
+        while longpath.char_count() <= PATH_MAX * 2 {
+            longpath.push_str("/overlong");
+        }
+        let last_slash = longpath.chars().rposition(|c| c == '/').unwrap();
+        let longpath_dir = &longpath[..last_slash];
+        assert_eq!(wdirname(&longpath), longpath_dir);
+        assert_eq!(wbasename(&longpath), L!("overlong"));
+    }
+
+    #[test]
+    #[serial]
+    fn test_wwrite_to_fd() {
+        let _cleanup = test_init();
+        let temp_file = fish_tempfile::new_file().unwrap();
+        let filename = CString::new(temp_file.path().to_str().unwrap()).unwrap();
+        let mut rng = get_rng();
+        let sizes = [1, 2, 3, 5, 13, 23, 64, 128, 255, 4096, 4096 * 2];
+        for &size in &sizes {
+            let fd = AutoCloseFd::new(unsafe {
+                libc::open(filename.as_ptr(), O_RDWR | O_TRUNC | O_CREAT, 0o666)
+            });
+            assert!(fd.is_valid());
+            let mut input = WString::new();
+            for _i in 0..size {
+                input.push(rng.r#gen());
+            }
+
+            let amt = wwrite_to_fd(&input, fd.fd()).unwrap();
+            let narrow = wcs2bytes(&input);
+            assert_eq!(amt, narrow.len());
+
+            assert!(unsafe { libc::lseek(fd.fd(), 0, SEEK_SET) } >= 0);
+
+            let mut contents = vec![0u8; narrow.len()];
+            let read_amt = unsafe {
+                libc::read(
+                    fd.fd(),
+                    if size == 0 {
+                        ptr::null_mut()
+                    } else {
+                        (&mut contents[0]) as *mut u8 as *mut c_void
+                    },
+                    narrow.len(),
+                )
+            };
+            assert!(usize::try_from(read_amt).unwrap() == narrow.len());
+            assert_eq!(&contents, &narrow);
+        }
+    }
+
+    #[test]
+    fn test_wstr_offset_in() {
+        use crate::wchar::L;
+        let base = L!("hello world");
+        assert_eq!(wstr_offset_in(&base[6..], base), 6);
+        assert_eq!(wstr_offset_in(&base[0..], base), 0);
+        assert_eq!(wstr_offset_in(&base[6..], &base[6..]), 0);
+        assert_eq!(wstr_offset_in(&base[base.len()..], base), base.len());
+    }
 }
