@@ -22,6 +22,8 @@ send, sendline, sleep, expect_prompt, expect_re, expect_str = (
 )
 expect_prompt()
 
+sendline("bind ctrl-l repaint")
+expect_prompt()
 # Clear twice (regression test for #7280).
 send("\f")
 expect_prompt(increment=False)
@@ -262,6 +264,12 @@ sleep(0.200)
 send("bgU\r")
 expect_prompt("echo ECHO")
 
+send("echo 125")
+send("\033")
+sleep(0.200)
+send("0$i34\r")
+expect_prompt("echo 12345")
+
 # Now test that exactly the expected bind modes are defined
 sendline("bind --list-modes")
 expect_prompt(
@@ -326,7 +334,7 @@ expect_str(" banana\r")
 expect_prompt("foonanana banana")
 
 # Ensure that nul can be bound properly (#3189).
-send("bind -k nul 'echo nul seen'\r")
+send("bind ctrl-space 'echo nul seen'\r")
 expect_prompt()
 send("\0" * 3)
 # We need to sleep briefly before emitting a newline, because when we execute the
@@ -371,23 +379,24 @@ expect_prompt("\n.*b c d")
 # Check that ctrl-z can be bound
 sendline('bind ctrl-z "echo bound ctrl-z"')
 expect_prompt()
-send("\x1A")
+send("\x1a")
 expect_str("bound ctrl-z")
 
-send('echo foobar')
-send('\x02\x02\x02') # ctrl-b, backward-char
-sendline('\x1bu') # alt+u, upcase word
+send("echo foobar")
+send("\x02\x02\x02")  # ctrl-b, backward-char
+sendline("\x1bu")  # alt+u, upcase word
 expect_prompt("fooBAR")
 
-sendline('bind ctrl-z history-prefix-search-backward')
+sendline("bind ctrl-z history-prefix-search-backward")
 expect_prompt()
 sendline("echo this continues")
 expect_prompt()
-send("\x1A")
+send("\x1a")
 sendline(" with this text")
 expect_prompt("this continues with this text")
 
-sendline("""
+sendline(
+    """
     bind ctrl-g "
         commandline --insert 'echo foo ar'
         commandline -f backward-word
@@ -396,9 +405,11 @@ sendline("""
         commandline -f backward-char
         commandline -f delete-char
     "
-""".strip())
-send('\x07') # ctrl-g
-send('\r')
+""".strip()
+)
+expect_prompt()
+send("\x07")  # ctrl-g
+send("\r")
 expect_prompt("foobar")
 
 # This should do nothing instead of crash
@@ -418,7 +429,7 @@ send("\x1b[27u")  # escape, to close pager
 # (for obvious reasons this MUST BE LAST)
 sendline("function myexit; echo exit; exit; end; bind ctrl-z myexit")
 expect_prompt()
-send("\x1A")
+send("\x1a")
 expect_str("exit")
 
 for t in range(0, 50):
