@@ -15,19 +15,21 @@ expect_prompt()
 # Test --showing-suggestion before we dirty the history
 sendline("echo hello")
 expect_prompt()
-sendline("function debug; commandline --showing-suggestion; set -g cmd_status $status; end")
+sendline(
+    "function debug; commandline --showing-suggestion; set -g cmd_status $status; end"
+)
 expect_prompt()
-sendline("bind ctrl-p debug");
+sendline("bind ctrl-p debug")
 expect_prompt()
 send("echo hell")
-sleep(0.1) # wait for suggestion to appear under CI
+sleep(0.1)  # wait for suggestion to appear under CI
 send(control("p"))
 sendline("")
 expect_prompt("hell")
 sendline("echo cmd_status: $cmd_status")
 expect_prompt("cmd_status: 0")
 send("echo goodb")
-sleep(0.1) # wait for suggestion to appear under CI
+sleep(0.1)  # wait for suggestion to appear under CI
 send(control("p"))
 sendline("")
 expect_prompt("goodb")
@@ -42,7 +44,7 @@ sendline(
     "function handle_tilde; echo; echo '@GUARD:1@'; commandline -b; echo '@/GUARD:1@'; commandline -b ''; end"
 )
 expect_prompt()
-sendline("echo \en one \"two three\" four'five six'{7} 'eight~")
+sendline("echo \\en one \"two three\" four'five six'{7} 'eight~")
 expect_prompt("\r\n@GUARD:1@\r\n(.*)\r\n@/GUARD:1@\r\n")
 
 # printing the buffer with -o should remove quoting
@@ -100,6 +102,18 @@ send(control("k"))
 sendline('echo "process extent is [$tmp]"')
 expect_str("process extent is [echo process # comment]")
 
+sendline(
+    """$fish -C 'commandline "sq 2; exit"; commandline --cursor 1; commandline -i e'"""
+)
+expect_str("seq 2")
+send("\r")
+expect_str("1\r\n2\r\n")
+
+sendline("""$fish -C 'commandline 123; read'""")
+expect_str("read> 123")
+sendline("456; exit")
+expect_str("123456")
+
 # DISABLED because it keeps failing under ASAN
 # sendline(r"bind ctrl-b 'set tmp (commandline --current-process | count)'")
 # sendline(r'commandline "echo line1 \\" "# comment" "line2"')
