@@ -31,7 +31,7 @@ expect_re(
     + "\r\nin command substitution"
     + "\r\nfish: Unable to evaluate string substitution"
     + re.escape("\r\neval (string replace dog tiger -- $history[1])")
-    + "\r\n *\^~+\^\w*"
+    + "\r\n *\\^~+\\^\\w*"
 )
 expect_prompt()
 
