@@ -1,8 +1,5 @@
 #!/usr/bin/env python3
 from pexpect_helper import SpawnedProc
-import os
-import platform
-import sys
 
 sp = SpawnedProc()
 send, sendline, sleep, expect_prompt, expect_re, expect_str = (
@@ -25,20 +22,20 @@ expect_prompt("")
 
 # Validate standalone behavior
 sendline("status current-commandline")
-expect_prompt("\r\nstatus current-commandline\r\n")
+expect_prompt("\r\n.*status current-commandline\r\n")
 
 # Validate behavior as part of a command chain
-sendline("true 7 && status current-commandline");
-expect_prompt("\r\ntrue 7 && status current-commandline\r\n");
+sendline("true 7 && status current-commandline")
+expect_prompt("\r\n.*true 7 && status current-commandline\r\n")
 
 # Validate behavior when used in a function
-sendline("function report; set -g last_cmdline (status current-commandline); end");
+sendline("function report; set -g last_cmdline (status current-commandline); end")
 expect_prompt("")
 sendline("report 27")
 expect_prompt("")
 sendline("echo $last_cmdline")
-expect_prompt("\r\nreport 27\r\n")
+expect_prompt("\r\n.*report 27\r\n")
 
 # Exit
-send("\x04") # <c-d>
+send("\x04")  # <c-d>
 expect_str("")
