@@ -27,3 +27,9 @@ sendline("true ($fish_test_helper abandon_tty)")
 expect_prompt()
 sendline("echo even cooler")
 expect_prompt("even cooler")
+
+sendline("TERM=not-dumb $fish -c read")
+sp.send_primary_device_attribute()
+sendline("something")
+sp.expect_re(r"\x1b\[\?u")
+expect_prompt()
