@@ -86,7 +86,6 @@ def makeenv(script_path: Path, home: Path) -> Dict[str, str]:
             "XDG_CACHE_HOME": str(xdg_cache),
             "fish_test_helper": str(home.parent / "fish_test_helper"),
             "LANG": "C",
-            "LC_CTYPE": "en_US.UTF-8",
         }
     )
 
@@ -159,12 +158,9 @@ async def main():
         files = [(os.path.abspath(path), path) for path in args.file]
     else:
         files = [
-            (os.path.abspath(path), str(path.relative_to(script_path)))
+            (os.path.abspath(path), str(path.relative_to(Path.cwd())))
             for path in sorted(script_path.glob("checks/*.fish"))
-        ]
-        files += [
-            (os.path.abspath(path), str(path.relative_to(script_path)))
-            for path in sorted(script_path.glob("pexpects/*.py"))
+            + sorted(script_path.glob("pexpects/*.py"))
         ]
 
     if not PEXPECT and any(x.endswith(".py") for (x, _) in files):
@@ -292,7 +288,7 @@ async def run_test(
     script_path: Path,
     def_subs,
     lconfig,
-    fishdir,
+    fishdir: Path,
 ) -> TestResult:
     if not test_file_path.endswith(".fish") and not test_file_path.endswith(".py"):
         return TestFail(arg, None, f"Not a valid test file: {arg}")
@@ -300,7 +296,6 @@ async def run_test(
     starttime = datetime.now()
     home = Path(tempfile.mkdtemp(prefix="fishtest-", dir=tmp_root))
     test_env = makeenv(script_path, home)
-    os.chdir(home)
     if test_file_path.endswith(".fish"):
         subs = def_subs.copy()
         subs.update(
@@ -312,7 +307,12 @@ async def run_test(
 
         # littlecheck
         ret = await littlecheck.check_path_async(
-            test_file_path, subs, lconfig, lambda x: print(x.message()), env=test_env
+            test_file_path,
+            subs,
+            lconfig,
+            lambda x: print(x.message()),
+            env=test_env,
+            cwd=home,
         )
         endtime = datetime.now()
         duration_ms = round((endtime - starttime).total_seconds() * 1000)
@@ -342,6 +342,7 @@ async def run_test(
             stdout=PIPE,
             stderr=PIPE,
             env=test_env,
+            cwd=home,
         )
         stdout, stderr = await proc.communicate()
         endtime = datetime.now()
